/*
	Exception/Interrupt Program
*/

#include	"TMPM370.h"
#include	"ipdefine.h"
#include	"D_Para.h"
#include	"D_Driver.h"
#include	"interrupt.h"

/****************************************************************************************
 * Module   : INTEMG0
 *---------------------------------------------------------------------------------------
 * Function : EMG 0 Interruption
 * Input    : none
 * Output   : none
 * Note     : Move at ~EMG port is L
 ****************************************************************************************/
void INTEMG0(){}


/****************************************************************************************
 * Module   : INTVCA
 *---------------------------------------------------------------------------------------
 * Function : Vector Engine A Interruption
 * Input    : none
 * Output   : none
 * Note     : Move at end of input VE (every PWM priod)
 ****************************************************************************************/
void INTVCA(void)
{
extern 	VAR16	M_Main_Counter;		// main loop counter
	M_Main_Counter++;				// Count for main routine Period

	while(VE->SCHTASKRUN&1){}		// Wait end of VE
#if	F_Fixed_Vdc ==1					// 1: Vdc from constant data
	VE->VDC0	= 	cVdc;
#endif

	R_Vd.word = VE->VD0;			// Read Vd from VE
	R_Id = (VAR16)(VE->ID0>>16);	// Read Id from VE
	R_Iq = (VAR16)(VE->IQ0>>16);	// Read Iq from VE

	D_Detect_Rotor_Position();		// Calculate omega by Id,Iq,Vd
	H_Encoder();					// Encoder
	D_Control_Speed();				// Control Speed

	/**** Set data to VE register ********************/
	/**** STOP Stage *******************/
	VE->MCTLF0	=	0x1;
	if(ITR_STAGE==0){				// VE stop
		if(STOP_0DETECT==0){		// STOP stage & before detect current 0
			VE->MODE0 = 0x0;		// Output OFF
			VE->TASKAPP = 0x0;		// VE start from output control
			VE->ACTSCH = 0x9;		// Schedule 9
			VE->OMEGA0 = 0x0;
			VE->ID0 = 0x0;
			VE->IQ0 = 0x0;
			VE->VD0 = 0x0;
			VE->VQ0 = 0x0;
			VE->VDIH0 = 0x0;
			VE->VDILH0 = 0x0;
			VE->VQIH0 = 0x0;
			VE->VQILH0 = 0x0;
			VE->IDREF0 = 0x0;
			VE->IQREF0 = 0x0;
			VE->THETA0 = 0x0;
			R_omega_com.word = 0;
			STOP_0DETECT=1;

		}else if(STOP_0DETECT==1){	// STOP stage & after detect current 0
			VE->MODE0 = 0x2;		// Input 0 current
			VE->TASKAPP = 0x0;		// VE start from output control
			VE->ACTSCH = 0x9;		// Schedule 9
			VE->OMEGA0=0x0;
			VE->ID0 = 0x0;
			VE->IQ0 = 0x0;
			VE->VD0 = 0x0;
			VE->VQ0 = 0x0;
			VE->VDIH0 = 0x0;
			VE->VDILH0 = 0x0;
			VE->VQIH0 = 0x0;
			VE->VQILH0 = 0x0;
			VE->IDREF0 = 0x0;
			VE->IQREF0 = 0x0;
			VE->THETA0 = 0x0;
			STOP_0DETECT = 0;
		}

#if		F_VoltageControl ==1				// VE is not Current Control for debug
	/**** Position Stage ***************/
	}else if(ITR_STAGE==1){				// Position stage
		VE->MODE0 = 0x4; 				// Output ON
		VE->TASKAPP = 0x6;				// VE start from SIN/COS
		VE->ACTSCH = 0x4;				// Schedule 4
		VE->VD0+= cVd;					// (S_Initp_time_user);
		VE->VQ0=0x0;
		VE->THETA0=0;
		VE->OMEGA0=0;
		PMD0->MDPOT = 0xE;				// UP/DOWN act = L/L
		PMD0->MDOUT = 0x73F;			// UP/DOWN = PWM/~PWM
		VE->OUTCR0 = 0x01FF;			// PWM/~PWM

	/**** Force Stage ******************/
	}else if(ITR_STAGE==2){ 			// Force stage
		VE->MODE0 = 0x4;				// Output ON
		VE->TASKAPP = 0x6;				// VE start from SIN/COS
		VE->ACTSCH = 0x4;				// Schedule 4
		VE->VD0 = cVd;					// Fixed Vd
		VE->VQ0 = cVq;					// Fixed Vq
		VE->THETA0 = R_theta.half[1]; 	// VE REGISTER

	/**** ChangeUp Stage ***************/
	}else if(ITR_STAGE==3){ 			// ChangeUp mode
		VE->MODE0 = 0x4; 				// Output ON
		VE->TASKAPP = 0x6; 				// VE start from SIN/COS
		VE->ACTSCH = 0x4; 				// Schedule 4
		VE->VD0 = cVd;					// Fixed Vd
		VE->VQ0 = cVq;					// Fixed Vq
		VE->THETA0 = R_theta.half[1]; 	// Theta set

	/**** Steady Stage *****************/
	}else if(ITR_STAGE==4){ 			// Steady stage
		VE->MODE0 = 0x4;				// Output ON
		VE->TASKAPP = 0x6;				// VE start from SIN/COS
		VE->ACTSCH = 0x4;				// Schedule 4
		VE->VD0 = R_Id_ref*0x10000;		// Id(VAR16) --> Vd(VAR32)
		VE->VQ0 = R_Iq_ref*0x10000;		// Iq(VAR16) --> Vq(VAR32)
		VE->THETA0 = R_theta.half[1];	// Theta set

#else							// Normal (with current control)
	/**** Position Stage ***************/
	}else if(ITR_STAGE==1){				// Position stage
		VE->MODE0 = 0x4; 				// Output ON
		VE->TASKAPP = 0x5;				// VE start from current control
		VE->ACTSCH = 0x1;				// Schedule 1
		VE->IDREF0 = R_Id_com.half[1];	// Set Id
		VE->IQREF0 = 0x0;				// Set Iq=0
		VE->THETA0 = 0;
		VE->OMEGA0 = 0;
		VE->OUTCR0 = 0x01FF;			// PWM/~PWM

	/**** Force Stage ******************/
	}else if(ITR_STAGE==2){ 			// Force stage
		VE->MODE0 = 0x4;				// Output ON
		VE->TASKAPP = 0x5;				// VE start from current control
		VE->ACTSCH = 0x1;				// Schedule 1
		VE->IDREF0 = R_Id_com.half[1];	// Set Id
		VE->IQREF0 = 0x0;				// Set Iq=0
		VE->THETA0 = R_theta.half[1];	 // Theta set

	/**** ChangeUp Stage ***************/
	}else if(ITR_STAGE==3){ 			// ChangeUp mode
		VE->MODE0 = 0x4; 				// Output ON
		VE->TASKAPP = 0x5; 				// VE start from current control
		VE->ACTSCH = 0x1; 				// Schedule 1
		VE->IDREF0 = R_Id_com.half[1];	// Id set
		VE->IQREF0 = R_Iq_com.half[1];	// Iq set
		VE->THETA0 = R_theta.half[1];	// Theta set

	/**** Steady Stage *****************/
	}else if(ITR_STAGE==4){ 			// Steady stage
		VE->MODE0 = 0x4;				// Output ON
		VE->TASKAPP = 0x5;				// VE start from current control
		VE->ACTSCH = 0x1;				// Schedule 1
		VE->IDREF0 = R_Id_ref;			// Id set
		VE->IQREF0 = R_Iq_ref;			// Iq set
		VE->THETA0 = R_theta.half[1];	// Theta set
#endif

	}
	VE->CPURUNTRG = 0x1;				// VE start

	PA->DATA &= 0x7F;					// PA8=L:DAC enable
	R_DebugOut_Counter =0;				// Start SIO for DAC
	Out_data();							// Set SIO Data for DAC

// Save VE data for debug
	temp5  = VE->ID0;
	temp6  = VE->IQ0;
	temp7  = VE->IDREF0;
	temp8  = VE->IQREF0;
	temp9  = VE->MODE0;
	temp10 = VE->TASKAPP;
	temp11 = VE->ACTSCH;
	temp12 = VE->VD0;
	temp13 = cCtrlPrd*1000000;

}

