/*******************************************************************************
* File Name 		 : m370_int.c
* Version			 : V1.0
* Date				 : 2009/06/23
* Description		 : Interrupt liblary source
********************************************************************************/

#include "TMPM370.h"
#include "sys_macro.h"
#include "m370_int.h"


/*******************************************************************************
* Function Name  : API_INT_Init
* Description	 : Initialization of the interruption
* Input 		 : 
* Output		 : 
* Return		 : 
*******************************************************************************/
void API_INT_Init(void)
{
/*----- MCU(M370) INTERRUPT INITIALIZE -----*/
	API_INT_claer_Init();
	CG->RSTFLG = 0x00;
	API_CG_Active_Reset();

/*----- Cortex-M3 NVIC INITIALIZE -----*/
	API_INT_PR_Reset();			/* Reset Interruption level */
	SCB->ICSR = 0x00000000;		/* Interruption control register */

	SCB->VTOR = 0x00000000;		/* Vector table offset register */

	SCB->SHP[0] = 0x00;			/* System handler priority register */
	SCB->SHP[1] = 0x00;
	SCB->SHP[2] = 0x00;
	SCB->SHP[3] = 0x00;
	SCB->SHP[4] = 0x00;
	SCB->SHP[5] = 0x00;
	SCB->SHP[6] = 0x00;
	SCB->SHP[7] = 0x00;
	SCB->SHP[8] = 0x00;
	SCB->SHP[9] = 0x00;
	SCB->SHP[10] = 0x00;
	SCB->SHP[11] = 0x00;

	SCB->SHCSR = 0x00000000;		/* System handler control register */
	SysTick->CTRL= 0x00000000;	/* SysTick register */
}

/*******************************************************************************
* Function Name  : API_INT_clear_Init
* Description	 : Clear all standby release factors
* Input 		 : 
* Output		 : 
* Return		 : 
*******************************************************************************/
void API_INT_claer_Init(void)
{
	CG->ICRCG = IP_INT_CLR_INT0;
	CG->ICRCG = IP_INT_CLR_INT1;
	CG->ICRCG = IP_INT_CLR_INT2;
	CG->ICRCG = IP_INT_CLR_INT3;
	CG->ICRCG = IP_INT_CLR_INT4;
	CG->ICRCG = IP_INT_CLR_INT5;
	CG->ICRCG = IP_INT_CLR_INTCECRX;
	CG->ICRCG = IP_INT_CLR_INTRMCRX0;
	CG->ICRCG = IP_INT_CLR_INTRTC;
	CG->ICRCG = IP_INT_CLR_INT6;
	CG->ICRCG = IP_INT_CLR_INT7;
	CG->ICRCG = IP_INT_CLR_INTRMCRX1;
}


/*******************************************************************************
* Function Name  : API_CG_Active_Set
* Description	 : Set active lebel of CG interrupt mode
* Input 		 : Enable Set 0-15
* Output		 : 
* Return		 : 
*******************************************************************************/
void API_CG_Active_Set(uint8_t ip_intnum, uint8_t active, uint8_t enable)
{
	uint8_t num;
	uint8_t bit;
	__IO uint32_t * ChIMCG;

	num = ip_intnum/4;
	bit = (ip_intnum%4)*8;

	switch(num){
	case 0:
		ChIMCG = &CG->IMCGA;
		break;
	case 1:
		ChIMCG = &CG->IMCGB;
		break;
	case 2:
		ChIMCG = &CG->IMCGC;
		break;
	case 3:
		ChIMCG = &CG->IMCGD;
		break;
	}
	
	*ChIMCG = (__IO uint32_t)((active | enable)<<bit);
}


/*******************************************************************************
* Function Name  : API_CG_Active_Reset
* Description	 : Initialize the active level of CG interruption
* Input 		 : 
* Output		 : 
* Return		 : 
*******************************************************************************/
void API_CG_Active_Reset(void)
{
	CG->IMCGA = 0x20202020;
	CG->IMCGB = 0x20202020;
	CG->IMCGC = 0x20202020;
	CG->IMCGD = 0x20202020;
}


/*******************************************************************************
* Function Name  : API_INT_CER_All_Set
* Description	 : 
* Input 		 : Disable all interruptions
* Output		 : 
* Return		 : 
*******************************************************************************/
void API_INT_CER_All_Set(void)
{
	NVIC->ICER[0] = 0xFF;
	NVIC->ICER[1] = 0xFF;
	NVIC->ICER[2] = 0xFF;
	NVIC->ICER[3] = 0xFF;
	NVIC->ICER[4] = 0xFF;
	NVIC->ICER[5] = 0xFF;
	NVIC->ICER[6] = 0xFF;
	NVIC->ICER[7] = 0xFF;
}



/*******************************************************************************
* Function Name  : API_INT_CER_Set
* Description	 : Disable appointed interruption
* Input 		 : Enable Clear IRQ 0-239
* Output		 : 
* Return		 : 
*******************************************************************************/
void API_INT_CER_Set(uint8_t intnum)
{
	uint8_t num,bit;

	num = intnum/32;
	bit = intnum%32;

	NVIC->ICER[num] = (0x00000001 << bit);
}


/*******************************************************************************
* Function Name  : API_INT_PR_Set
* Description	 : Set level of appointed interruption
* Input 		 : Priority IRQ 0-239
* Output		 : 
* Return		 : 
*******************************************************************************/
void API_INT_PR_Set(uint8_t intnum, uint8_t pri)
{
	NVIC->IP[intnum] = pri;
}


/*******************************************************************************
* Function Name  : API_INT_PR_Reset
* Description	 : Initialize a lebel of interruption
* Input 		 : 
* Output		 : 
* Return		 : 
*******************************************************************************/
void API_INT_PR_Reset(void)
{
	uint8_t i;
	for (i=0; i<240; i++){
		NVIC->IP[i] = 0x00;
	}
}

/*******************************************************************************
* Function Name  : API_INT_SPR_ALL_Set
* Description	 : All clear the waiting interruptions
* Input 		 : 
* Output		 : 
* Return		 : 
*******************************************************************************/
void API_INT_SPR_ALL_Set(void)
{
	NVIC->ISPR[0] = 0xFF;
	NVIC->ISPR[1] = 0xFF;
	NVIC->ISPR[2] = 0xFF;
	NVIC->ISPR[3] = 0xFF;
	NVIC->ISPR[4] = 0xFF;
	NVIC->ISPR[5] = 0xFF;
	NVIC->ISPR[6] = 0xFF;
	NVIC->ISPR[7] = 0xFF;
}


/*******************************************************************************
* Function Name  : API_INT_SPR_Set
* Description	 : Clear the reserved request of interruption
* Input 		 : Clear Pending IRQ 0-239
* Output		 : 
* Return		 : 
*******************************************************************************/
void API_INT_SPR_Set(uint8_t intnum)
{
	uint8_t num,bit;

	num = intnum/32;
	bit = intnum%32;
	NVIC->ICPR[num] = (0x00000001 << bit);
}


/*******************************************************************************
* Function Name  : API_INT_SER_Set
* Description	 : Enable the interruption
* Input 		 : Enable Set IRQ 0-239
* Output		 : 
* Return		 : 
*******************************************************************************/
void API_INT_SER_Set(uint8_t intnum)
{
	uint8_t num,bit;

	num = intnum/32;
	bit = intnum%32;
	NVIC->ISER[num] = 0x00000001 << bit;
}

/*********************************** END OF FILE ******************************/
