/*******************************************************************************
* File Name 		 : syys_macro.h
* Version			 : V1.0
* Date				 : 2009/06/23
* Description		 : File of defined macro by user
********************************************************************************/
#ifndef	_SYS_MACRO_H_
#define	_SYS_MACRO_H_



#define	TRUE		1			/* TRUE */
#define	FALSE		0			/* FALSE */
#define	SET			1			/* Bit set */
#define	CLEAR		0			/* Bit clear */


typedef int8_t					BOOL;		/* TRUE or FALSE */

typedef uint8_t					bit8_f;		/* Bit field ( 8bit)      */
typedef uint16_t				bit16_f;	/* Bit field (16bit)      */


typedef struct {
	bit8_f b0:1;
	bit8_f b1:1;
	bit8_f b2:1;
	bit8_f b3:1;
	bit8_f b4:1;
	bit8_f b5:1;
	bit8_f b6:1;
	bit8_f b7:1;
} BYTE_FIELD;

typedef struct {
	bit16_f b0 :1;
	bit16_f b1 :1;
	bit16_f b2 :1;
	bit16_f b3 :1;
	bit16_f b4 :1;
	bit16_f b5 :1;
	bit16_f b6 :1;
	bit16_f b7 :1;
	bit16_f b8 :1;
	bit16_f b9 :1;
	bit16_f b10:1;
	bit16_f b11:1;
	bit16_f b12:1;
	bit16_f b13:1;
	bit16_f b14:1;
	bit16_f b15:1;
} WORD_FIELD;

/*=======  BYTE ========*/
typedef union {
	uint8_t		 	byte;
	BYTE_FIELD 		bit;
} TYPE_BYTE;

/*======= WORD ========*/
typedef union {
	uint16_t		word;
	uint8_t			byte[2];
	WORD_FIELD 		bit;
} TYPE_WORD;

typedef enum { DISABLE = 0, ENABLE =1 } IpFunctionState;


/*---------------------------------------------------------------------------*/
/* rbg`}N */
#define	BIT_LEN_1	0x01	/*!< BitLength = 1 */
#define	BIT_LEN_2	0x03	/*!< BitLength = 2 */
#define	BIT_LEN_3	0x07	/*!< BitLength = 3 */
#define	BIT_LEN_4	0x0f	/*!< BitLength = 4 */
#define	BIT_LEN_5	0x1f	/*!< BitLength = 5 */
#define	BIT_LEN_6	0x3f	/*!< BitLength = 6 */
#define	BIT_LEN_7	0x7f	/*!< BitLength = 7 */
#define	BIT_LEN_8	0xff	/*!< BitLength = 8 */

/*---------------------------------------------------------------------------*/
/* rbgʒuw}N */
#define	BIT_POS_0	0x00	/*!< BitPosition = LSB */
#define	BIT_POS_1	0x01	/*!< BitPosition =   1 */
#define	BIT_POS_2	0x02	/*!< BitPosition =   2 */
#define	BIT_POS_3	0x03	/*!< BitPosition =   3 */
#define	BIT_POS_4	0x04	/*!< BitPosition =   4 */
#define	BIT_POS_5	0x05	/*!< BitPosition =   5 */
#define	BIT_POS_6	0x06	/*!< BitPosition =   6 */
#define	BIT_POS_7	0x07	/*!< BitPosition = MSB */


/* Macro of bit calucuration */
/* Define the bit : 4bit     */
#define		B_0000	0x00
#define		B_0001	0x01
#define		B_0010	0x02
#define		B_0011	0x03
#define		B_0100	0x04
#define		B_0101	0x05
#define		B_0110	0x06
#define		B_0111	0x07
#define		B_1000	0x08
#define		B_1001	0x09
#define		B_1010	0x0a
#define		B_1011	0x0b
#define		B_1100	0x0c
#define		B_1101	0x0d
#define		B_1110	0x0e
#define		B_1111	0x0f

/* Macro of 16bit Converter */
#define		BIT2HEX8(  UP4 ,LO4 )				(( UP4 << 4 ) | LO4 )
#define		BIT2HEX16( B0 ,B1 ,B2 ,B3 )			((BIT2HEX8( B0,B1 )<<8) | BIT2HEX8( B2,B3 ))
#define		BIT2HEX32( B0 ,B1 ,B2 ,B3 ,B4 ,B5 ,B6 ,B7 )	((BIT2HEX16( B0,B1,B2,B3 ) << 16) | BIT2HEX16( B4,B5,B6,B7 ))

/*! BIT_SET BIT_CLR Machro */
/*  Sample
	BIT_SET_8( &data ,B_10000000 );
	BIT_SET_16( &data ,B_00000000 ,B_00000001 );
	BIT_SET_32( &data ,B_00000000 ,B_00000000 ,B_00000000 ,B_00000001 );
	BIT_CLR_8( &data ,B_10000000 );
	BIT_CLR_16( &data ,B_00000000 ,B_00000001 );
	BIT_CLR_32( &data ,B_00000000 ,B_00000000 ,B_00000000 ,B_00000001 );
*/
#define	B_00000000	0x00
#define	B_00000001	0x01
#define	B_00000010	0x02
#define	B_00000100	0x04
#define	B_00001000	0x08
#define	B_00010000	0x10
#define	B_00100000	0x20
#define	B_01000000	0x40
#define	B_10000000	0x80

#define	BIT_SET_8(  DATA ,PAT )							*(DATA) |= (PAT)
#define	BIT_SET_16( DATA ,PATH ,PATL )					*(DATA) |= ( ((PATH)<<8)+(PATL) )
#define	BIT_SET_32( DATA ,PATHH ,PATH ,PATL ,PATLL )	*(DATA) |= ( ((PATHH)<<24)+((PATH)<<16)+((PATL)<<8) +(PATLL) )
#define	BIT_CLR_8(  DATA ,PAT )							*(DATA) &= ~( (PAT) )
#define	BIT_CLR_16( DATA ,PATH ,PATL )					*(DATA) &= ~( ((PATH)<<8)+(PATL) )
#define	BIT_CLR_32( DATA ,PATHH ,PATH ,PATL ,PATLL )	*(DATA) &= ~( ((PATHH)<<24)+((PATH)<<16)+((PATL)<<8) +(PATLL) )



#define	BIT(a,b,c,d,e,f,g,h)	(a*0x80+b*0x40+c*0x20+d*0x10+e*0x08+f*0x04+g*0x02+h)


#define SRAM_BASE			0x20000000
#define BITBAND_SRAM_BASE	0x22000000
#define BITBAND_SRAM(a,b)	*((volatile uint32_t *)(BITBAND_SRAM_BASE + ((uint32_t)(&(a))-SRAM_BASE)*32 + (b*4)))

#define IP_BASE				0x40000000
#define BITBAND_IP_BASE		0x42000000
#define BITBAND_IP(a,b)		*((volatile uint8_t *)(BITBAND_IP_BASE + ((uint32_t)(&(a))-IP_BASE)*32 + (b*4)))

/*
#define BIT0		0
#define BIT1		1
#define BIT2		2
#define BIT3		3
#define BIT4		4
#define BIT5		5
#define BIT6		6
#define BIT7		7
#define BIT8		8
#define BIT9		9
#define BIT10		10
#define BIT11		11
#define BIT12		12
#define BIT13		13
#define BIT14		14
#define BIT15		15
#define BIT16		16
#define BIT17		17
#define BIT18		18
#define BIT19		19
#define BIT20		20
#define BIT21		21
#define BIT22		22
#define BIT23		23
#define BIT24		24
#define BIT25		25
#define BIT26		26
#define BIT27		27
#define BIT28		28
#define BIT29		29
#define BIT30		30
#define BIT31		31
*/

#endif	/*_SYS_MACRO_H_ */

/*************************** END OF FILE **************************************/
