/******************************************************************************
 * @file:    TMPM370.h
 * @purpose: CMSIS Cortex-M3 Core Peripheral Access Layer Header File for the 
 *           TOSHIBA 'TMPM370' Device Series 
 * @version: V2.0.2.1
 * @date:    2009/12/28
 * 
 * THE SOURCE CODE AND ITS RELATED DOCUMENTATION IS PROVIDED "AS IS". TOSHIBA
 * CORPORATION MAKES NO OTHER WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR,
 * STATUTORY AND DISCLAIMS ANY AND ALL IMPLIED WARRANTIES OF MERCHANTABILITY,
 * SATISFACTORY QUALITY, NON INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * THE SOURCE CODE AND DOCUMENTATION MAY INCLUDE ERRORS. TOSHIBA CORPORATION
 * RESERVES THE RIGHT TO INCORPORATE MODIFICATIONS TO THE SOURCE CODE IN LATER
 * REVISIONS OF IT, AND TO MAKE IMPROVEMENTS OR CHANGES IN THE DOCUMENTATION OR
 * THE PRODUCTS OR TECHNOLOGIES DESCRIBED THEREIN AT ANY TIME.
 * 
 * TOSHIBA CORPORATION SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT OR
 * CONSEQUENTIAL DAMAGE OR LIABILITY ARISING FROM YOUR USE OF THE SOURCE CODE OR
 * ANY DOCUMENTATION, INCLUDING BUT NOT LIMITED TO, LOST REVENUES, DATA OR
 * PROFITS, DAMAGES OF ANY SPECIAL, INCIDENTAL OR CONSEQUENTIAL NATURE, PUNITIVE
 * DAMAGES, LOSS OF PROPERTY OR LOSS OF PROFITS ARISING OUT OF OR IN CONNECTION
 * WITH THIS AGREEMENT, OR BEING UNUSABLE, EVEN IF ADVISED OF THE POSSIBILITY OR
 * PROBABILITY OF SUCH DAMAGES AND WHETHER A CLAIM FOR SUCH DAMAGE IS BASED UPON
 * WARRANTY, CONTRACT, TORT, NEGLIGENCE OR OTHERWISE.
 * 
 * (C)Copyright TOSHIBA CORPORATION 2009 All rights reserved
 ******************************************************************************/

/** @addtogroup TOSHIBA_TX03_MICROCONTROLLER
  * @{
  */

/** @addtogroup TMPM370
  * @{
  */

#ifndef __TMPM370_H__
#define __TMPM370_H__

#ifdef __cplusplus
extern "C" {
#endif 

/** @addtogroup Configuration_of_CMSIS
  * @{
  */

/** Interrupt Number Definition */
typedef enum IRQn
{
/******  Cortex-M3 Processor Exceptions Numbers ****************************************************/
  NonMaskableInt_IRQn           = -14,      /*!< 2 Non Maskable Interrupt                          */
  HardFault_IRQn                = -13,      /*!< 3 Cortex-M3 Hard Fault Interrupt                  */
  MemoryManagement_IRQn         = -12,      /*!< 4 Cortex-M3 Memory Management Interrupt           */
  BusFault_IRQn                 = -11,      /*!< 5 Cortex-M3 Bus Fault Interrupt                   */
  UsageFault_IRQn               = -10,      /*!< 6 Cortex-M3 Usage Fault Interrupt                 */
  SVCall_IRQn                   = -5,       /*!< 11 Cortex-M3 SV Call Interrupt                    */
  DebugMonitor_IRQn             = -4,       /*!< 12 Cortex-M3 Debug Monitor Interrupt              */
  PendSV_IRQn                   = -2,       /*!< 14 Cortex-M3 Pend SV Interrupt                    */
  SysTick_IRQn                  = -1,       /*!< 15 Cortex-M3 System Tick Interrupt                */

/******  TMPM370 Specific Interrupt Numbers ********************************************************/
  INT0_IRQn                     = 0,        /*!< Interrupt pin (PH0/96 pin(-FYFG)/98 pin(-FYDFG))  */
  INT1_IRQn                     = 1,        /*!< Interrupt pin (PH1/95 pin(-FYFG)/97 pin(-FYDFG))  */
  INT2_IRQn                     = 2,        /*!< Interrupt pin (PH2/94 pin(-FYFG)/96 pin(-FYDFG))  */
  INT3_IRQn                     = 3,        /*!< Interrupt pin (PA0/ 2 pin(-FYFG)/ 4 pin(-FYDFG))  */
  INT4_IRQn                     = 4,        /*!< Interrupt pin (PA2/ 4 pin(-FYFG)/ 6 pin(-FYDFG))  */
  INT5_IRQn                     = 5,        /*!< Interrupt pin (PE4/15 pin(-FYFG)/17 pin(-FYDFG))  */
  INTRX0_IRQn                   = 6,        /*!< Serial reception (channel.0)                      */
  INTTX0_IRQn                   = 7,        /*!< Serial transmission (channel.0)                   */
  INTRX1_IRQn                   = 8,        /*!< Serial reception (channel.1)                      */
  INTTX1_IRQn                   = 9,        /*!< Serial transmission (channel.1)                   */
  INTVCNA_IRQn                  = 10,       /*!< Vector Engine interrupt A                         */
  INTVCNB_IRQn                  = 11,       /*!< Vector Engine interrupt B                         */
  INTEMG0_IRQn                  = 12,       /*!< PMD0 EMG interrupt                                */
  INTEMG1_IRQn                  = 13,       /*!< PMD1 EMG interrupt                                */
  INTOVV0_IRQn                  = 14,       /*!< PMD0 OVV interrupt                                */
  INTOVV1_IRQn                  = 15,       /*!< PMD1 OVV interrupt                                */
  INTADAPDA_IRQn                = 16,       /*!< ADC A conversion triggered by PMD0 is finished    */
  INTADBPDA_IRQn                = 17,       /*!< ADC B conversion triggered by PMD0 is finished    */
  INTADAPDB_IRQn                = 18,       /*!< ADC A conversion triggered by PMD1 is finished    */
  INTADBPDB_IRQn                = 19,       /*!< ADC B conversion triggered by PMD1 is finished    */
  INTTB00_IRQn                  = 20,       /*!< 16bit TMRB0 compare match detection 0/ Over flow  */
  INTTB01_IRQn                  = 21,       /*!< 16bit TMRB0 compare match detection 1             */
  INTTB10_IRQn                  = 22,       /*!< 16bit TMRB1 compare match detection 0/ Over flow  */
  INTTB11_IRQn                  = 23,       /*!< 16bit TMRB1 compare match detection 1             */
  INTTB40_IRQn                  = 24,       /*!< 16bit TMRB4 compare match detection 0/ Over flow  */
  INTTB41_IRQn                  = 25,       /*!< 16bit TMRB4 compare match detection 1             */
  INTTB50_IRQn                  = 26,       /*!< 16bit TMRB5 compare match detection 0/ Over flow  */
  INTTB51_IRQn                  = 27,       /*!< 16bit TMRB5 compare match detection 1             */
  INTPMD0_IRQn                  = 28,       /*!< PMD0 PWM interrupt                                */
  INTPMD1_IRQn                  = 29,       /*!< PMD1 PWM interrupt                                */
  INTCAP00_IRQn                 = 30,       /*!< 16bit TMRB0 input capture 0                       */
  INTCAP01_IRQn                 = 31,       /*!< 16bit TMRB0 input capture 1                       */
  INTCAP10_IRQn                 = 32,       /*!< 16bit TMRB1 input capture 0                       */
  INTCAP11_IRQn                 = 33,       /*!< 16bit TMRB1 input capture 1                       */
  INTCAP40_IRQn                 = 34,       /*!< 16bit TMRB4 input capture 0                       */
  INTCAP41_IRQn                 = 35,       /*!< 16bit TMRB4 input capture 1                       */
  INTCAP50_IRQn                 = 36,       /*!< 16bit TMRB5 input capture 0                       */
  INTCAP51_IRQn                 = 37,       /*!< 16bit TMRB5 input capture 1                       */
  INT6_IRQn                     = 38,       /*!< Interrupt pin (PE6/17 pin(-FYFG)/19 pin(-FYDFG))  */
  INT7_IRQn                     = 39,       /*!< Interrupt pin (PE7/18 pin(-FYFG)/20 pin(-FYDFG))  */
  INTRX2_IRQn                   = 40,       /*!< Serial reception (channel.2)                      */
  INTTX2_IRQn                   = 41,       /*!< Serial transmission (channel.2)                   */
  INTAD0CPA_IRQn                = 42,       /*!< AD0 conversion monitoring function interrupt A    */
  INTAD1CPA_IRQn                = 43,       /*!< AD1 conversion monitoring function interrupt A    */
  INTAD0CPB_IRQn                = 44,       /*!< AD0 conversion monitoring function interrupt B    */
  INTAD1CPB_IRQn                = 45,       /*!< AD1 conversion monitoring function interrupt B    */
  INTTB20_IRQn                  = 46,       /*!< 16bit TMRB2 compare match detection 0/ Over flow  */
  INTTB21_IRQn                  = 47,       /*!< 16bit TMRB2 compare match detection 1             */
  INTTB30_IRQn                  = 48,       /*!< 16bit TMRB3 compare match detection 0/ Over flow  */
  INTTB31_IRQn                  = 49,       /*!< 16bit TMRB3 compare match detection 1             */
  INTCAP20_IRQn                 = 50,       /*!< 16bit TMRB2 input capture 0                       */
  INTCAP21_IRQn                 = 51,       /*!< 16bit TMRB2 input capture 1                       */
  INTCAP30_IRQn                 = 52,       /*!< 16bit TMRB3 input capture 0                       */
  INTCAP31_IRQn                 = 53,       /*!< 16bit TMRB3 input capture 1                       */
  INTAD0SFT_IRQn                = 54,       /*!< ADC0 conversion started by software is finished   */
  INTAD1SFT_IRQn                = 55,       /*!< ADC1 conversion started by software is finished   */
  INTAD0TMR_IRQn                = 56,       /*!< ADC0 conversion triggered by timer is finished    */
  INTAD1TMR_IRQn                = 57,       /*!< ADC1 conversion triggered by timer is finished    */
  INT8_IRQn                     = 58,       /*!< Interrupt pin (PA7/ 9 pin(-FYFG)/11 pin(-FYDFG))  */
  INT9_IRQn                     = 59,       /*!< Interrupt pin (PD3/33 pin(-FYFG)/35 pin(-FYDFG))  */
  INTA_IRQn                     = 60,       /*!< Interrupt pin (PL1/21 pin(-FYFG)/23 pin(-FYDFG))  */
  INTB_IRQn                     = 61,       /*!< Interrupt pin (PL0/20 pin(-FYFG)/22 pin(-FYDFG))  */
  INTENC0_IRQn                  = 62,       /*!< Ender input0 interrupt                            */
  INTENC1_IRQn                  = 63,       /*!< Ender input1 interrupt                            */
  INTRX3_IRQn                   = 64,       /*!< Serial reception (channel.3)                      */
  INTTX3_IRQn                   = 65,       /*!< Serial transmission (channel.3)                   */
  INTTB60_IRQn                  = 66,       /*!< 16bit TMRB6 compare match detection 0 / Over flow */
  INTTB61_IRQn                  = 67,       /*!< 16bit TMRB6 compare match detection 1             */
  INTTB70_IRQn                  = 68,       /*!< 16bit TMRB7 compare match detection 0 / Over flow */
  INTTB71_IRQn                  = 69,       /*!< 16bit TMRB7 compare match detection 1             */
  INTCAP60_IRQn                 = 70,       /*!< 16bit TMRB6 input capture 0                       */
  INTCAP61_IRQn                 = 71,       /*!< 16bit TMRB6 input capture 1                       */
  INTCAP70_IRQn                 = 72,       /*!< 16bit TMRB7 input capture 0                       */
  INTCAP71_IRQn                 = 73,       /*!< 16bit TMRB7 input capture 1                       */
  INTC_IRQn                     = 74,       /*!< Interrupt pin (PJ6/74 pin(-FYFG)/76 pin(-FYDFG))  */
  INTD_IRQn                     = 75,       /*!< Interrupt pin (PJ7/73 pin(-FYFG)/75 pin(-FYDFG))  */
  INTE_IRQn                     = 76,       /*!< Interrupt pin (PK0/72 pin(-FYFG)/74 pin(-FYDFG))  */
  INTF_IRQn                     = 77        /*!< Interrupt pin (PK1/71 pin(-FYFG)/73 pin(-FYDFG))  */
} IRQn_Type;


/** Processor and Core Peripheral Section */

/* Configuration of the Cortex-M3 Processor and Core Peripherals */
#define __MPU_PRESENT             0         /*!< MPU present or not                                */
#define __NVIC_PRIO_BITS          3         /*!< Number of Bits used for Priority Levels           */
#define __Vendor_SysTickConfig    0         /*!< Set to 1 if different SysTick Config is used      */

/** @} */ /* End of group Configuration_of_CMSIS */

#include "core_cm3.h"                       /* Cortex-M3 processor and core peripherals            */
#include "system_TMPM370.h"                 /* TMPM370 System                                      */

/** @addtogroup Device_Peripheral_registers
  * @{
  */

/** Device Specific Peripheral registers structures */

/**
  * @brief Clock Generator (CG)
  */
typedef struct
{
  __IO uint32_t SYSCR;          /*!< System Control Register              */
  __IO uint32_t OSCCR;          /*!< Oscillation Control Register         */
  __IO uint32_t STBYCR;         /*!< Standby Control Register             */
  __IO uint32_t PLLSEL;         /*!< PLL Selection Register               */
  __IO uint32_t CKSEL;          /*!< System Clock Selection Register      */
  __IO uint32_t ICRCG;          /*!< CG Interrupt Request Clear Register  */
  __I  uint32_t NMIFLG;         /*!< NMI Flag Register                    */
  __IO uint32_t RSTFLG;         /*!< Reset Flag Register                  */
  __IO uint32_t IMCGA;          /*!< CG Interrupt Mode Control Register A */
  __IO uint32_t IMCGB;          /*!< CG Interrupt Mode Control Register B */
  __IO uint32_t IMCGC;          /*!< CG Interrupt Mode Control Register C */
  __IO uint32_t IMCGD;          /*!< CG Interrupt Mode Control Register D */
} CG_TypeDef;

/**
  * @brief General Purpose Input/Output Port (PA)
  */
typedef struct
{
  __IO uint32_t DATA;           /*!< PA Register                      */
  __IO uint32_t CR;             /*!< PA Control Register              */
  __IO uint32_t FR1;            /*!< PA Function Register 1           */
  __IO uint32_t FR2;            /*!< PA Function Register 2           */
       uint32_t RESERVED0[6];
  __IO uint32_t OD;             /*!< PA Open-Drain Control Register   */
  __IO uint32_t PUP;            /*!< PA Pull-Up Control Register      */
  __IO uint32_t PDN;            /*!< PA Pull-Down Control Register    */
       uint32_t RESERVED1;
  __IO uint32_t IE;             /*!< PA Input Enable Control Register */
} PA_TypeDef;

/**
  * @brief General Purpose Input/Output Port (PB)
  */
typedef struct
{
  __IO uint32_t DATA;           /*!< PB Register                      */
  __IO uint32_t CR;             /*!< PB Control Register              */
  __IO uint32_t FR1;            /*!< PB Function Register 1           */
       uint32_t RESERVED0[8];
  __IO uint32_t PUP;            /*!< PB Pull-Up Control Register      */
  __IO uint32_t PDN;            /*!< PB Pull-Down Control Register    */
       uint32_t RESERVED1;
  __IO uint32_t IE;             /*!< PB Input Enable Control Register */
} PB_TypeDef;

/**
  * @brief General Purpose Input/Output Port (PC)
  */
typedef struct
{
  __IO uint32_t DATA;           /*!< PC Register                      */
  __IO uint32_t CR;             /*!< PC Control Register              */
  __IO uint32_t FR1;            /*!< PC Function Register 1           */
       uint32_t RESERVED0[7];
  __IO uint32_t OD;             /*!< PC Open-Drain Control Register   */
  __IO uint32_t PUP;            /*!< PC Pull-Up Control Register      */
  __IO uint32_t PDN;            /*!< PC Pull-Down Control Register    */
       uint32_t RESERVED1;
  __IO uint32_t IE;             /*!< PC Input Enable Control Register */
} PC_TypeDef;

/**
  * @brief General Purpose Input/Output Port (PD)
  */
typedef struct
{
  __IO uint32_t DATA;           /*!< PD Register                      */
  __IO uint32_t CR;             /*!< PD Control Register              */
  __IO uint32_t FR1;            /*!< PD Function Register 1           */
  __IO uint32_t FR2;            /*!< PD Function Register 2           */
       uint32_t RESERVED0[6];
  __IO uint32_t OD;             /*!< PD Open-Drain Control Register   */
  __IO uint32_t PUP;            /*!< PD Pull-Up Control Register      */
  __IO uint32_t PDN;            /*!< PD Pull-Down Control Register    */
       uint32_t RESERVED1;
  __IO uint32_t IE;             /*!< PD Input Enable Control Register */
} PD_TypeDef;

/**
  * @brief General Purpose Input/Output Port (PE)
  */
typedef struct
{
  __IO uint32_t DATA;           /*!< PE Register                      */
  __IO uint32_t CR;             /*!< PE Control Register              */
  __IO uint32_t FR1;            /*!< PE Function Register 1           */
  __IO uint32_t FR2;            /*!< PE Function Register 2           */
       uint32_t RESERVED0[6];
  __IO uint32_t OD;             /*!< PE Open-Drain Control Register   */
  __IO uint32_t PUP;            /*!< PE Pull-Up Control Register      */
  __IO uint32_t PDN;            /*!< PE Pull-Down Control Register    */
       uint32_t RESERVED1;
  __IO uint32_t IE;             /*!< PE Input Enable Control Register */
} PE_TypeDef;

/**
  * @brief General Purpose Input/Output Port (PF)
  */
typedef struct
{
  __IO uint32_t DATA;           /*!< PF Register                      */
  __IO uint32_t CR;             /*!< PF Control Register              */
  __IO uint32_t FR1;            /*!< PF Function Register 1           */
  __IO uint32_t FR2;            /*!< PF Function Register 2           */
  __IO uint32_t FR3;            /*!< PF Function Register 3           */
       uint32_t RESERVED0[5];
  __IO uint32_t OD;             /*!< PF Open Drain Control Register   */
  __IO uint32_t PUP;            /*!< PF Pull-Up Control Register      */
  __IO uint32_t PDN;            /*!< PF Pull-Down Control Register    */
       uint32_t RESERVED1;
  __IO uint32_t IE;             /*!< PF Input Enable Control Register */
} PF_TypeDef;

/**
  * @brief General Purpose Input/Output Port (PG)
  */
typedef struct
{
  __IO uint32_t DATA;           /*!< PG Register                      */
  __IO uint32_t CR;             /*!< PG Control Register              */
  __IO uint32_t FR1;            /*!< PG Function Register 1           */
       uint32_t RESERVED0[7];
  __IO uint32_t OD;             /*!< PG Open Drain Control Register   */
  __IO uint32_t PUP;            /*!< PG Pull-Up Control Register      */
  __IO uint32_t PDN;            /*!< PG Pull-Down Control Register    */
       uint32_t RESERVED1;
  __IO uint32_t IE;             /*!< PG Input Enable Control Register */
} PG_TypeDef;

/**
  * @brief General Purpose Input/Output Port (PH)
  */
typedef struct
{
  __IO uint32_t DATA;           /*!< PH Register                      */
  __IO uint32_t CR;             /*!< PH Control Register              */
  __IO uint32_t FR1;            /*!< PH Function Register 1           */
       uint32_t RESERVED0[7];
  __IO uint32_t OD;             /*!< PH Open Drain Control Register   */
  __IO uint32_t PUP;            /*!< PH Pull-Up Control Register      */
  __IO uint32_t PDN;            /*!< PH Pull-Down Control Register    */
       uint32_t RESERVED1;
  __IO uint32_t IE;             /*!< PH Input Enable Control Register */
} PH_TypeDef;

/**
  * @brief General Purpose Input/Output Port (PI)
  */
typedef struct
{
  __IO uint32_t DATA;           /*!< PI Register                      */
  __IO uint32_t CR;             /*!< PI Control Register              */
       uint32_t RESERVED0[8];
  __IO uint32_t OD;             /*!< PI Open Drain Control Register   */
  __IO uint32_t PUP;            /*!< PI Pull-Up Control Register      */
  __IO uint32_t PDN;            /*!< PI Pull-Down Control Register    */
       uint32_t RESERVED1;
  __IO uint32_t IE;             /*!< PI Input Enable Control Register */
} PI_TypeDef;

/**
  * @brief General Purpose Input/Output Port (PJ)
  */
typedef struct
{
  __IO uint32_t DATA;           /*!< PJ Register                      */
  __IO uint32_t CR;             /*!< PJ Control Register              */
  __IO uint32_t FR1;            /*!< PJ Function Register 1           */
       uint32_t RESERVED0[7];
  __IO uint32_t OD;             /*!< PJ Open Drain Control Register   */
  __IO uint32_t PUP;            /*!< PJ Pull-Up Control Register      */
  __IO uint32_t PDN;            /*!< PJ Pull-Down Control Register    */
       uint32_t RESERVED1;
  __IO uint32_t IE;             /*!< PJ Input Enable Control Register */
} PJ_TypeDef;

/**
  * @brief General Purpose Input/Output Port (PK)
  */
typedef struct
{
  __IO uint32_t DATA;           /*!< PK Register                      */
  __IO uint32_t CR;             /*!< PK Control Register              */
  __IO uint32_t FR1;            /*!< PK Function Register 1           */
       uint32_t RESERVED0[7];
  __IO uint32_t OD;             /*!< PK Open Drain Control Register   */
  __IO uint32_t PUP;            /*!< PK Pull-Up Control Register      */
  __IO uint32_t PDN;            /*!< PK Pull-Down Control Register    */
       uint32_t RESERVED1;
  __IO uint32_t IE;             /*!< PK Input Enable Control Register */
} PK_TypeDef;

/**
  * @brief General Purpose Input/Output Port (PL)
  */
typedef struct
{
  __IO uint32_t DATA;           /*!< PL Register                      */
       uint32_t RESERVED0;
  __IO uint32_t FR1;            /*!< PL Function Register 1           */
       uint32_t RESERVED1[11];
  __IO uint32_t IE;             /*!< PL Input Enable Control Register */
} PL_TypeDef;

/**
  * @brief 16-bit Timer/Event Counter (TB)
  */
typedef struct
{
  __IO uint32_t EN;             /*!< TB Enable Register            */
  __IO uint32_t RUN;            /*!< TB RUN Register               */
  __IO uint32_t CR;             /*!< TB Control Register           */
  __IO uint32_t MOD;            /*!< TB Mode Register              */
  __IO uint32_t FFCR;           /*!< TB Flip-Flop Control Register */
  __I  uint32_t ST;             /*!< TB Status Register            */
  __IO uint32_t IM;             /*!< TB Interrupt Mask Register    */
  __I  uint32_t UC;             /*!< TB Read Capture Register      */
  __IO uint32_t RG0;            /*!< TB RG0 Timer Register         */
  __IO uint32_t RG1;            /*!< TB RG1 Timer Register         */
  __I  uint32_t CP0;            /*!< TB CP0 Capture Register       */
  __I  uint32_t CP1;            /*!< TB CP1 Capture Register       */
} TB_TypeDef;

/**
  * @brief Encoder input (ENC)
  */
typedef struct
{
  __IO uint32_t TNCR;             /*!< ENC Input Control Register  */
  __IO uint32_t RELOAD;           /*!< ENC Counter Reload Register */
  __IO uint32_t INT;              /*!< ENC Compare Register        */
  __IO uint32_t CNT;              /*!< ENC Counter Register        */
} EN_TypeDef;

/**
  * @brief Serial Channel (SC)
  */
typedef struct
{
  __IO uint32_t EN;             /*!< SC Enable Register                        */
  __IO uint32_t BUF;            /*!< SC Buffer Register                        */
  __IO uint32_t CR;             /*!< SC Control Register                       */
  __IO uint32_t MOD0;           /*!< SC Mode Control Register 0                */
  __IO uint32_t BRCR;           /*!< SC Baud Rate Generator Control Register   */
  __IO uint32_t BRADD;          /*!< SC Baud Rate Generator Control Register 2 */
  __IO uint32_t MOD1;           /*!< SC Mode Control Register 1                */
  __IO uint32_t MOD2;           /*!< SC Mode Control Register 2                */
  __IO uint32_t RFC;            /*!< SC RX FIFO Configuration Register         */
  __IO uint32_t TFC;            /*!< SC TX FIFO Configuration Register         */
  __I  uint32_t RST;            /*!< SC RX FIFO Status Register                */
  __I  uint32_t TST;            /*!< SC TX FIFO Status Register                */
  __IO uint32_t FCNF;           /*!< SC FIFO Configuration Register            */
} SC_TypeDef;

/**
  * @brief Analog-to-Digital Converter (AD)
  */
typedef struct
{
  __IO uint32_t CLK;            /*!< AD Conversion Clock Setting Register          */
  __IO uint32_t MOD0;           /*!< AD Mode Setting Register 0                    */
  __IO uint32_t MOD1;           /*!< AD Mode Setting Register 1                    */
  __I  uint32_t MOD2;           /*!< AD Mode Setting Register 2                    */
  __IO uint32_t CMPCR0;         /*!< AD Monitoring Setting Register 0              */
  __IO uint32_t CMPCR1;         /*!< AD Monitoring Setting Register 1              */
  __IO uint32_t CMP0;           /*!< AD Conversion Result Comparison Register 0    */
  __IO uint32_t CMP1;           /*!< AD Conversion Result Comparison Register 1    */
  __I  uint32_t REG0;           /*!< AD Conversion Result Register 0               */
  __I  uint32_t REG1;           /*!< AD Conversion Result Register 1               */
  __I  uint32_t REG2;           /*!< AD Conversion Result Register 2               */
  __I  uint32_t REG3;           /*!< AD Conversion Result Register 3               */
  __I  uint32_t REG4;           /*!< AD Conversion Result Register 4               */
  __I  uint32_t REG5;           /*!< AD Conversion Result Register 5               */
  __I  uint32_t REG6;           /*!< AD Conversion Result Register 6               */
  __I  uint32_t REG7;           /*!< AD Conversion Result Register 7               */
  __I  uint32_t REG8;           /*!< AD Conversion Result Register 8               */
  __I  uint32_t REG9;           /*!< AD Conversion Result Register 9               */
  __I  uint32_t REG10;          /*!< AD Conversion Result Register 10              */
  __I  uint32_t REG11;          /*!< AD Conversion Result Register 11              */
  __IO uint32_t PSEL0;          /*!< PMD Trigger Program Number Select Register 0  */
  __IO uint32_t PSEL1;          /*!< PMD Trigger Program Number Select Register 1  */
  __IO uint32_t PSEL2;          /*!< PMD Trigger Program Number Select Register 2  */
  __IO uint32_t PSEL3;          /*!< PMD Trigger Program Number Select Register 3  */
  __IO uint32_t PSEL4;          /*!< PMD Trigger Program Number Select Register 4  */
  __IO uint32_t PSEL5;          /*!< PMD Trigger Program Number Select Register 5  */
  __IO uint32_t PSEL6;          /*!< PMD Trigger Program Number Select Register 6  */
  __IO uint32_t PSEL7;          /*!< PMD Trigger Program Number Select Register 7  */
  __IO uint32_t PSEL8;          /*!< PMD Trigger Program Number Select Register 8  */
  __IO uint32_t PSEL9;          /*!< PMD Trigger Program Number Select Register 9  */
  __IO uint32_t PSEL10;         /*!< PMD Trigger Program Number Select Register 10 */
  __IO uint32_t PSEL11;         /*!< PMD Trigger Program Number Select Register 11 */
  __IO uint32_t PINTS0;         /*!< PMD Trigger Interrupt Select Register 0       */
  __IO uint32_t PINTS1;         /*!< PMD Trigger Interrupt Select Register 1       */
  __IO uint32_t PINTS2;         /*!< PMD Trigger Interrupt Select Register 2       */
  __IO uint32_t PINTS3;         /*!< PMD Trigger Interrupt Select Register 3       */
  __IO uint32_t PINTS4;         /*!< PMD Trigger Interrupt Select Register 4       */
  __IO uint32_t PINTS5;         /*!< PMD Trigger Interrupt Select Register 5       */
  __IO uint8_t  PSET00;         /*!< PMD Trigger Program Register 00               */
  __IO uint8_t  PSET01;         /*!< PMD Trigger Program Register 01               */
  __IO uint8_t  PSET02;         /*!< PMD Trigger Program Register 02               */
  __IO uint8_t  PSET03;         /*!< PMD Trigger Program Register 03               */
  __IO uint8_t  PSET10;         /*!< PMD Trigger Program Register 10               */
  __IO uint8_t  PSET11;         /*!< PMD Trigger Program Register 11               */
  __IO uint8_t  PSET12;         /*!< PMD Trigger Program Register 12               */
  __IO uint8_t  PSET13;         /*!< PMD Trigger Program Register 13               */
  __IO uint8_t  PSET20;         /*!< PMD Trigger Program Register 20               */
  __IO uint8_t  PSET21;         /*!< PMD Trigger Program Register 21               */
  __IO uint8_t  PSET22;         /*!< PMD Trigger Program Register 22               */
  __IO uint8_t  PSET23;         /*!< PMD Trigger Program Register 23               */
  __IO uint8_t  PSET30;         /*!< PMD Trigger Program Register 30               */
  __IO uint8_t  PSET31;         /*!< PMD Trigger Program Register 31               */
  __IO uint8_t  PSET32;         /*!< PMD Trigger Program Register 32               */
  __IO uint8_t  PSET33;         /*!< PMD Trigger Program Register 33               */
  __IO uint8_t  PSET40;         /*!< PMD Trigger Program Register 40               */
  __IO uint8_t  PSET41;         /*!< PMD Trigger Program Register 41               */
  __IO uint8_t  PSET42;         /*!< PMD Trigger Program Register 42               */
  __IO uint8_t  PSET43;         /*!< PMD Trigger Program Register 43               */
  __IO uint8_t  PSET50;         /*!< PMD Trigger Program Register 50               */
  __IO uint8_t  PSET51;         /*!< PMD Trigger Program Register 51               */
  __IO uint8_t  PSET52;         /*!< PMD Trigger Program Register 52               */
  __IO uint8_t  PSET53;         /*!< PMD Trigger Program Register 53               */
  __IO uint8_t  TSET0;          /*!< Timer Trigger Program Register 0              */
  __IO uint8_t  TSET1;          /*!< Timer Trigger Program Register 1              */
  __IO uint8_t  TSET2;          /*!< Timer Trigger Program Register 2              */
  __IO uint8_t  TSET3;          /*!< Timer Trigger Program Register 3              */
  __IO uint8_t  TSET4;          /*!< Timer Trigger Program Register 4              */
  __IO uint8_t  TSET5;          /*!< Timer Trigger Program Register 5              */
  __IO uint8_t  TSET6;          /*!< Timer Trigger Program Register 6              */
  __IO uint8_t  TSET7;          /*!< Timer Trigger Program Register 7              */
  __IO uint8_t  TSET8;          /*!< Timer Trigger Program Register 8              */
  __IO uint8_t  TSET9;          /*!< Timer Trigger Program Register 9              */
  __IO uint8_t  TSET10;         /*!< Timer Trigger Program Register 10             */
  __IO uint8_t  TSET11;         /*!< Timer Trigger Program Register 11             */
  __IO uint8_t  SSET0;          /*!< Software Trigger Program Register 0           */
  __IO uint8_t  SSET1;          /*!< Software Trigger Program Register 1           */
  __IO uint8_t  SSET2;          /*!< Software Trigger Program Register 2           */
  __IO uint8_t  SSET3;          /*!< Software Trigger Program Register 3           */
  __IO uint8_t  SSET4;          /*!< Software Trigger Program Register 4           */
  __IO uint8_t  SSET5;          /*!< Software Trigger Program Register 5           */
  __IO uint8_t  SSET6;          /*!< Software Trigger Program Register 6           */
  __IO uint8_t  SSET7;          /*!< Software Trigger Program Register 7           */
  __IO uint8_t  SSET8;          /*!< Software Trigger Program Register 8           */
  __IO uint8_t  SSET9;          /*!< Software Trigger Program Register 9           */
  __IO uint8_t  SSET10;         /*!< Software Trigger Program Register 10          */
  __IO uint8_t  SSET11;         /*!< Software Trigger Program Register 11          */
  __IO uint8_t  ASET0;          /*!< Constant Trigger Program Register 0           */
  __IO uint8_t  ASET1;          /*!< Constant Trigger Program Register 1           */
  __IO uint8_t  ASET2;          /*!< Constant Trigger Program Register 2           */
  __IO uint8_t  ASET3;          /*!< Constant Trigger Program Register 3           */
  __IO uint8_t  ASET4;          /*!< Constant Trigger Program Register 4           */
  __IO uint8_t  ASET5;          /*!< Constant Trigger Program Register 5           */
  __IO uint8_t  ASET6;          /*!< Constant Trigger Program Register 6           */
  __IO uint8_t  ASET7;          /*!< Constant Trigger Program Register 7           */
  __IO uint8_t  ASET8;          /*!< Constant Trigger Program Register 8           */
  __IO uint8_t  ASET9;          /*!< Constant Trigger Program Register 9           */
  __IO uint8_t  ASET10;         /*!< Constant Trigger Program Register 10          */
  __IO uint8_t  ASET11;         /*!< Constant Trigger Program Register 11          */
} AD_TypeDef;

/**
  * @brief Op-Amps (AMP)
  */
typedef struct
{
  __IO uint32_t CTLA;           /*!< AMP Control Register A */
       uint32_t RESERVED0;
  __IO uint32_t CTLB;           /*!< AMP Control Register B */
       uint32_t RESERVED1;
  __IO uint32_t CTLC;           /*!< AMP Control Register C */
       uint32_t RESERVED2;
  __IO uint32_t CTLD;           /*!< AMP Control Register D */
} AMP_TypeDef;

/**
  * @brief Analog Comparators (CMP)
  */
typedef struct
{
  __IO uint32_t CTLA;           /*!< CMP Control Register A */
       uint32_t RESERVED0;
  __IO uint32_t CTLB;           /*!< CMP Control Register B */
       uint32_t RESERVED1;
  __IO uint32_t CTLC;           /*!< CMP Control Register C */
       uint32_t RESERVED2;
  __IO uint32_t CTLD;           /*!< CMP Control Register D */
} CMP_TypeDef;

/**
  * @brief Watchdog Timer (WD)
  */
typedef struct
{
  __IO uint32_t MOD;            /*!< WD Mode Register    */
  __O  uint32_t CR;             /*!< WD Control Register */
} WD_TypeDef;

/**
  * @brief Oscillation Frequency Detector (OFD)
  */
typedef struct
{
  __IO uint32_t CR1;            /*!< OFD Control Register 1 */
  __IO uint32_t CR2;            /*!< OFD Control Register 2 */
  __IO uint32_t MNPLLOFF;       /*!< Lower Detection Frequency Setting Register(In case of PLL OFF)  */
  __IO uint32_t MNPLLON;        /*!< Lower Detection Frequency Setting Register(In case of PLL ON)   */
  __IO uint32_t MXPLLOFF;       /*!< Higher Detection Frequency Setting Register(In case of PLL OFF) */
  __IO uint32_t MXPLLON;        /*!< Higher Detection Frequency Setting Register(In case of PLL ON)  */
} OFD_TypeDef;

/**
  * @brief Power On Reset (POR), Voltage Detecting circuit (VLTD)
  */
typedef struct
{
  __IO uint32_t CR;             /*!< Voltage Detection Control Register */
  __I  uint32_t SR;             /*!< Voltage Detection Status Register  */
} VD_TypeDef;

/**
  * @brief Vector Engine (VE)
  */
typedef struct
{
  __IO uint32_t EN;             /*!< VE enable/disable Register                                        */
  __O  uint32_t CPURUNTRG;      /*!< CPU Start Trigger Selection Register                              */
  __IO uint32_t TASKAPP;        /*!< Task Selection Register                                           */
  __IO uint32_t ACTSCH;         /*!< Operation Schedule Selection Register                             */
  __IO uint32_t REPTIME;        /*!< Schedule Repeat Count Register                                    */
  __IO uint32_t TRGMODE;        /*!< Start trigger mode Register                                       */
  __IO uint32_t ERRINTEN;       /*!< Error Interrupt enable/disable Register                           */
  __O  uint32_t COMPEND;        /*!< VE Forced Termination Register                                    */
  __I  uint32_t ERRDET;         /*!< Error Detection Register                                          */
  __I  uint32_t SCHTASKRUN;     /*!< Schedule executing flag/executing task Register                   */
       uint32_t RESERVED0;
  __IO uint32_t TMPREG0;        /*!< Temporary Register 0                                              */
  __IO uint32_t TMPREG1;        /*!< Temporary Register 1                                              */
  __IO uint32_t TMPREG2;        /*!< Temporary Register 2                                              */
  __IO uint32_t TMPREG3;        /*!< Temporary Register 3                                              */
  __IO uint32_t TMPREG4;        /*!< Temporary Register 4                                              */
  __IO uint32_t TMPREG5;        /*!< Temporary Register 5                                              */
  
  __IO uint32_t MCTLF0;         /*!< Status flags Register 0                                           */
  __IO uint32_t MODE0;          /*!< Task control mode Register 0                                      */
  __IO uint32_t FMODE0;         /*!< Flow control Register 0                                           */
  __IO uint32_t TPWM0;          /*!< PWM period rate Register 0                                        */
  __IO uint32_t OMEGA0;         /*!< Rotation speed Register 0                                         */
  __IO uint32_t THETA0;         /*!< Motor phase Register 0                                            */
  __IO uint32_t IDREF0;         /*!< d-axis reference value Register 0                                 */
  __IO uint32_t IQREF0;         /*!< q-axis reference value Register 0                                 */
  __IO uint32_t VD0;            /*!< d-axis voltage Register 0                                         */
  __IO uint32_t VQ0;            /*!< q-axis voltage Register 0                                         */
  __IO uint32_t CIDKI0;         /*!< Integral coefficient for PI control of d-axis Register 0          */
  __IO uint32_t CIDKP0;         /*!< Proportional coefficient for PI control of d-axis Register 0      */
  __IO uint32_t CIQKI0;         /*!< Integral coefficient for PI control of q-axis Register 0          */
  __IO uint32_t CIQKP0;         /*!< Proportional coefficient for PI control of q-axis Register 0      */
  __IO uint32_t VDIH0;          /*!< Upper 32 bits of integral term (VDI) of d-axis voltage Register 0 */
  __IO uint32_t VDILH0;         /*!< Lower 32 bits of integral term (VDI) of d-axis voltage Register 0 */
  __IO uint32_t VQIH0;          /*!< Upper 32 bits of integral term (VQI) of q-axis voltage Register 0 */
  __IO uint32_t VQILH0;         /*!< Lower 32 bits of integral term (VQI) of q-axis voltage Register 0 */
  __IO uint32_t FPWMCHG0;       /*!< Switching speed Register 0                                        */
  __IO uint32_t MDPRD0;         /*!< PWM period Register 0                                             */
  __IO uint32_t MINPLS0;        /*!< Minimum pulse width Register 0                                    */
  __IO uint32_t TRGCRC0;        /*!< Synchronizing trigger correction value Register 0                 */
       uint32_t RESERVED1; 
  __IO uint32_t COS0;           /*!< Cosine value at THETA for output conversion Register 0            */
  __IO uint32_t SIN0;           /*!< Sine value at THETA for output conversion Register 0              */
  __IO uint32_t COSM0;          /*!< Previous cosine value for input processing Register 0             */
  __IO uint32_t SINM0;          /*!< Previous sine value for input processing Register 0               */
  __IO uint32_t SECTOR0;        /*!< Sector information Register 0                                     */
  __IO uint32_t SECTORM0;       /*!< Previous sector information for input processing Register 0       */
  __IO uint32_t IAO0;           /*!< AD conversion result of a-phase zero-current Register 0           */
  __IO uint32_t IBO0;           /*!< AD conversion result of b-phase zero-current Register 0           */
  __IO uint32_t ICO0;           /*!< AD conversion result of c-phase zero-current Register 0           */
  __IO uint32_t IAADC0;         /*!< AD conversion result of a-phase current Register 0                */
  __IO uint32_t IBADC0;         /*!< AD conversion result of b-phase current Register 0                */
  __IO uint32_t ICADC0;         /*!< AD conversion result of c-phase current Register 0                */
  __IO uint32_t VDC0;           /*!< DC supply voltage Register 0                                      */
  __IO uint32_t ID0;            /*!< d-axis current Register 0                                         */
  __IO uint32_t IQ0;            /*!< q-axis current Register 0                                         */
  __IO uint32_t MCTLF1;         /*!< Status flags Register 1                                           */
  __IO uint32_t MODE1;          /*!< Task control mode Register 1                                      */
  __IO uint32_t FMODE1;         /*!< Flow control Register 1                                           */
  __IO uint32_t TPWM1;          /*!< PWM period rate Register 1                                        */
  __IO uint32_t OMEGA1;         /*!< Rotation speed Register 1                                         */
  __IO uint32_t THETA1;         /*!< Motor phase Register 1                                            */
  __IO uint32_t IDREF1;         /*!< d-axis reference value Register 1                                 */
  __IO uint32_t IQREF1;         /*!< q-axis reference value Register 1                                 */
  __IO uint32_t VD1;            /*!< d-axis voltage Register 1                                         */
  __IO uint32_t VQ1;            /*!< q-axis voltage Register 1                                         */
  __IO uint32_t CIDKI1;         /*!< Integral coefficient for PI control of d-axis Register 1          */
  __IO uint32_t CIDKP1;         /*!< Proportional coefficient for PI control of d-axis Register 1      */
  __IO uint32_t CIQKI1;         /*!< Integral coefficient for PI control of q-axis Register 1          */
  __IO uint32_t CIQKP1;         /*!< Proportional coefficient for PI control of q-axis Register 1      */
  __IO uint32_t VDIH1;          /*!< Upper 32 bits of integral term (VDI) of d-axis voltage Register 1 */
  __IO uint32_t VDILH1;         /*!< Lower 32 bits of integral term (VDI) of d-axis voltage Register 1 */
  __IO uint32_t VQIH1;          /*!< Upper 32 bits of integral term (VQI) of q-axis voltage Register 1 */
  __IO uint32_t VQILH1;         /*!< Lower 32 bits of integral term (VQI) of q-axis voltage Register 1 */
  __IO uint32_t FPWMCHG1;       /*!< Switching speed Register 1                                        */
  __IO uint32_t MDPRD1;         /*!< PWM period Register 1                                             */
  __IO uint32_t MINPLS1;        /*!< Minimum pulse width Register 1                                    */
  __IO uint32_t TRGCRC1;        /*!< Synchronizing trigger correction value Register 1                 */
       uint32_t RESERVED2; 
  __IO uint32_t COS1;           /*!< Cosine value at THETA for output conversion Register 1            */
  __IO uint32_t SIN1;           /*!< Sine value at THETA for output conversion Register 1              */
  __IO uint32_t COSM1;          /*!< Previous cosine value for input processing Register 1             */
  __IO uint32_t SINM1;          /*!< Previous sine value for input processing Register 1               */
  __IO uint32_t SECTOR1;        /*!< Sector information Register 1                                     */
  __IO uint32_t SECTORM1;       /*!< Previous sector information for input processing Register 1       */
  __IO uint32_t IAO1;           /*!< AD conversion result of a-phase zero-current Register 1           */
  __IO uint32_t IBO1;           /*!< AD conversion result of b-phase zero-current Register 1           */
  __IO uint32_t ICO1;           /*!< AD conversion result of c-phase zero-current Register 1           */
  __IO uint32_t IAADC1;         /*!< AD conversion result of a-phase current Register 1                */
  __IO uint32_t IBADC1;         /*!< AD conversion result of b-phase current Register 1                */
  __IO uint32_t ICADC1;         /*!< AD conversion result of c-phase current Register 1                */
  __IO uint32_t VDC1;           /*!< DC supply voltage Register 1                                      */
  __IO uint32_t ID1;            /*!< d-axis current Register 1                                         */
  __IO uint32_t IQ1;            /*!< q-axis current Register 1                                         */
       uint32_t RESERVED3;
  __IO uint32_t TADC;           /*!< ADC conversion time (based on PWM clock) Register                 */
  __IO uint32_t CMPU0;          /*!< PMD control: CMPU setting Register 0                              */
  __IO uint32_t CMPV0;          /*!< PMD control: CMPV setting Register 0                              */
  __IO uint32_t CMPW0;          /*!< PMD control: CMPW setting Register 0                              */
  __IO uint32_t OUTCR0;         /*!< PMD control: Output control Register 0                            */
  __IO uint32_t TRGCMP00;       /*!< PMD control: TRGCMP0 setting Register 0                           */
  __IO uint32_t TRGCMP10;       /*!< PMD control: TRGCMP1 setting Register 0                           */
  __IO uint32_t TRGSEL0;        /*!< PMD control: Trigger selection Register 0                         */
  __O  uint32_t EMGRS0;         /*!< PMD control: EMG return Register 0                                */
  __IO uint32_t CMPU1;          /*!< PMD control: CMPU setting Register 1                              */
  __IO uint32_t CMPV1;          /*!< PMD control: CMPV setting Register 1                              */
  __IO uint32_t CMPW1;          /*!< PMD control: CMPW setting Register 1                              */
  __IO uint32_t OUTCR1;         /*!< PMD control: Output control Register 1                            */
  __IO uint32_t TRGCMP01;       /*!< PMD control: TRGCMP0 setting Register 1                           */
  __IO uint32_t TRGCMP11;       /*!< PMD control: TRGCMP1 setting Register 1                           */
  __IO uint32_t TRGSEL1;        /*!< PMD control: Trigger selection Register 1                         */
  __O  uint32_t EMGRS1;         /*!< PMD control: EMG return Register 1                                */
} VE_TypeDef;

/**
  * @brief Programmable Motor Driver (PMD)
  */
typedef struct
{
  __IO uint32_t MDEN;           /*!< PMD Enable Register                  */
  __IO uint32_t PORTMD;         /*!< Port Output Mode Register            */
  __IO uint32_t MDCR;           /*!< PMD Control Register                 */
  __I  uint32_t CNTSTA;         /*!< PWM Counter Status Register          */
  __I  uint32_t MDCNT;          /*!< PWM Counter Register                 */
  __IO uint32_t MDPRD;          /*!< PWM Period Register                  */
  __IO uint32_t CMPU;           /*!< PMD Compare U Register               */
  __IO uint32_t CMPV;           /*!< PMD Compare V Register               */
  __IO uint32_t CMPW;           /*!< PMD Compare W Register               */
  __IO uint32_t MODESEL;        /*!< Mode Select Register                 */
  __IO uint32_t MDOUT;          /*!< PMD Output Control Register          */
  __IO uint32_t MDPOT;          /*!< PMD Output Setting Register          */
  __O  uint32_t EMGREL;         /*!< EMG Release Register                 */
  __IO uint32_t EMGCR;          /*!< EMG Control Register                 */
  __I  uint32_t EMGSTA;         /*!< EMG Status Register                  */
  __IO uint32_t OVVCR;          /*!< OVV Control Register                 */
  __I  uint32_t OVVSTA;         /*!< OVV Status Register                  */
  __IO uint32_t DTR;            /*!< Dead Time Register                   */
  __IO uint32_t TRGCMP0;        /*!< Trigger Compare 0 Register           */
  __IO uint32_t TRGCMP1;        /*!< Trigger Compare 1 Register           */
  __IO uint32_t TRGCMP2;        /*!< Trigger Compare 2 Register           */
  __IO uint32_t TRGCMP3;        /*!< Trigger Compare 3 Register           */
  __IO uint32_t TRGCR;          /*!< Trigger Control Register             */
  __IO uint32_t TRGMD;          /*!< Trigger Output Mode Setting Register */
  __IO uint32_t TRGSEL;         /*!< Trigger Output Select Register       */
} PMD_TypeDef;

/**
  * @brief Flash Control (FC)
  */
typedef struct
{
  __IO uint32_t SECBIT;         /*!< FC Security Bit Register  */
       uint32_t RESERVED0[3];
  __I  uint32_t FLCS;           /*!< FC Flash Control Register */
} FC_TypeDef;


/* Memory map */
#define FLASH_BASE            (0x00000000UL)
#define RAM_BASE              (0x20000000UL)
#define PERI_BASE             (0x40000000UL)

#define PA_BASE               (PERI_BASE  + 0x00000UL)
#define PB_BASE               (PERI_BASE  + 0x00040UL)
#define PC_BASE               (PERI_BASE  + 0x00080UL)
#define PD_BASE               (PERI_BASE  + 0x000C0UL)
#define PE_BASE               (PERI_BASE  + 0x00100UL)
#define PF_BASE               (PERI_BASE  + 0x00140UL)
#define PG_BASE               (PERI_BASE  + 0x00180UL)
#define PH_BASE               (PERI_BASE  + 0x001C0UL)
#define PI_BASE               (PERI_BASE  + 0x00200UL)
#define PJ_BASE               (PERI_BASE  + 0x00240UL)
#define PK_BASE               (PERI_BASE  + 0x00280UL)
#define PL_BASE               (PERI_BASE  + 0x002C0UL)
#define TB0_BASE              (PERI_BASE  + 0x10000UL)
#define TB1_BASE              (PERI_BASE  + 0x10040UL)
#define TB2_BASE              (PERI_BASE  + 0x10080UL)
#define TB3_BASE              (PERI_BASE  + 0x100C0UL)
#define TB4_BASE              (PERI_BASE  + 0x10100UL)
#define TB5_BASE              (PERI_BASE  + 0x10140UL)
#define TB6_BASE              (PERI_BASE  + 0x10180UL)
#define TB7_BASE              (PERI_BASE  + 0x101C0UL)
#define EN0_BASE              (PERI_BASE  + 0x10400UL)
#define EN1_BASE              (PERI_BASE  + 0x10500UL)
#define SC0_BASE              (PERI_BASE  + 0x20080UL)
#define SC1_BASE              (PERI_BASE  + 0x200C0UL)
#define SC2_BASE              (PERI_BASE  + 0x20100UL)
#define SC3_BASE              (PERI_BASE  + 0x20140UL)
#define ADA_BASE              (PERI_BASE  + 0x30000UL)
#define ADB_BASE              (PERI_BASE  + 0x30200UL)
#define AMP_BASE              (PERI_BASE  + 0x30400UL)
#define CMP_BASE              (PERI_BASE  + 0x30420UL)
#define WD_BASE               (PERI_BASE  + 0x40000UL)
#define CG_BASE               (PERI_BASE  + 0x40200UL)
#define OFD_BASE              (PERI_BASE  + 0x40800UL)
#define VD_BASE               (PERI_BASE  + 0x40900UL)
#define VE_BASE               (PERI_BASE  + 0x50000UL)
#define PMD0_BASE             (PERI_BASE  + 0x50400UL)
#define PMD1_BASE             (PERI_BASE  + 0x50480UL)
#define FC_BASE               (PERI_BASE  + 0x1FFF010UL)


/* Peripheral declaration */
#define PA                    ((     PA_TypeDef *)       PA_BASE)
#define PB                    ((     PB_TypeDef *)       PB_BASE)
#define PC                    ((     PC_TypeDef *)       PC_BASE)
#define PD                    ((     PD_TypeDef *)       PD_BASE)
#define PE                    ((     PE_TypeDef *)       PE_BASE)
#define PF                    ((     PF_TypeDef *)       PF_BASE)
#define PG                    ((     PG_TypeDef *)       PG_BASE)
#define PH                    ((     PH_TypeDef *)       PH_BASE)
#define PI                    ((     PI_TypeDef *)       PI_BASE)
#define PJ                    ((     PJ_TypeDef *)       PJ_BASE)
#define PK                    ((     PK_TypeDef *)       PK_BASE)
#define PL                    ((     PL_TypeDef *)       PL_BASE)
#define TB0                   ((     TB_TypeDef *)      TB0_BASE)
#define TB1                   ((     TB_TypeDef *)      TB1_BASE)
#define TB2                   ((     TB_TypeDef *)      TB2_BASE)
#define TB3                   ((     TB_TypeDef *)      TB3_BASE)
#define TB4                   ((     TB_TypeDef *)      TB4_BASE)
#define TB5                   ((     TB_TypeDef *)      TB5_BASE)
#define TB6                   ((     TB_TypeDef *)      TB6_BASE)
#define TB7                   ((     TB_TypeDef *)      TB7_BASE)
#define EN0                   ((     EN_TypeDef *)      EN0_BASE)
#define EN1                   ((     EN_TypeDef *)      EN1_BASE)
#define SC0                   ((     SC_TypeDef *)      SC0_BASE)
#define SC1                   ((     SC_TypeDef *)      SC1_BASE)
#define SC2                   ((     SC_TypeDef *)      SC2_BASE)
#define SC3                   ((     SC_TypeDef *)      SC3_BASE)
#define ADA                   ((     AD_TypeDef *)      ADA_BASE)
#define ADB                   ((     AD_TypeDef *)      ADB_BASE)
#define AMP                   ((    AMP_TypeDef *)      AMP_BASE)
#define CMP                   ((    CMP_TypeDef *)      CMP_BASE)
#define WD                    ((     WD_TypeDef *)       WD_BASE)
#define CG                    ((     CG_TypeDef *)       CG_BASE)
#define OFD                   ((    OFD_TypeDef *)      OFD_BASE)
#define VD                    ((     VD_TypeDef *)       VD_BASE)
#define VE                    ((     VE_TypeDef *)       VE_BASE)
#define PMD0                  ((    PMD_TypeDef *)     PMD0_BASE)
#define PMD1                  ((    PMD_TypeDef *)     PMD1_BASE)
#define FC                    ((     FC_TypeDef *)       FC_BASE)


/* Bit-Band for Device Specific Peripheral Registers */
#define BITBAND_OFFSET (0x02000000UL)
#define BITBAND_PERI_BASE (PERI_BASE + BITBAND_OFFSET)
#define BITBAND_PERI(addr, bitnum) (BITBAND_PERI_BASE + (((uint32_t)(addr) - PERI_BASE) << 5) + ((uint32_t)(bitnum) << 2))

/* General Purpose Input/Output Port (PA) */
#define PA_DATA_PA0           (*((__IO uint32_t *)BITBAND_PERI(&PA->DATA,0)))
#define PA_DATA_PA1           (*((__IO uint32_t *)BITBAND_PERI(&PA->DATA,1)))
#define PA_DATA_PA2           (*((__IO uint32_t *)BITBAND_PERI(&PA->DATA,2)))
#define PA_DATA_PA3           (*((__IO uint32_t *)BITBAND_PERI(&PA->DATA,3)))
#define PA_DATA_PA4           (*((__IO uint32_t *)BITBAND_PERI(&PA->DATA,4)))
#define PA_DATA_PA5           (*((__IO uint32_t *)BITBAND_PERI(&PA->DATA,5)))
#define PA_DATA_PA6           (*((__IO uint32_t *)BITBAND_PERI(&PA->DATA,6)))
#define PA_DATA_PA7           (*((__IO uint32_t *)BITBAND_PERI(&PA->DATA,7)))
#define PA_CR_PA0C            (*((__IO uint32_t *)BITBAND_PERI(&PA->CR,0)))
#define PA_CR_PA1C            (*((__IO uint32_t *)BITBAND_PERI(&PA->CR,1)))
#define PA_CR_PA2C            (*((__IO uint32_t *)BITBAND_PERI(&PA->CR,2)))
#define PA_CR_PA3C            (*((__IO uint32_t *)BITBAND_PERI(&PA->CR,3)))
#define PA_CR_PA4C            (*((__IO uint32_t *)BITBAND_PERI(&PA->CR,4)))
#define PA_CR_PA5C            (*((__IO uint32_t *)BITBAND_PERI(&PA->CR,5)))
#define PA_CR_PA6C            (*((__IO uint32_t *)BITBAND_PERI(&PA->CR,6)))
#define PA_CR_PA7C            (*((__IO uint32_t *)BITBAND_PERI(&PA->CR,7)))
#define PA_FR1_PA0F1          (*((__IO uint32_t *)BITBAND_PERI(&PA->FR1,0)))
#define PA_FR1_PA1F1          (*((__IO uint32_t *)BITBAND_PERI(&PA->FR1,1)))
#define PA_FR1_PA2F1          (*((__IO uint32_t *)BITBAND_PERI(&PA->FR1,2)))
#define PA_FR1_PA3F1          (*((__IO uint32_t *)BITBAND_PERI(&PA->FR1,3)))
#define PA_FR1_PA4F1          (*((__IO uint32_t *)BITBAND_PERI(&PA->FR1,4)))
#define PA_FR1_PA5F1          (*((__IO uint32_t *)BITBAND_PERI(&PA->FR1,5)))
#define PA_FR1_PA6F1          (*((__IO uint32_t *)BITBAND_PERI(&PA->FR1,6)))
#define PA_FR1_PA7F1          (*((__IO uint32_t *)BITBAND_PERI(&PA->FR1,7)))
#define PA_FR2_PA0F2          (*((__IO uint32_t *)BITBAND_PERI(&PA->FR2,0)))
#define PA_FR2_PA2F2          (*((__IO uint32_t *)BITBAND_PERI(&PA->FR2,2)))
#define PA_FR2_PA4F2          (*((__IO uint32_t *)BITBAND_PERI(&PA->FR2,4)))
#define PA_FR2_PA5F2          (*((__IO uint32_t *)BITBAND_PERI(&PA->FR2,5)))
#define PA_FR2_PA6F2          (*((__IO uint32_t *)BITBAND_PERI(&PA->FR2,6)))
#define PA_FR2_PA7F2          (*((__IO uint32_t *)BITBAND_PERI(&PA->FR2,7)))
#define PA_OD_PA0OD           (*((__IO uint32_t *)BITBAND_PERI(&PA->OD,0)))
#define PA_OD_PA1OD           (*((__IO uint32_t *)BITBAND_PERI(&PA->OD,1)))
#define PA_OD_PA2OD           (*((__IO uint32_t *)BITBAND_PERI(&PA->OD,2)))
#define PA_OD_PA3OD           (*((__IO uint32_t *)BITBAND_PERI(&PA->OD,3)))
#define PA_OD_PA4OD           (*((__IO uint32_t *)BITBAND_PERI(&PA->OD,4)))
#define PA_OD_PA5OD           (*((__IO uint32_t *)BITBAND_PERI(&PA->OD,5)))
#define PA_OD_PA6OD           (*((__IO uint32_t *)BITBAND_PERI(&PA->OD,6)))
#define PA_OD_PA7OD           (*((__IO uint32_t *)BITBAND_PERI(&PA->OD,7)))
#define PA_PUP_PA0UP          (*((__IO uint32_t *)BITBAND_PERI(&PA->PUP,0)))
#define PA_PUP_PA1UP          (*((__IO uint32_t *)BITBAND_PERI(&PA->PUP,1)))
#define PA_PUP_PA2UP          (*((__IO uint32_t *)BITBAND_PERI(&PA->PUP,2)))
#define PA_PUP_PA3UP          (*((__IO uint32_t *)BITBAND_PERI(&PA->PUP,3)))
#define PA_PUP_PA4UP          (*((__IO uint32_t *)BITBAND_PERI(&PA->PUP,4)))
#define PA_PUP_PA5UP          (*((__IO uint32_t *)BITBAND_PERI(&PA->PUP,5)))
#define PA_PUP_PA6UP          (*((__IO uint32_t *)BITBAND_PERI(&PA->PUP,6)))
#define PA_PUP_PA7UP          (*((__IO uint32_t *)BITBAND_PERI(&PA->PUP,7)))
#define PA_PDN_PA0DN          (*((__IO uint32_t *)BITBAND_PERI(&PA->PDN,0)))
#define PA_PDN_PA1DN          (*((__IO uint32_t *)BITBAND_PERI(&PA->PDN,1)))
#define PA_PDN_PA2DN          (*((__IO uint32_t *)BITBAND_PERI(&PA->PDN,2)))
#define PA_PDN_PA3DN          (*((__IO uint32_t *)BITBAND_PERI(&PA->PDN,3)))
#define PA_PDN_PA4DN          (*((__IO uint32_t *)BITBAND_PERI(&PA->PDN,4)))
#define PA_PDN_PA5DN          (*((__IO uint32_t *)BITBAND_PERI(&PA->PDN,5)))
#define PA_PDN_PA6DN          (*((__IO uint32_t *)BITBAND_PERI(&PA->PDN,6)))
#define PA_PDN_PA7DN          (*((__IO uint32_t *)BITBAND_PERI(&PA->PDN,7)))
#define PA_IE_PA0IE           (*((__IO uint32_t *)BITBAND_PERI(&PA->IE,0)))
#define PA_IE_PA1IE           (*((__IO uint32_t *)BITBAND_PERI(&PA->IE,1)))
#define PA_IE_PA2IE           (*((__IO uint32_t *)BITBAND_PERI(&PA->IE,2)))
#define PA_IE_PA3IE           (*((__IO uint32_t *)BITBAND_PERI(&PA->IE,3)))
#define PA_IE_PA4IE           (*((__IO uint32_t *)BITBAND_PERI(&PA->IE,4)))
#define PA_IE_PA5IE           (*((__IO uint32_t *)BITBAND_PERI(&PA->IE,5)))
#define PA_IE_PA6IE           (*((__IO uint32_t *)BITBAND_PERI(&PA->IE,6)))
#define PA_IE_PA7IE           (*((__IO uint32_t *)BITBAND_PERI(&PA->IE,7)))

/* General Purpose Input/Output Port (PB) */
#define PB_DATA_PB0           (*((__IO uint32_t *)BITBAND_PERI(&PB->DATA,0)))
#define PB_DATA_PB1           (*((__IO uint32_t *)BITBAND_PERI(&PB->DATA,1)))
#define PB_DATA_PB2           (*((__IO uint32_t *)BITBAND_PERI(&PB->DATA,2)))
#define PB_DATA_PB3           (*((__IO uint32_t *)BITBAND_PERI(&PB->DATA,3)))
#define PB_DATA_PB4           (*((__IO uint32_t *)BITBAND_PERI(&PB->DATA,4)))
#define PB_DATA_PB5           (*((__IO uint32_t *)BITBAND_PERI(&PB->DATA,5)))
#define PB_DATA_PB6           (*((__IO uint32_t *)BITBAND_PERI(&PB->DATA,6)))
#define PB_DATA_PB7           (*((__IO uint32_t *)BITBAND_PERI(&PB->DATA,7)))
#define PB_CR_PB0C            (*((__IO uint32_t *)BITBAND_PERI(&PB->CR,0)))
#define PB_CR_PB1C            (*((__IO uint32_t *)BITBAND_PERI(&PB->CR,1)))
#define PB_CR_PB2C            (*((__IO uint32_t *)BITBAND_PERI(&PB->CR,2)))
#define PB_CR_PB3C            (*((__IO uint32_t *)BITBAND_PERI(&PB->CR,3)))
#define PB_CR_PB4C            (*((__IO uint32_t *)BITBAND_PERI(&PB->CR,4)))
#define PB_CR_PB5C            (*((__IO uint32_t *)BITBAND_PERI(&PB->CR,5)))
#define PB_CR_PB6C            (*((__IO uint32_t *)BITBAND_PERI(&PB->CR,6)))
#define PB_CR_PB7C            (*((__IO uint32_t *)BITBAND_PERI(&PB->CR,7)))
#define PB_FR1_PB0F1          (*((__IO uint32_t *)BITBAND_PERI(&PB->FR1,0)))
#define PB_FR1_PB1F1          (*((__IO uint32_t *)BITBAND_PERI(&PB->FR1,1)))
#define PB_FR1_PB2F1          (*((__IO uint32_t *)BITBAND_PERI(&PB->FR1,2)))
#define PB_FR1_PB3F1          (*((__IO uint32_t *)BITBAND_PERI(&PB->FR1,3)))
#define PB_FR1_PB4F1          (*((__IO uint32_t *)BITBAND_PERI(&PB->FR1,4)))
#define PB_FR1_PB5F1          (*((__IO uint32_t *)BITBAND_PERI(&PB->FR1,5)))
#define PB_FR1_PB6F1          (*((__IO uint32_t *)BITBAND_PERI(&PB->FR1,6)))
#define PB_FR1_PB7F1          (*((__IO uint32_t *)BITBAND_PERI(&PB->FR1,7)))
#define PB_PUP_PB0UP          (*((__IO uint32_t *)BITBAND_PERI(&PB->PUP,0)))
#define PB_PUP_PB1UP          (*((__IO uint32_t *)BITBAND_PERI(&PB->PUP,1)))
#define PB_PUP_PB2UP          (*((__IO uint32_t *)BITBAND_PERI(&PB->PUP,2)))
#define PB_PUP_PB3UP          (*((__IO uint32_t *)BITBAND_PERI(&PB->PUP,3)))
#define PB_PUP_PB5UP          (*((__IO uint32_t *)BITBAND_PERI(&PB->PUP,5)))
#define PB_PUP_PB6UP          (*((__IO uint32_t *)BITBAND_PERI(&PB->PUP,6)))
#define PB_PUP_PB7UP          (*((__IO uint32_t *)BITBAND_PERI(&PB->PUP,7)))
#define PB_PDN_PB4DN          (*((__IO uint32_t *)BITBAND_PERI(&PB->PDN,4)))
#define PB_IE_PB0IE           (*((__IO uint32_t *)BITBAND_PERI(&PB->IE,0)))
#define PB_IE_PB1IE           (*((__IO uint32_t *)BITBAND_PERI(&PB->IE,1)))
#define PB_IE_PB2IE           (*((__IO uint32_t *)BITBAND_PERI(&PB->IE,2)))
#define PB_IE_PB3IE           (*((__IO uint32_t *)BITBAND_PERI(&PB->IE,3)))
#define PB_IE_PB4IE           (*((__IO uint32_t *)BITBAND_PERI(&PB->IE,4)))
#define PB_IE_PB5IE           (*((__IO uint32_t *)BITBAND_PERI(&PB->IE,5)))
#define PB_IE_PB6IE           (*((__IO uint32_t *)BITBAND_PERI(&PB->IE,6)))
#define PB_IE_PB7IE           (*((__IO uint32_t *)BITBAND_PERI(&PB->IE,7)))

/* General Purpose Input/Output Port (PC) */
#define PC_DATA_PC0           (*((__IO uint32_t *)BITBAND_PERI(&PC->DATA,0)))
#define PC_DATA_PC1           (*((__IO uint32_t *)BITBAND_PERI(&PC->DATA,1)))
#define PC_DATA_PC2           (*((__IO uint32_t *)BITBAND_PERI(&PC->DATA,2)))
#define PC_DATA_PC3           (*((__IO uint32_t *)BITBAND_PERI(&PC->DATA,3)))
#define PC_DATA_PC4           (*((__IO uint32_t *)BITBAND_PERI(&PC->DATA,4)))
#define PC_DATA_PC5           (*((__IO uint32_t *)BITBAND_PERI(&PC->DATA,5)))
#define PC_DATA_PC6           (*((__IO uint32_t *)BITBAND_PERI(&PC->DATA,6)))
#define PC_DATA_PC7           (*((__IO uint32_t *)BITBAND_PERI(&PC->DATA,7)))
#define PC_CR_PC0C            (*((__IO uint32_t *)BITBAND_PERI(&PC->CR,0)))
#define PC_CR_PC1C            (*((__IO uint32_t *)BITBAND_PERI(&PC->CR,1)))
#define PC_CR_PC2C            (*((__IO uint32_t *)BITBAND_PERI(&PC->CR,2)))
#define PC_CR_PC3C            (*((__IO uint32_t *)BITBAND_PERI(&PC->CR,3)))
#define PC_CR_PC4C            (*((__IO uint32_t *)BITBAND_PERI(&PC->CR,4)))
#define PC_CR_PC5C            (*((__IO uint32_t *)BITBAND_PERI(&PC->CR,5)))
#define PC_CR_PC6C            (*((__IO uint32_t *)BITBAND_PERI(&PC->CR,6)))
#define PC_CR_PC7C            (*((__IO uint32_t *)BITBAND_PERI(&PC->CR,7)))
#define PC_FR1_PC0F1          (*((__IO uint32_t *)BITBAND_PERI(&PC->FR1,0)))
#define PC_FR1_PC1F1          (*((__IO uint32_t *)BITBAND_PERI(&PC->FR1,1)))
#define PC_FR1_PC2F1          (*((__IO uint32_t *)BITBAND_PERI(&PC->FR1,2)))
#define PC_FR1_PC3F1          (*((__IO uint32_t *)BITBAND_PERI(&PC->FR1,3)))
#define PC_FR1_PC4F1          (*((__IO uint32_t *)BITBAND_PERI(&PC->FR1,4)))
#define PC_FR1_PC5F1          (*((__IO uint32_t *)BITBAND_PERI(&PC->FR1,5)))
#define PC_FR1_PC6F1          (*((__IO uint32_t *)BITBAND_PERI(&PC->FR1,6)))
#define PC_FR1_PC7F1          (*((__IO uint32_t *)BITBAND_PERI(&PC->FR1,7)))
#define PC_OD_PC0OD           (*((__IO uint32_t *)BITBAND_PERI(&PC->OD,0)))
#define PC_OD_PC1OD           (*((__IO uint32_t *)BITBAND_PERI(&PC->OD,1)))
#define PC_OD_PC2OD           (*((__IO uint32_t *)BITBAND_PERI(&PC->OD,2)))
#define PC_OD_PC3OD           (*((__IO uint32_t *)BITBAND_PERI(&PC->OD,3)))
#define PC_OD_PC4OD           (*((__IO uint32_t *)BITBAND_PERI(&PC->OD,4)))
#define PC_OD_PC5OD           (*((__IO uint32_t *)BITBAND_PERI(&PC->OD,5)))
#define PC_OD_PC6OD           (*((__IO uint32_t *)BITBAND_PERI(&PC->OD,6)))
#define PC_OD_PC7OD           (*((__IO uint32_t *)BITBAND_PERI(&PC->OD,7)))
#define PC_PUP_PC0UP          (*((__IO uint32_t *)BITBAND_PERI(&PC->PUP,0)))
#define PC_PUP_PC1UP          (*((__IO uint32_t *)BITBAND_PERI(&PC->PUP,1)))
#define PC_PUP_PC2UP          (*((__IO uint32_t *)BITBAND_PERI(&PC->PUP,2)))
#define PC_PUP_PC3UP          (*((__IO uint32_t *)BITBAND_PERI(&PC->PUP,3)))
#define PC_PUP_PC4UP          (*((__IO uint32_t *)BITBAND_PERI(&PC->PUP,4)))
#define PC_PUP_PC5UP          (*((__IO uint32_t *)BITBAND_PERI(&PC->PUP,5)))
#define PC_PUP_PC6UP          (*((__IO uint32_t *)BITBAND_PERI(&PC->PUP,6)))
#define PC_PUP_PC7UP          (*((__IO uint32_t *)BITBAND_PERI(&PC->PUP,7)))
#define PC_PDN_PC0DN          (*((__IO uint32_t *)BITBAND_PERI(&PC->PDN,0)))
#define PC_PDN_PC1DN          (*((__IO uint32_t *)BITBAND_PERI(&PC->PDN,1)))
#define PC_PDN_PC2DN          (*((__IO uint32_t *)BITBAND_PERI(&PC->PDN,2)))
#define PC_PDN_PC3DN          (*((__IO uint32_t *)BITBAND_PERI(&PC->PDN,3)))
#define PC_PDN_PC4DN          (*((__IO uint32_t *)BITBAND_PERI(&PC->PDN,4)))
#define PC_PDN_PC5DN          (*((__IO uint32_t *)BITBAND_PERI(&PC->PDN,5)))
#define PC_PDN_PC6DN          (*((__IO uint32_t *)BITBAND_PERI(&PC->PDN,6)))
#define PC_PDN_PC7DN          (*((__IO uint32_t *)BITBAND_PERI(&PC->PDN,7)))
#define PC_IE_PC0IE           (*((__IO uint32_t *)BITBAND_PERI(&PC->IE,0)))
#define PC_IE_PC1IE           (*((__IO uint32_t *)BITBAND_PERI(&PC->IE,1)))
#define PC_IE_PC2IE           (*((__IO uint32_t *)BITBAND_PERI(&PC->IE,2)))
#define PC_IE_PC3IE           (*((__IO uint32_t *)BITBAND_PERI(&PC->IE,3)))
#define PC_IE_PC4IE           (*((__IO uint32_t *)BITBAND_PERI(&PC->IE,4)))
#define PC_IE_PC5IE           (*((__IO uint32_t *)BITBAND_PERI(&PC->IE,5)))
#define PC_IE_PC6IE           (*((__IO uint32_t *)BITBAND_PERI(&PC->IE,6)))
#define PC_IE_PC7IE           (*((__IO uint32_t *)BITBAND_PERI(&PC->IE,7)))

/* General Purpose Input/Output Port (PD) */
#define PD_DATA_PD0           (*((__IO uint32_t *)BITBAND_PERI(&PD->DATA,0)))
#define PD_DATA_PD1           (*((__IO uint32_t *)BITBAND_PERI(&PD->DATA,1)))
#define PD_DATA_PD2           (*((__IO uint32_t *)BITBAND_PERI(&PD->DATA,2)))
#define PD_DATA_PD3           (*((__IO uint32_t *)BITBAND_PERI(&PD->DATA,3)))
#define PD_DATA_PD4           (*((__IO uint32_t *)BITBAND_PERI(&PD->DATA,4)))
#define PD_DATA_PD5           (*((__IO uint32_t *)BITBAND_PERI(&PD->DATA,5)))
#define PD_DATA_PD6           (*((__IO uint32_t *)BITBAND_PERI(&PD->DATA,6)))
#define PD_CR_PD0C            (*((__IO uint32_t *)BITBAND_PERI(&PD->CR,0)))
#define PD_CR_PD1C            (*((__IO uint32_t *)BITBAND_PERI(&PD->CR,1)))
#define PD_CR_PD2C            (*((__IO uint32_t *)BITBAND_PERI(&PD->CR,2)))
#define PD_CR_PD3C            (*((__IO uint32_t *)BITBAND_PERI(&PD->CR,3)))
#define PD_CR_PD4C            (*((__IO uint32_t *)BITBAND_PERI(&PD->CR,4)))
#define PD_CR_PD5C            (*((__IO uint32_t *)BITBAND_PERI(&PD->CR,5)))
#define PD_CR_PD6C            (*((__IO uint32_t *)BITBAND_PERI(&PD->CR,6)))
#define PD_FR1_PD0F1          (*((__IO uint32_t *)BITBAND_PERI(&PD->FR1,0)))
#define PD_FR1_PD1F1          (*((__IO uint32_t *)BITBAND_PERI(&PD->FR1,1)))
#define PD_FR1_PD2F1          (*((__IO uint32_t *)BITBAND_PERI(&PD->FR1,2)))
#define PD_FR1_PD3F1          (*((__IO uint32_t *)BITBAND_PERI(&PD->FR1,3)))
#define PD_FR1_PD4F1          (*((__IO uint32_t *)BITBAND_PERI(&PD->FR1,4)))
#define PD_FR1_PD5F1          (*((__IO uint32_t *)BITBAND_PERI(&PD->FR1,5)))
#define PD_FR1_PD6F1          (*((__IO uint32_t *)BITBAND_PERI(&PD->FR1,6)))
#define PD_FR2_PD0F2          (*((__IO uint32_t *)BITBAND_PERI(&PD->FR2,0)))
#define PD_FR2_PD1F2          (*((__IO uint32_t *)BITBAND_PERI(&PD->FR2,1)))
#define PD_FR2_PD4F2          (*((__IO uint32_t *)BITBAND_PERI(&PD->FR2,4)))
#define PD_OD_PD0OD           (*((__IO uint32_t *)BITBAND_PERI(&PD->OD,0)))
#define PD_OD_PD1OD           (*((__IO uint32_t *)BITBAND_PERI(&PD->OD,1)))
#define PD_OD_PD2OD           (*((__IO uint32_t *)BITBAND_PERI(&PD->OD,2)))
#define PD_OD_PD3OD           (*((__IO uint32_t *)BITBAND_PERI(&PD->OD,3)))
#define PD_OD_PD4OD           (*((__IO uint32_t *)BITBAND_PERI(&PD->OD,4)))
#define PD_OD_PD5OD           (*((__IO uint32_t *)BITBAND_PERI(&PD->OD,5)))
#define PD_OD_PD6OD           (*((__IO uint32_t *)BITBAND_PERI(&PD->OD,6)))
#define PD_PUP_PD0UP          (*((__IO uint32_t *)BITBAND_PERI(&PD->PUP,0)))
#define PD_PUP_PD1UP          (*((__IO uint32_t *)BITBAND_PERI(&PD->PUP,1)))
#define PD_PUP_PD2UP          (*((__IO uint32_t *)BITBAND_PERI(&PD->PUP,2)))
#define PD_PUP_PD3UP          (*((__IO uint32_t *)BITBAND_PERI(&PD->PUP,3)))
#define PD_PUP_PD4UP          (*((__IO uint32_t *)BITBAND_PERI(&PD->PUP,4)))
#define PD_PUP_PD5UP          (*((__IO uint32_t *)BITBAND_PERI(&PD->PUP,5)))
#define PD_PUP_PD6UP          (*((__IO uint32_t *)BITBAND_PERI(&PD->PUP,6)))
#define PD_PDN_PD0DN          (*((__IO uint32_t *)BITBAND_PERI(&PD->PDN,0)))
#define PD_PDN_PD1DN          (*((__IO uint32_t *)BITBAND_PERI(&PD->PDN,1)))
#define PD_PDN_PD2DN          (*((__IO uint32_t *)BITBAND_PERI(&PD->PDN,2)))
#define PD_PDN_PD3DN          (*((__IO uint32_t *)BITBAND_PERI(&PD->PDN,3)))
#define PD_PDN_PD4DN          (*((__IO uint32_t *)BITBAND_PERI(&PD->PDN,4)))
#define PD_PDN_PD5DN          (*((__IO uint32_t *)BITBAND_PERI(&PD->PDN,5)))
#define PD_PDN_PD6DN          (*((__IO uint32_t *)BITBAND_PERI(&PD->PDN,6)))
#define PD_IE_PD0IE           (*((__IO uint32_t *)BITBAND_PERI(&PD->IE,0)))
#define PD_IE_PD1IE           (*((__IO uint32_t *)BITBAND_PERI(&PD->IE,1)))
#define PD_IE_PD2IE           (*((__IO uint32_t *)BITBAND_PERI(&PD->IE,2)))
#define PD_IE_PD3IE           (*((__IO uint32_t *)BITBAND_PERI(&PD->IE,3)))
#define PD_IE_PD4IE           (*((__IO uint32_t *)BITBAND_PERI(&PD->IE,4)))
#define PD_IE_PD5IE           (*((__IO uint32_t *)BITBAND_PERI(&PD->IE,5)))
#define PD_IE_PD6IE           (*((__IO uint32_t *)BITBAND_PERI(&PD->IE,6)))

/* General Purpose Input/Output Port (PE) */
#define PE_DATA_PE0           (*((__IO uint32_t *)BITBAND_PERI(&PE->DATA,0)))
#define PE_DATA_PE1           (*((__IO uint32_t *)BITBAND_PERI(&PE->DATA,1)))
#define PE_DATA_PE2           (*((__IO uint32_t *)BITBAND_PERI(&PE->DATA,2)))
#define PE_DATA_PE3           (*((__IO uint32_t *)BITBAND_PERI(&PE->DATA,3)))
#define PE_DATA_PE4           (*((__IO uint32_t *)BITBAND_PERI(&PE->DATA,4)))
#define PE_DATA_PE5           (*((__IO uint32_t *)BITBAND_PERI(&PE->DATA,5)))
#define PE_DATA_PE6           (*((__IO uint32_t *)BITBAND_PERI(&PE->DATA,6)))
#define PE_DATA_PE7           (*((__IO uint32_t *)BITBAND_PERI(&PE->DATA,7)))
#define PE_CR_PE0C            (*((__IO uint32_t *)BITBAND_PERI(&PE->CR,0)))
#define PE_CR_PE1C            (*((__IO uint32_t *)BITBAND_PERI(&PE->CR,1)))
#define PE_CR_PE2C            (*((__IO uint32_t *)BITBAND_PERI(&PE->CR,2)))
#define PE_CR_PE3C            (*((__IO uint32_t *)BITBAND_PERI(&PE->CR,3)))
#define PE_CR_PE4C            (*((__IO uint32_t *)BITBAND_PERI(&PE->CR,4)))
#define PE_CR_PE5C            (*((__IO uint32_t *)BITBAND_PERI(&PE->CR,5)))
#define PE_CR_PE6C            (*((__IO uint32_t *)BITBAND_PERI(&PE->CR,6)))
#define PE_CR_PE7C            (*((__IO uint32_t *)BITBAND_PERI(&PE->CR,7)))
#define PE_FR1_PE0F1          (*((__IO uint32_t *)BITBAND_PERI(&PE->FR1,0)))
#define PE_FR1_PE1F1          (*((__IO uint32_t *)BITBAND_PERI(&PE->FR1,1)))
#define PE_FR1_PE2F1          (*((__IO uint32_t *)BITBAND_PERI(&PE->FR1,2)))
#define PE_FR1_PE3F1          (*((__IO uint32_t *)BITBAND_PERI(&PE->FR1,3)))
#define PE_FR1_PE4F1          (*((__IO uint32_t *)BITBAND_PERI(&PE->FR1,4)))
#define PE_FR1_PE5F1          (*((__IO uint32_t *)BITBAND_PERI(&PE->FR1,5)))
#define PE_FR1_PE6F1          (*((__IO uint32_t *)BITBAND_PERI(&PE->FR1,6)))
#define PE_FR1_PE7F1          (*((__IO uint32_t *)BITBAND_PERI(&PE->FR1,7)))
#define PE_FR2_PE2F2          (*((__IO uint32_t *)BITBAND_PERI(&PE->FR2,2)))
#define PE_FR2_PE4F2          (*((__IO uint32_t *)BITBAND_PERI(&PE->FR2,4)))
#define PE_FR2_PE6F2          (*((__IO uint32_t *)BITBAND_PERI(&PE->FR2,6)))
#define PE_FR2_PE7F2          (*((__IO uint32_t *)BITBAND_PERI(&PE->FR2,7)))
#define PE_OD_PE0OD           (*((__IO uint32_t *)BITBAND_PERI(&PE->OD,0)))
#define PE_OD_PE1OD           (*((__IO uint32_t *)BITBAND_PERI(&PE->OD,1)))
#define PE_OD_PE2OD           (*((__IO uint32_t *)BITBAND_PERI(&PE->OD,2)))
#define PE_OD_PE3OD           (*((__IO uint32_t *)BITBAND_PERI(&PE->OD,3)))
#define PE_OD_PE4OD           (*((__IO uint32_t *)BITBAND_PERI(&PE->OD,4)))
#define PE_OD_PE5OD           (*((__IO uint32_t *)BITBAND_PERI(&PE->OD,5)))
#define PE_OD_PE6OD           (*((__IO uint32_t *)BITBAND_PERI(&PE->OD,6)))
#define PE_OD_PE7OD           (*((__IO uint32_t *)BITBAND_PERI(&PE->OD,7)))
#define PE_PUP_PE0UP          (*((__IO uint32_t *)BITBAND_PERI(&PE->PUP,0)))
#define PE_PUP_PE1UP          (*((__IO uint32_t *)BITBAND_PERI(&PE->PUP,1)))
#define PE_PUP_PE2UP          (*((__IO uint32_t *)BITBAND_PERI(&PE->PUP,2)))
#define PE_PUP_PE3UP          (*((__IO uint32_t *)BITBAND_PERI(&PE->PUP,3)))
#define PE_PUP_PE4UP          (*((__IO uint32_t *)BITBAND_PERI(&PE->PUP,4)))
#define PE_PUP_PE5UP          (*((__IO uint32_t *)BITBAND_PERI(&PE->PUP,5)))
#define PE_PUP_PE6UP          (*((__IO uint32_t *)BITBAND_PERI(&PE->PUP,6)))
#define PE_PUP_PE7UP          (*((__IO uint32_t *)BITBAND_PERI(&PE->PUP,7)))
#define PE_PDN_PE0DN          (*((__IO uint32_t *)BITBAND_PERI(&PE->PDN,0)))
#define PE_PDN_PE1DN          (*((__IO uint32_t *)BITBAND_PERI(&PE->PDN,1)))
#define PE_PDN_PE2DN          (*((__IO uint32_t *)BITBAND_PERI(&PE->PDN,2)))
#define PE_PDN_PE3DN          (*((__IO uint32_t *)BITBAND_PERI(&PE->PDN,3)))
#define PE_PDN_PE4DN          (*((__IO uint32_t *)BITBAND_PERI(&PE->PDN,4)))
#define PE_PDN_PE5DN          (*((__IO uint32_t *)BITBAND_PERI(&PE->PDN,5)))
#define PE_PDN_PE6DN          (*((__IO uint32_t *)BITBAND_PERI(&PE->PDN,6)))
#define PE_PDN_PE7DN          (*((__IO uint32_t *)BITBAND_PERI(&PE->PDN,7)))
#define PE_IE_PE0IE           (*((__IO uint32_t *)BITBAND_PERI(&PE->IE,0)))
#define PE_IE_PE1IE           (*((__IO uint32_t *)BITBAND_PERI(&PE->IE,1)))
#define PE_IE_PE2IE           (*((__IO uint32_t *)BITBAND_PERI(&PE->IE,2)))
#define PE_IE_PE3IE           (*((__IO uint32_t *)BITBAND_PERI(&PE->IE,3)))
#define PE_IE_PE4IE           (*((__IO uint32_t *)BITBAND_PERI(&PE->IE,4)))
#define PE_IE_PE5IE           (*((__IO uint32_t *)BITBAND_PERI(&PE->IE,5)))
#define PE_IE_PE6IE           (*((__IO uint32_t *)BITBAND_PERI(&PE->IE,6)))
#define PE_IE_PE7IE           (*((__IO uint32_t *)BITBAND_PERI(&PE->IE,7)))

/* General Purpose Input/Output Port (PF) */
#define PF_DATA_PF0           (*((__IO uint32_t *)BITBAND_PERI(&PF->DATA,0)))
#define PF_DATA_PF1           (*((__IO uint32_t *)BITBAND_PERI(&PF->DATA,1)))
#define PF_DATA_PF2           (*((__IO uint32_t *)BITBAND_PERI(&PF->DATA,2)))
#define PF_DATA_PF3           (*((__IO uint32_t *)BITBAND_PERI(&PF->DATA,3)))
#define PF_DATA_PF4           (*((__IO uint32_t *)BITBAND_PERI(&PF->DATA,4)))
#define PF_CR_PF0C            (*((__IO uint32_t *)BITBAND_PERI(&PF->CR,0)))
#define PF_CR_PF1C            (*((__IO uint32_t *)BITBAND_PERI(&PF->CR,1)))
#define PF_CR_PF2C            (*((__IO uint32_t *)BITBAND_PERI(&PF->CR,2)))
#define PF_CR_PF3C            (*((__IO uint32_t *)BITBAND_PERI(&PF->CR,3)))
#define PF_CR_PF4C            (*((__IO uint32_t *)BITBAND_PERI(&PF->CR,4)))
#define PF_FR1_PF0F1          (*((__IO uint32_t *)BITBAND_PERI(&PF->FR1,0)))
#define PF_FR1_PF1F1          (*((__IO uint32_t *)BITBAND_PERI(&PF->FR1,1)))
#define PF_FR1_PF2F1          (*((__IO uint32_t *)BITBAND_PERI(&PF->FR1,2)))
#define PF_FR1_PF3F1          (*((__IO uint32_t *)BITBAND_PERI(&PF->FR1,3)))
#define PF_FR1_PF4F1          (*((__IO uint32_t *)BITBAND_PERI(&PF->FR1,4)))
#define PF_FR2_PF2F2          (*((__IO uint32_t *)BITBAND_PERI(&PF->FR2,2)))
#define PF_FR2_PF3F2          (*((__IO uint32_t *)BITBAND_PERI(&PF->FR2,3)))
#define PF_FR2_PF4F2          (*((__IO uint32_t *)BITBAND_PERI(&PF->FR2,4)))
#define PF_FR2_PF2F3          (*((__IO uint32_t *)BITBAND_PERI(&PF->FR3,2)))
#define PF_OD_PF0OD           (*((__IO uint32_t *)BITBAND_PERI(&PF->OD,0)))
#define PF_OD_PF1OD           (*((__IO uint32_t *)BITBAND_PERI(&PF->OD,1)))
#define PF_OD_PF2OD           (*((__IO uint32_t *)BITBAND_PERI(&PF->OD,2)))
#define PF_OD_PF3OD           (*((__IO uint32_t *)BITBAND_PERI(&PF->OD,3)))
#define PF_OD_PF4OD           (*((__IO uint32_t *)BITBAND_PERI(&PF->OD,4)))
#define PF_PUP_PF0UP          (*((__IO uint32_t *)BITBAND_PERI(&PF->PUP,0)))
#define PF_PUP_PF1UP          (*((__IO uint32_t *)BITBAND_PERI(&PF->PUP,1)))
#define PF_PUP_PF2UP          (*((__IO uint32_t *)BITBAND_PERI(&PF->PUP,2)))
#define PF_PUP_PF3UP          (*((__IO uint32_t *)BITBAND_PERI(&PF->PUP,3)))
#define PF_PUP_PF4UP          (*((__IO uint32_t *)BITBAND_PERI(&PF->PUP,4)))
#define PF_PDN_PF0DN          (*((__IO uint32_t *)BITBAND_PERI(&PF->PDN,0)))
#define PF_PDN_PF1DN          (*((__IO uint32_t *)BITBAND_PERI(&PF->PDN,1)))
#define PF_PDN_PF2DN          (*((__IO uint32_t *)BITBAND_PERI(&PF->PDN,2)))
#define PF_PDN_PF3DN          (*((__IO uint32_t *)BITBAND_PERI(&PF->PDN,3)))
#define PF_PDN_PF4DN          (*((__IO uint32_t *)BITBAND_PERI(&PF->PDN,4)))
#define PF_IE_PF0IE           (*((__IO uint32_t *)BITBAND_PERI(&PF->IE,0)))
#define PF_IE_PF1IE           (*((__IO uint32_t *)BITBAND_PERI(&PF->IE,1)))
#define PF_IE_PF2IE           (*((__IO uint32_t *)BITBAND_PERI(&PF->IE,2)))
#define PF_IE_PF3IE           (*((__IO uint32_t *)BITBAND_PERI(&PF->IE,3)))
#define PF_IE_PF4IE           (*((__IO uint32_t *)BITBAND_PERI(&PF->IE,4)))

/* General Purpose Input/Output Port (PG) */
#define PG_DATA_PG0           (*((__IO uint32_t *)BITBAND_PERI(&PG->DATA,0)))
#define PG_DATA_PG1           (*((__IO uint32_t *)BITBAND_PERI(&PG->DATA,1)))
#define PG_DATA_PG2           (*((__IO uint32_t *)BITBAND_PERI(&PG->DATA,2)))
#define PG_DATA_PG3           (*((__IO uint32_t *)BITBAND_PERI(&PG->DATA,3)))
#define PG_DATA_PG4           (*((__IO uint32_t *)BITBAND_PERI(&PG->DATA,4)))
#define PG_DATA_PG5           (*((__IO uint32_t *)BITBAND_PERI(&PG->DATA,5)))
#define PG_DATA_PG6           (*((__IO uint32_t *)BITBAND_PERI(&PG->DATA,6)))
#define PG_DATA_PG7           (*((__IO uint32_t *)BITBAND_PERI(&PG->DATA,7)))
#define PG_CR_PG0C            (*((__IO uint32_t *)BITBAND_PERI(&PG->CR,0)))
#define PG_CR_PG1C            (*((__IO uint32_t *)BITBAND_PERI(&PG->CR,1)))
#define PG_CR_PG2C            (*((__IO uint32_t *)BITBAND_PERI(&PG->CR,2)))
#define PG_CR_PG3C            (*((__IO uint32_t *)BITBAND_PERI(&PG->CR,3)))
#define PG_CR_PG4C            (*((__IO uint32_t *)BITBAND_PERI(&PG->CR,4)))
#define PG_CR_PG5C            (*((__IO uint32_t *)BITBAND_PERI(&PG->CR,5)))
#define PG_CR_PG6C            (*((__IO uint32_t *)BITBAND_PERI(&PG->CR,6)))
#define PG_CR_PG7C            (*((__IO uint32_t *)BITBAND_PERI(&PG->CR,7)))
#define PG_FR1_PG0F1          (*((__IO uint32_t *)BITBAND_PERI(&PG->FR1,0)))
#define PG_FR1_PG1F1          (*((__IO uint32_t *)BITBAND_PERI(&PG->FR1,1)))
#define PG_FR1_PG2F1          (*((__IO uint32_t *)BITBAND_PERI(&PG->FR1,2)))
#define PG_FR1_PG3F1          (*((__IO uint32_t *)BITBAND_PERI(&PG->FR1,3)))
#define PG_FR1_PG4F1          (*((__IO uint32_t *)BITBAND_PERI(&PG->FR1,4)))
#define PG_FR1_PG5F1          (*((__IO uint32_t *)BITBAND_PERI(&PG->FR1,5)))
#define PG_FR1_PG6F1          (*((__IO uint32_t *)BITBAND_PERI(&PG->FR1,6)))
#define PG_FR1_PG7F1          (*((__IO uint32_t *)BITBAND_PERI(&PG->FR1,7)))
#define PG_OD_PG0OD           (*((__IO uint32_t *)BITBAND_PERI(&PG->OD,0)))
#define PG_OD_PG1OD           (*((__IO uint32_t *)BITBAND_PERI(&PG->OD,1)))
#define PG_OD_PG2OD           (*((__IO uint32_t *)BITBAND_PERI(&PG->OD,2)))
#define PG_OD_PG3OD           (*((__IO uint32_t *)BITBAND_PERI(&PG->OD,3)))
#define PG_OD_PG4OD           (*((__IO uint32_t *)BITBAND_PERI(&PG->OD,4)))
#define PG_OD_PG5OD           (*((__IO uint32_t *)BITBAND_PERI(&PG->OD,5)))
#define PG_OD_PG6OD           (*((__IO uint32_t *)BITBAND_PERI(&PG->OD,6)))
#define PG_OD_PG7OD           (*((__IO uint32_t *)BITBAND_PERI(&PG->OD,7)))
#define PG_PUP_PG0UP          (*((__IO uint32_t *)BITBAND_PERI(&PG->PUP,0)))
#define PG_PUP_PG1UP          (*((__IO uint32_t *)BITBAND_PERI(&PG->PUP,1)))
#define PG_PUP_PG2UP          (*((__IO uint32_t *)BITBAND_PERI(&PG->PUP,2)))
#define PG_PUP_PG3UP          (*((__IO uint32_t *)BITBAND_PERI(&PG->PUP,3)))
#define PG_PUP_PG4UP          (*((__IO uint32_t *)BITBAND_PERI(&PG->PUP,4)))
#define PG_PUP_PG5UP          (*((__IO uint32_t *)BITBAND_PERI(&PG->PUP,5)))
#define PG_PUP_PG6UP          (*((__IO uint32_t *)BITBAND_PERI(&PG->PUP,6)))
#define PG_PUP_PG7UP          (*((__IO uint32_t *)BITBAND_PERI(&PG->PUP,7)))
#define PG_PDN_PG0DN          (*((__IO uint32_t *)BITBAND_PERI(&PG->PDN,0)))
#define PG_PDN_PG1DN          (*((__IO uint32_t *)BITBAND_PERI(&PG->PDN,1)))
#define PG_PDN_PG2DN          (*((__IO uint32_t *)BITBAND_PERI(&PG->PDN,2)))
#define PG_PDN_PG3DN          (*((__IO uint32_t *)BITBAND_PERI(&PG->PDN,3)))
#define PG_PDN_PG4DN          (*((__IO uint32_t *)BITBAND_PERI(&PG->PDN,4)))
#define PG_PDN_PG5DN          (*((__IO uint32_t *)BITBAND_PERI(&PG->PDN,5)))
#define PG_PDN_PG6DN          (*((__IO uint32_t *)BITBAND_PERI(&PG->PDN,6)))
#define PG_PDN_PG7DN          (*((__IO uint32_t *)BITBAND_PERI(&PG->PDN,7)))
#define PG_IE_PG0IE           (*((__IO uint32_t *)BITBAND_PERI(&PG->IE,0)))
#define PG_IE_PG1IE           (*((__IO uint32_t *)BITBAND_PERI(&PG->IE,1)))
#define PG_IE_PG2IE           (*((__IO uint32_t *)BITBAND_PERI(&PG->IE,2)))
#define PG_IE_PG3IE           (*((__IO uint32_t *)BITBAND_PERI(&PG->IE,3)))
#define PG_IE_PG4IE           (*((__IO uint32_t *)BITBAND_PERI(&PG->IE,4)))
#define PG_IE_PG5IE           (*((__IO uint32_t *)BITBAND_PERI(&PG->IE,5)))
#define PG_IE_PG6IE           (*((__IO uint32_t *)BITBAND_PERI(&PG->IE,6)))
#define PG_IE_PG7IE           (*((__IO uint32_t *)BITBAND_PERI(&PG->IE,7)))

/* General Purpose Input/Output Port (PH) */
#define PH_DATA_PH0           (*((__IO uint32_t *)BITBAND_PERI(&PH->DATA,0)))
#define PH_DATA_PH1           (*((__IO uint32_t *)BITBAND_PERI(&PH->DATA,1)))
#define PH_DATA_PH2           (*((__IO uint32_t *)BITBAND_PERI(&PH->DATA,2)))
#define PH_DATA_PH3           (*((__IO uint32_t *)BITBAND_PERI(&PH->DATA,3)))
#define PH_DATA_PH4           (*((__IO uint32_t *)BITBAND_PERI(&PH->DATA,4)))
#define PH_DATA_PH5           (*((__IO uint32_t *)BITBAND_PERI(&PH->DATA,5)))
#define PH_DATA_PH6           (*((__IO uint32_t *)BITBAND_PERI(&PH->DATA,6)))
#define PH_DATA_PH7           (*((__IO uint32_t *)BITBAND_PERI(&PH->DATA,7)))
#define PH_CR_PH0C            (*((__IO uint32_t *)BITBAND_PERI(&PH->CR,0)))
#define PH_CR_PH1C            (*((__IO uint32_t *)BITBAND_PERI(&PH->CR,1)))
#define PH_CR_PH2C            (*((__IO uint32_t *)BITBAND_PERI(&PH->CR,2)))
#define PH_CR_PH3C            (*((__IO uint32_t *)BITBAND_PERI(&PH->CR,3)))
#define PH_CR_PH4C            (*((__IO uint32_t *)BITBAND_PERI(&PH->CR,4)))
#define PH_CR_PH5C            (*((__IO uint32_t *)BITBAND_PERI(&PH->CR,5)))
#define PH_CR_PH6C            (*((__IO uint32_t *)BITBAND_PERI(&PH->CR,6)))
#define PH_CR_PH7C            (*((__IO uint32_t *)BITBAND_PERI(&PH->CR,7)))
#define PH_FR1_PH0F1          (*((__IO uint32_t *)BITBAND_PERI(&PH->FR1,0)))
#define PH_FR1_PH1F1          (*((__IO uint32_t *)BITBAND_PERI(&PH->FR1,1)))
#define PH_FR1_PH2F1          (*((__IO uint32_t *)BITBAND_PERI(&PH->FR1,2)))
#define PH_OD_PH0OD           (*((__IO uint32_t *)BITBAND_PERI(&PH->OD,0)))
#define PH_OD_PH1OD           (*((__IO uint32_t *)BITBAND_PERI(&PH->OD,1)))
#define PH_OD_PH2OD           (*((__IO uint32_t *)BITBAND_PERI(&PH->OD,2)))
#define PH_OD_PH3OD           (*((__IO uint32_t *)BITBAND_PERI(&PH->OD,3)))
#define PH_OD_PH4OD           (*((__IO uint32_t *)BITBAND_PERI(&PH->OD,4)))
#define PH_OD_PH5OD           (*((__IO uint32_t *)BITBAND_PERI(&PH->OD,5)))
#define PH_OD_PH6OD           (*((__IO uint32_t *)BITBAND_PERI(&PH->OD,6)))
#define PH_OD_PH7OD           (*((__IO uint32_t *)BITBAND_PERI(&PH->OD,7)))
#define PH_PUP_PH0UP          (*((__IO uint32_t *)BITBAND_PERI(&PH->PUP,0)))
#define PH_PUP_PH1UP          (*((__IO uint32_t *)BITBAND_PERI(&PH->PUP,1)))
#define PH_PUP_PH2UP          (*((__IO uint32_t *)BITBAND_PERI(&PH->PUP,2)))
#define PH_PUP_PH3UP          (*((__IO uint32_t *)BITBAND_PERI(&PH->PUP,3)))
#define PH_PUP_PH4UP          (*((__IO uint32_t *)BITBAND_PERI(&PH->PUP,4)))
#define PH_PUP_PH5UP          (*((__IO uint32_t *)BITBAND_PERI(&PH->PUP,5)))
#define PH_PUP_PH6UP          (*((__IO uint32_t *)BITBAND_PERI(&PH->PUP,6)))
#define PH_PUP_PH7UP          (*((__IO uint32_t *)BITBAND_PERI(&PH->PUP,7)))
#define PH_PDN_PH0DN          (*((__IO uint32_t *)BITBAND_PERI(&PH->PDN,0)))
#define PH_PDN_PH1DN          (*((__IO uint32_t *)BITBAND_PERI(&PH->PDN,1)))
#define PH_PDN_PH2DN          (*((__IO uint32_t *)BITBAND_PERI(&PH->PDN,2)))
#define PH_PDN_PH3DN          (*((__IO uint32_t *)BITBAND_PERI(&PH->PDN,3)))
#define PH_PDN_PH4DN          (*((__IO uint32_t *)BITBAND_PERI(&PH->PDN,4)))
#define PH_PDN_PH5DN          (*((__IO uint32_t *)BITBAND_PERI(&PH->PDN,5)))
#define PH_PDN_PH6DN          (*((__IO uint32_t *)BITBAND_PERI(&PH->PDN,6)))
#define PH_PDN_PH7DN          (*((__IO uint32_t *)BITBAND_PERI(&PH->PDN,7)))
#define PH_IE_PH0IE           (*((__IO uint32_t *)BITBAND_PERI(&PH->IE,0)))
#define PH_IE_PH1IE           (*((__IO uint32_t *)BITBAND_PERI(&PH->IE,1)))
#define PH_IE_PH2IE           (*((__IO uint32_t *)BITBAND_PERI(&PH->IE,2)))
#define PH_IE_PH3IE           (*((__IO uint32_t *)BITBAND_PERI(&PH->IE,3)))
#define PH_IE_PH4IE           (*((__IO uint32_t *)BITBAND_PERI(&PH->IE,4)))
#define PH_IE_PH5IE           (*((__IO uint32_t *)BITBAND_PERI(&PH->IE,5)))
#define PH_IE_PH6IE           (*((__IO uint32_t *)BITBAND_PERI(&PH->IE,6)))
#define PH_IE_PH7IE           (*((__IO uint32_t *)BITBAND_PERI(&PH->IE,7)))

/* General Purpose Input/Output Port (PI) */
#define PI_DATA_PI0           (*((__IO uint32_t *)BITBAND_PERI(&PI->DATA,0)))
#define PI_DATA_PI1           (*((__IO uint32_t *)BITBAND_PERI(&PI->DATA,1)))
#define PI_DATA_PI2           (*((__IO uint32_t *)BITBAND_PERI(&PI->DATA,2)))
#define PI_DATA_PI3           (*((__IO uint32_t *)BITBAND_PERI(&PI->DATA,3)))
#define PI_CR_PI0C            (*((__IO uint32_t *)BITBAND_PERI(&PI->CR,0)))
#define PI_CR_PI1C            (*((__IO uint32_t *)BITBAND_PERI(&PI->CR,1)))
#define PI_CR_PI2C            (*((__IO uint32_t *)BITBAND_PERI(&PI->CR,2)))
#define PI_CR_PI3C            (*((__IO uint32_t *)BITBAND_PERI(&PI->CR,3)))
#define PI_OD_PI0OD           (*((__IO uint32_t *)BITBAND_PERI(&PI->OD,0)))
#define PI_OD_PI1OD           (*((__IO uint32_t *)BITBAND_PERI(&PI->OD,1)))
#define PI_OD_PI2OD           (*((__IO uint32_t *)BITBAND_PERI(&PI->OD,2)))
#define PI_OD_PI3OD           (*((__IO uint32_t *)BITBAND_PERI(&PI->OD,3)))
#define PI_PUP_PI0UP          (*((__IO uint32_t *)BITBAND_PERI(&PI->PUP,0)))
#define PI_PUP_PI1UP          (*((__IO uint32_t *)BITBAND_PERI(&PI->PUP,1)))
#define PI_PUP_PI2UP          (*((__IO uint32_t *)BITBAND_PERI(&PI->PUP,2)))
#define PI_PUP_PI3UP          (*((__IO uint32_t *)BITBAND_PERI(&PI->PUP,3)))
#define PI_PDN_PI0DN          (*((__IO uint32_t *)BITBAND_PERI(&PI->PDN,0)))
#define PI_PDN_PI1DN          (*((__IO uint32_t *)BITBAND_PERI(&PI->PDN,1)))
#define PI_PDN_PI2DN          (*((__IO uint32_t *)BITBAND_PERI(&PI->PDN,2)))
#define PI_PDN_PI3DN          (*((__IO uint32_t *)BITBAND_PERI(&PI->PDN,3)))
#define PI_IE_PI0IE           (*((__IO uint32_t *)BITBAND_PERI(&PI->IE,0)))
#define PI_IE_PI1IE           (*((__IO uint32_t *)BITBAND_PERI(&PI->IE,1)))
#define PI_IE_PI2IE           (*((__IO uint32_t *)BITBAND_PERI(&PI->IE,2)))
#define PI_IE_PI3IE           (*((__IO uint32_t *)BITBAND_PERI(&PI->IE,3)))

/* General Purpose Input/Output Port (PJ) */
#define PJ_DATA_PJ0           (*((__IO uint32_t *)BITBAND_PERI(&PJ->DATA,0)))
#define PJ_DATA_PJ1           (*((__IO uint32_t *)BITBAND_PERI(&PJ->DATA,1)))
#define PJ_DATA_PJ2           (*((__IO uint32_t *)BITBAND_PERI(&PJ->DATA,2)))
#define PJ_DATA_PJ3           (*((__IO uint32_t *)BITBAND_PERI(&PJ->DATA,3)))
#define PJ_DATA_PJ4           (*((__IO uint32_t *)BITBAND_PERI(&PJ->DATA,4)))
#define PJ_DATA_PJ5           (*((__IO uint32_t *)BITBAND_PERI(&PJ->DATA,5)))
#define PJ_DATA_PJ6           (*((__IO uint32_t *)BITBAND_PERI(&PJ->DATA,6)))
#define PJ_DATA_PJ7           (*((__IO uint32_t *)BITBAND_PERI(&PJ->DATA,7)))
#define PJ_CR_PJ0C            (*((__IO uint32_t *)BITBAND_PERI(&PJ->CR,0)))
#define PJ_CR_PJ1C            (*((__IO uint32_t *)BITBAND_PERI(&PJ->CR,1)))
#define PJ_CR_PJ2C            (*((__IO uint32_t *)BITBAND_PERI(&PJ->CR,2)))
#define PJ_CR_PJ3C            (*((__IO uint32_t *)BITBAND_PERI(&PJ->CR,3)))
#define PJ_CR_PJ4C            (*((__IO uint32_t *)BITBAND_PERI(&PJ->CR,4)))
#define PJ_CR_PJ5C            (*((__IO uint32_t *)BITBAND_PERI(&PJ->CR,5)))
#define PJ_CR_PJ6C            (*((__IO uint32_t *)BITBAND_PERI(&PJ->CR,6)))
#define PJ_CR_PJ7C            (*((__IO uint32_t *)BITBAND_PERI(&PJ->CR,7)))
#define PJ_FR1_PJ6F1          (*((__IO uint32_t *)BITBAND_PERI(&PJ->FR1,6)))
#define PJ_FR1_PJ7F1          (*((__IO uint32_t *)BITBAND_PERI(&PJ->FR1,7)))
#define PJ_OD_PJ0OD           (*((__IO uint32_t *)BITBAND_PERI(&PJ->OD,0)))
#define PJ_OD_PJ1OD           (*((__IO uint32_t *)BITBAND_PERI(&PJ->OD,1)))
#define PJ_OD_PJ2OD           (*((__IO uint32_t *)BITBAND_PERI(&PJ->OD,2)))
#define PJ_OD_PJ3OD           (*((__IO uint32_t *)BITBAND_PERI(&PJ->OD,3)))
#define PJ_OD_PJ4OD           (*((__IO uint32_t *)BITBAND_PERI(&PJ->OD,4)))
#define PJ_OD_PJ5OD           (*((__IO uint32_t *)BITBAND_PERI(&PJ->OD,5)))
#define PJ_OD_PJ6OD           (*((__IO uint32_t *)BITBAND_PERI(&PJ->OD,6)))
#define PJ_OD_PJ7OD           (*((__IO uint32_t *)BITBAND_PERI(&PJ->OD,7)))
#define PJ_PUP_PJ0UP          (*((__IO uint32_t *)BITBAND_PERI(&PJ->PUP,0)))
#define PJ_PUP_PJ1UP          (*((__IO uint32_t *)BITBAND_PERI(&PJ->PUP,1)))
#define PJ_PUP_PJ2UP          (*((__IO uint32_t *)BITBAND_PERI(&PJ->PUP,2)))
#define PJ_PUP_PJ3UP          (*((__IO uint32_t *)BITBAND_PERI(&PJ->PUP,3)))
#define PJ_PUP_PJ4UP          (*((__IO uint32_t *)BITBAND_PERI(&PJ->PUP,4)))
#define PJ_PUP_PJ5UP          (*((__IO uint32_t *)BITBAND_PERI(&PJ->PUP,5)))
#define PJ_PUP_PJ6UP          (*((__IO uint32_t *)BITBAND_PERI(&PJ->PUP,6)))
#define PJ_PUP_PJ7UP          (*((__IO uint32_t *)BITBAND_PERI(&PJ->PUP,7)))
#define PJ_PDN_PJ0DN          (*((__IO uint32_t *)BITBAND_PERI(&PJ->PDN,0)))
#define PJ_PDN_PJ1DN          (*((__IO uint32_t *)BITBAND_PERI(&PJ->PDN,1)))
#define PJ_PDN_PJ2DN          (*((__IO uint32_t *)BITBAND_PERI(&PJ->PDN,2)))
#define PJ_PDN_PJ3DN          (*((__IO uint32_t *)BITBAND_PERI(&PJ->PDN,3)))
#define PJ_PDN_PJ4DN          (*((__IO uint32_t *)BITBAND_PERI(&PJ->PDN,4)))
#define PJ_PDN_PJ5DN          (*((__IO uint32_t *)BITBAND_PERI(&PJ->PDN,5)))
#define PJ_PDN_PJ6DN          (*((__IO uint32_t *)BITBAND_PERI(&PJ->PDN,6)))
#define PJ_PDN_PJ7DN          (*((__IO uint32_t *)BITBAND_PERI(&PJ->PDN,7)))
#define PJ_IE_PJ0IE           (*((__IO uint32_t *)BITBAND_PERI(&PJ->IE,0)))
#define PJ_IE_PJ1IE           (*((__IO uint32_t *)BITBAND_PERI(&PJ->IE,1)))
#define PJ_IE_PJ2IE           (*((__IO uint32_t *)BITBAND_PERI(&PJ->IE,2)))
#define PJ_IE_PJ3IE           (*((__IO uint32_t *)BITBAND_PERI(&PJ->IE,3)))
#define PJ_IE_PJ4IE           (*((__IO uint32_t *)BITBAND_PERI(&PJ->IE,4)))
#define PJ_IE_PJ5IE           (*((__IO uint32_t *)BITBAND_PERI(&PJ->IE,5)))
#define PJ_IE_PJ6IE           (*((__IO uint32_t *)BITBAND_PERI(&PJ->IE,6)))
#define PJ_IE_PJ7IE           (*((__IO uint32_t *)BITBAND_PERI(&PJ->IE,7)))

/* General Purpose Input/Output Port (PK) */
#define PK_DATA_PK0           (*((__IO uint32_t *)BITBAND_PERI(&PK->DATA,0)))
#define PK_DATA_PK1           (*((__IO uint32_t *)BITBAND_PERI(&PK->DATA,1)))
#define PK_CR_PK0C            (*((__IO uint32_t *)BITBAND_PERI(&PK->CR,0)))
#define PK_CR_PK1C            (*((__IO uint32_t *)BITBAND_PERI(&PK->CR,1)))
#define PK_FR1_PK0F1          (*((__IO uint32_t *)BITBAND_PERI(&PK->FR1,0)))
#define PK_FR1_PK1F1          (*((__IO uint32_t *)BITBAND_PERI(&PK->FR1,1)))
#define PK_OD_PK0OD           (*((__IO uint32_t *)BITBAND_PERI(&PK->OD,0)))
#define PK_OD_PK1OD           (*((__IO uint32_t *)BITBAND_PERI(&PK->OD,1)))
#define PK_PUP_PK0UP          (*((__IO uint32_t *)BITBAND_PERI(&PK->PUP,0)))
#define PK_PUP_PK1UP          (*((__IO uint32_t *)BITBAND_PERI(&PK->PUP,1)))
#define PK_PDN_PK0DN          (*((__IO uint32_t *)BITBAND_PERI(&PK->PDN,0)))
#define PK_PDN_PK1DN          (*((__IO uint32_t *)BITBAND_PERI(&PK->PDN,1)))
#define PK_IE_PK0IE           (*((__IO uint32_t *)BITBAND_PERI(&PK->IE,0)))
#define PK_IE_PK1IE           (*((__IO uint32_t *)BITBAND_PERI(&PK->IE,1)))

/* General Purpose Input Port (PL) */
#define PL_DATA_PL0           (*((__I  uint32_t *)BITBAND_PERI(&PL->DATA,0)))
#define PL_DATA_PL1           (*((__I  uint32_t *)BITBAND_PERI(&PL->DATA,1)))
#define PL_FR1_PL0F1          (*((__IO uint32_t *)BITBAND_PERI(&PL->FR1,0)))
#define PL_FR1_PL1F1          (*((__IO uint32_t *)BITBAND_PERI(&PL->FR1,1)))
#define PL_IE_PL0IE           (*((__IO uint32_t *)BITBAND_PERI(&PL->IE,0)))
#define PL_IE_PL1IE           (*((__IO uint32_t *)BITBAND_PERI(&PL->IE,1)))

/* 16-bit Timer/Event Counter (TB) */
#define TB0_EN_TB0EN          (*((__IO uint32_t *)BITBAND_PERI(&TB0->EN,7)))
#define TB0_RUN_TB0RUN        (*((__IO uint32_t *)BITBAND_PERI(&TB0->RUN,0)))
#define TB0_RUN_TB0PRUN       (*((__IO uint32_t *)BITBAND_PERI(&TB0->RUN,2)))
#define TB0_CR_CSSEL0         (*((__IO uint32_t *)BITBAND_PERI(&TB0->CR,0)))
#define TB0_CR_TRGSEL0        (*((__IO uint32_t *)BITBAND_PERI(&TB0->CR,1)))
#define TB0_CR_I2TB0          (*((__IO uint32_t *)BITBAND_PERI(&TB0->CR,3)))
#define TB0_CR_TB0WBF         (*((__IO uint32_t *)BITBAND_PERI(&TB0->CR,7)))
#define TB0_MOD_TB0CLK0       (*((__IO uint32_t *)BITBAND_PERI(&TB0->MOD,0)))
#define TB0_MOD_TB0CLK1       (*((__IO uint32_t *)BITBAND_PERI(&TB0->MOD,1)))
#define TB0_MOD_TB0CLE        (*((__IO uint32_t *)BITBAND_PERI(&TB0->MOD,2)))
#define TB0_MOD_TB0CPM0       (*((__IO uint32_t *)BITBAND_PERI(&TB0->MOD,3)))
#define TB0_MOD_TB0CPM1       (*((__IO uint32_t *)BITBAND_PERI(&TB0->MOD,4)))
#define TB0_MOD_TB0CP0        (*((__O  uint32_t *)BITBAND_PERI(&TB0->MOD,5)))
#define TB0_MOD_TB0RSWR       (*((__IO uint32_t *)BITBAND_PERI(&TB0->MOD,6)))
#define TB0_FFCR_TB0FF0C0     (*((__IO uint32_t *)BITBAND_PERI(&TB0->FFCR,0)))
#define TB0_FFCR_TB0FF0C1     (*((__IO uint32_t *)BITBAND_PERI(&TB0->FFCR,1)))
#define TB0_FFCR_TB0E0T1      (*((__IO uint32_t *)BITBAND_PERI(&TB0->FFCR,2)))
#define TB0_FFCR_TB0E1T1      (*((__IO uint32_t *)BITBAND_PERI(&TB0->FFCR,3)))
#define TB0_FFCR_TB0C0T1      (*((__IO uint32_t *)BITBAND_PERI(&TB0->FFCR,4)))
#define TB0_FFCR_TB0C1T1      (*((__IO uint32_t *)BITBAND_PERI(&TB0->FFCR,5)))
#define TB0_ST_INTTB00        (*((__I  uint32_t *)BITBAND_PERI(&TB0->ST,0)))
#define TB0_ST_INTTB01        (*((__I  uint32_t *)BITBAND_PERI(&TB0->ST,1)))
#define TB0_ST_INTTBOF0       (*((__I  uint32_t *)BITBAND_PERI(&TB0->ST,2)))
#define TB0_IM_TBIM00         (*((__IO uint32_t *)BITBAND_PERI(&TB0->IM,0)))
#define TB0_IM_TBIM01         (*((__IO uint32_t *)BITBAND_PERI(&TB0->IM,1)))
#define TB0_IM_TBIMOF0        (*((__IO uint32_t *)BITBAND_PERI(&TB0->IM,2)))

#define TB1_EN_TB1EN          (*((__IO uint32_t *)BITBAND_PERI(&TB1->EN,7)))
#define TB1_RUN_TB1RUN        (*((__IO uint32_t *)BITBAND_PERI(&TB1->RUN,0)))
#define TB1_RUN_TB1PRUN       (*((__IO uint32_t *)BITBAND_PERI(&TB1->RUN,2)))
#define TB1_CR_CSSEL1         (*((__IO uint32_t *)BITBAND_PERI(&TB1->CR,0)))
#define TB1_CR_TRGSEL1        (*((__IO uint32_t *)BITBAND_PERI(&TB1->CR,1)))
#define TB1_CR_I2TB1          (*((__IO uint32_t *)BITBAND_PERI(&TB1->CR,3)))
#define TB1_CR_TB1WBF         (*((__IO uint32_t *)BITBAND_PERI(&TB1->CR,7)))
#define TB1_MOD_TB1CLK0       (*((__IO uint32_t *)BITBAND_PERI(&TB1->MOD,0)))
#define TB1_MOD_TB1CLK1       (*((__IO uint32_t *)BITBAND_PERI(&TB1->MOD,1)))
#define TB1_MOD_TB1CLE        (*((__IO uint32_t *)BITBAND_PERI(&TB1->MOD,2)))
#define TB1_MOD_TB1CPM0       (*((__IO uint32_t *)BITBAND_PERI(&TB1->MOD,3)))
#define TB1_MOD_TB1CPM1       (*((__IO uint32_t *)BITBAND_PERI(&TB1->MOD,4)))
#define TB1_MOD_TB1CP0        (*((__O  uint32_t *)BITBAND_PERI(&TB1->MOD,5)))
#define TB1_MOD_TB1RSWR       (*((__IO uint32_t *)BITBAND_PERI(&TB1->MOD,6)))
#define TB1_FFCR_TB1FF0C0     (*((__IO uint32_t *)BITBAND_PERI(&TB1->FFCR,0)))
#define TB1_FFCR_TB1FF0C1     (*((__IO uint32_t *)BITBAND_PERI(&TB1->FFCR,1)))
#define TB1_FFCR_TB1E0T1      (*((__IO uint32_t *)BITBAND_PERI(&TB1->FFCR,2)))
#define TB1_FFCR_TB1E1T1      (*((__IO uint32_t *)BITBAND_PERI(&TB1->FFCR,3)))
#define TB1_FFCR_TB1C0T1      (*((__IO uint32_t *)BITBAND_PERI(&TB1->FFCR,4)))
#define TB1_FFCR_TB1C1T1      (*((__IO uint32_t *)BITBAND_PERI(&TB1->FFCR,5)))
#define TB1_ST_INTTB10        (*((__I  uint32_t *)BITBAND_PERI(&TB1->ST,0)))
#define TB1_ST_INTTB11        (*((__I  uint32_t *)BITBAND_PERI(&TB1->ST,1)))
#define TB1_ST_INTTBOF1       (*((__I  uint32_t *)BITBAND_PERI(&TB1->ST,2)))
#define TB1_IM_TBIM10         (*((__IO uint32_t *)BITBAND_PERI(&TB1->IM,0)))
#define TB1_IM_TBIM11         (*((__IO uint32_t *)BITBAND_PERI(&TB1->IM,1)))
#define TB1_IM_TBIMOF1        (*((__IO uint32_t *)BITBAND_PERI(&TB1->IM,2)))

#define TB2_EN_TB2EN          (*((__IO uint32_t *)BITBAND_PERI(&TB2->EN,7)))
#define TB2_RUN_TB2RUN        (*((__IO uint32_t *)BITBAND_PERI(&TB2->RUN,0)))
#define TB2_RUN_TB2PRUN       (*((__IO uint32_t *)BITBAND_PERI(&TB2->RUN,2)))
#define TB2_CR_CSSEL2         (*((__IO uint32_t *)BITBAND_PERI(&TB2->CR,0)))
#define TB2_CR_TRGSEL2        (*((__IO uint32_t *)BITBAND_PERI(&TB2->CR,1)))
#define TB2_CR_I2TB2          (*((__IO uint32_t *)BITBAND_PERI(&TB2->CR,3)))
#define TB2_CR_TB2WBF         (*((__IO uint32_t *)BITBAND_PERI(&TB2->CR,7)))
#define TB2_MOD_TB2CLK0       (*((__IO uint32_t *)BITBAND_PERI(&TB2->MOD,0)))
#define TB2_MOD_TB2CLK1       (*((__IO uint32_t *)BITBAND_PERI(&TB2->MOD,1)))
#define TB2_MOD_TB2CLE        (*((__IO uint32_t *)BITBAND_PERI(&TB2->MOD,2)))
#define TB2_MOD_TB2CPM0       (*((__IO uint32_t *)BITBAND_PERI(&TB2->MOD,3)))
#define TB2_MOD_TB2CPM1       (*((__IO uint32_t *)BITBAND_PERI(&TB2->MOD,4)))
#define TB2_MOD_TB2CP0        (*((__O  uint32_t *)BITBAND_PERI(&TB2->MOD,5)))
#define TB2_MOD_TB2RSWR       (*((__IO uint32_t *)BITBAND_PERI(&TB2->MOD,6)))
#define TB2_FFCR_TB2FF0C0     (*((__IO uint32_t *)BITBAND_PERI(&TB2->FFCR,0)))
#define TB2_FFCR_TB2FF0C1     (*((__IO uint32_t *)BITBAND_PERI(&TB2->FFCR,1)))
#define TB2_FFCR_TB2E0T1      (*((__IO uint32_t *)BITBAND_PERI(&TB2->FFCR,2)))
#define TB2_FFCR_TB2E1T1      (*((__IO uint32_t *)BITBAND_PERI(&TB2->FFCR,3)))
#define TB2_FFCR_TB2C0T1      (*((__IO uint32_t *)BITBAND_PERI(&TB2->FFCR,4)))
#define TB2_FFCR_TB2C1T1      (*((__IO uint32_t *)BITBAND_PERI(&TB2->FFCR,5)))
#define TB2_ST_INTTB20        (*((__I  uint32_t *)BITBAND_PERI(&TB2->ST,0)))
#define TB2_ST_INTTB21        (*((__I  uint32_t *)BITBAND_PERI(&TB2->ST,1)))
#define TB2_ST_INTTBOF2       (*((__I  uint32_t *)BITBAND_PERI(&TB2->ST,2)))
#define TB2_IM_TBIM20         (*((__IO uint32_t *)BITBAND_PERI(&TB2->IM,0)))
#define TB2_IM_TBIM21         (*((__IO uint32_t *)BITBAND_PERI(&TB2->IM,1)))
#define TB2_IM_TBIMOF2        (*((__IO uint32_t *)BITBAND_PERI(&TB2->IM,2)))

#define TB3_EN_TB3EN          (*((__IO uint32_t *)BITBAND_PERI(&TB3->EN,7)))
#define TB3_RUN_TB3RUN        (*((__IO uint32_t *)BITBAND_PERI(&TB3->RUN,0)))
#define TB3_RUN_TB3PRUN       (*((__IO uint32_t *)BITBAND_PERI(&TB3->RUN,2)))
#define TB3_CR_CSSEL3         (*((__IO uint32_t *)BITBAND_PERI(&TB3->CR,0)))
#define TB3_CR_TRGSEL3        (*((__IO uint32_t *)BITBAND_PERI(&TB3->CR,1)))
#define TB3_CR_I2TB3          (*((__IO uint32_t *)BITBAND_PERI(&TB3->CR,3)))
#define TB3_CR_TB3WBF         (*((__IO uint32_t *)BITBAND_PERI(&TB3->CR,7)))
#define TB3_MOD_TB3CLK0       (*((__IO uint32_t *)BITBAND_PERI(&TB3->MOD,0)))
#define TB3_MOD_TB3CLK1       (*((__IO uint32_t *)BITBAND_PERI(&TB3->MOD,1)))
#define TB3_MOD_TB3CLE        (*((__IO uint32_t *)BITBAND_PERI(&TB3->MOD,2)))
#define TB3_MOD_TB3CPM0       (*((__IO uint32_t *)BITBAND_PERI(&TB3->MOD,3)))
#define TB3_MOD_TB3CPM1       (*((__IO uint32_t *)BITBAND_PERI(&TB3->MOD,4)))
#define TB3_MOD_TB3CP0        (*((__O  uint32_t *)BITBAND_PERI(&TB3->MOD,5)))
#define TB3_MOD_TB3RSWR       (*((__IO uint32_t *)BITBAND_PERI(&TB3->MOD,6)))
#define TB3_FFCR_TB3FF0C0     (*((__IO uint32_t *)BITBAND_PERI(&TB3->FFCR,0)))
#define TB3_FFCR_TB3FF0C1     (*((__IO uint32_t *)BITBAND_PERI(&TB3->FFCR,1)))
#define TB3_FFCR_TB3E0T1      (*((__IO uint32_t *)BITBAND_PERI(&TB3->FFCR,2)))
#define TB3_FFCR_TB3E1T1      (*((__IO uint32_t *)BITBAND_PERI(&TB3->FFCR,3)))
#define TB3_FFCR_TB3C0T1      (*((__IO uint32_t *)BITBAND_PERI(&TB3->FFCR,4)))
#define TB3_FFCR_TB3C1T1      (*((__IO uint32_t *)BITBAND_PERI(&TB3->FFCR,5)))
#define TB3_ST_INTTB30        (*((__I  uint32_t *)BITBAND_PERI(&TB3->ST,0)))
#define TB3_ST_INTTB31        (*((__I  uint32_t *)BITBAND_PERI(&TB3->ST,1)))
#define TB3_ST_INTTBOF3       (*((__I  uint32_t *)BITBAND_PERI(&TB3->ST,2)))
#define TB3_IM_TBIM30         (*((__IO uint32_t *)BITBAND_PERI(&TB3->IM,0)))
#define TB3_IM_TBIM31         (*((__IO uint32_t *)BITBAND_PERI(&TB3->IM,1)))
#define TB3_IM_TBIMOF3        (*((__IO uint32_t *)BITBAND_PERI(&TB3->IM,2)))

#define TB4_EN_TB4EN          (*((__IO uint32_t *)BITBAND_PERI(&TB4->EN,7)))
#define TB4_RUN_TB4RUN        (*((__IO uint32_t *)BITBAND_PERI(&TB4->RUN,0)))
#define TB4_RUN_TB4PRUN       (*((__IO uint32_t *)BITBAND_PERI(&TB4->RUN,2)))
#define TB4_CR_CSSEL4         (*((__IO uint32_t *)BITBAND_PERI(&TB4->CR,0)))
#define TB4_CR_TRGSEL4        (*((__IO uint32_t *)BITBAND_PERI(&TB4->CR,1)))
#define TB4_CR_I2TB4          (*((__IO uint32_t *)BITBAND_PERI(&TB4->CR,3)))
#define TB4_CR_TB4WBF         (*((__IO uint32_t *)BITBAND_PERI(&TB4->CR,7)))
#define TB4_MOD_TB4CLK0       (*((__IO uint32_t *)BITBAND_PERI(&TB4->MOD,0)))
#define TB4_MOD_TB4CLK1       (*((__IO uint32_t *)BITBAND_PERI(&TB4->MOD,1)))
#define TB4_MOD_TB4CLE        (*((__IO uint32_t *)BITBAND_PERI(&TB4->MOD,2)))
#define TB4_MOD_TB4CPM0       (*((__IO uint32_t *)BITBAND_PERI(&TB4->MOD,3)))
#define TB4_MOD_TB4CPM1       (*((__IO uint32_t *)BITBAND_PERI(&TB4->MOD,4)))
#define TB4_MOD_TB4CP0        (*((__O  uint32_t *)BITBAND_PERI(&TB4->MOD,5)))
#define TB4_MOD_TB4RSWR       (*((__IO uint32_t *)BITBAND_PERI(&TB4->MOD,6)))
#define TB4_FFCR_TB4FF0C0     (*((__IO uint32_t *)BITBAND_PERI(&TB4->FFCR,0)))
#define TB4_FFCR_TB4FF0C1     (*((__IO uint32_t *)BITBAND_PERI(&TB4->FFCR,1)))
#define TB4_FFCR_TB4E0T1      (*((__IO uint32_t *)BITBAND_PERI(&TB4->FFCR,2)))
#define TB4_FFCR_TB4E1T1      (*((__IO uint32_t *)BITBAND_PERI(&TB4->FFCR,3)))
#define TB4_FFCR_TB4C0T1      (*((__IO uint32_t *)BITBAND_PERI(&TB4->FFCR,4)))
#define TB4_FFCR_TB4C1T1      (*((__IO uint32_t *)BITBAND_PERI(&TB4->FFCR,5)))
#define TB4_ST_INTTB40        (*((__I  uint32_t *)BITBAND_PERI(&TB4->ST,0)))
#define TB4_ST_INTTB41        (*((__I  uint32_t *)BITBAND_PERI(&TB4->ST,1)))
#define TB4_ST_INTTBOF4       (*((__I  uint32_t *)BITBAND_PERI(&TB4->ST,2)))
#define TB4_IM_TBIM40         (*((__IO uint32_t *)BITBAND_PERI(&TB4->IM,0)))
#define TB4_IM_TBIM41         (*((__IO uint32_t *)BITBAND_PERI(&TB4->IM,1)))
#define TB4_IM_TBIMOF4        (*((__IO uint32_t *)BITBAND_PERI(&TB4->IM,2)))

#define TB5_EN_TB5EN          (*((__IO uint32_t *)BITBAND_PERI(&TB5->EN,7)))
#define TB5_RUN_TB5RUN        (*((__IO uint32_t *)BITBAND_PERI(&TB5->RUN,0)))
#define TB5_RUN_TB5PRUN       (*((__IO uint32_t *)BITBAND_PERI(&TB5->RUN,2)))
#define TB5_CR_CSSEL5         (*((__IO uint32_t *)BITBAND_PERI(&TB5->CR,0)))
#define TB5_CR_TRGSEL5        (*((__IO uint32_t *)BITBAND_PERI(&TB5->CR,1)))
#define TB5_CR_I2TB5          (*((__IO uint32_t *)BITBAND_PERI(&TB5->CR,3)))
#define TB5_CR_TB5WBF         (*((__IO uint32_t *)BITBAND_PERI(&TB5->CR,7)))
#define TB5_MOD_TB5CLK0       (*((__IO uint32_t *)BITBAND_PERI(&TB5->MOD,0)))
#define TB5_MOD_TB5CLK1       (*((__IO uint32_t *)BITBAND_PERI(&TB5->MOD,1)))
#define TB5_MOD_TB5CLE        (*((__IO uint32_t *)BITBAND_PERI(&TB5->MOD,2)))
#define TB5_MOD_TB5CPM0       (*((__IO uint32_t *)BITBAND_PERI(&TB5->MOD,3)))
#define TB5_MOD_TB5CPM1       (*((__IO uint32_t *)BITBAND_PERI(&TB5->MOD,4)))
#define TB5_MOD_TB5CP0        (*((__O  uint32_t *)BITBAND_PERI(&TB5->MOD,5)))
#define TB5_MOD_TB5RSWR       (*((__IO uint32_t *)BITBAND_PERI(&TB5->MOD,6)))
#define TB5_FFCR_TB5FF0C0     (*((__IO uint32_t *)BITBAND_PERI(&TB5->FFCR,0)))
#define TB5_FFCR_TB5FF0C1     (*((__IO uint32_t *)BITBAND_PERI(&TB5->FFCR,1)))
#define TB5_FFCR_TB5E0T1      (*((__IO uint32_t *)BITBAND_PERI(&TB5->FFCR,2)))
#define TB5_FFCR_TB5E1T1      (*((__IO uint32_t *)BITBAND_PERI(&TB5->FFCR,3)))
#define TB5_FFCR_TB5C0T1      (*((__IO uint32_t *)BITBAND_PERI(&TB5->FFCR,4)))
#define TB5_FFCR_TB5C1T1      (*((__IO uint32_t *)BITBAND_PERI(&TB5->FFCR,5)))
#define TB5_ST_INTTB50        (*((__I  uint32_t *)BITBAND_PERI(&TB5->ST,0)))
#define TB5_ST_INTTB51        (*((__I  uint32_t *)BITBAND_PERI(&TB5->ST,1)))
#define TB5_ST_INTTBOF5       (*((__I  uint32_t *)BITBAND_PERI(&TB5->ST,2)))
#define TB5_IM_TBIM50         (*((__IO uint32_t *)BITBAND_PERI(&TB5->IM,0)))
#define TB5_IM_TBIM51         (*((__IO uint32_t *)BITBAND_PERI(&TB5->IM,1)))
#define TB5_IM_TBIMOF5        (*((__IO uint32_t *)BITBAND_PERI(&TB5->IM,2)))

#define TB6_EN_TB6EN          (*((__IO uint32_t *)BITBAND_PERI(&TB6->EN,7)))
#define TB6_RUN_TB6RUN        (*((__IO uint32_t *)BITBAND_PERI(&TB6->RUN,0)))
#define TB6_RUN_TB6PRUN       (*((__IO uint32_t *)BITBAND_PERI(&TB6->RUN,2)))
#define TB6_CR_CSSEL6         (*((__IO uint32_t *)BITBAND_PERI(&TB6->CR,0)))
#define TB6_CR_TRGSEL6        (*((__IO uint32_t *)BITBAND_PERI(&TB6->CR,1)))
#define TB6_CR_I2TB6          (*((__IO uint32_t *)BITBAND_PERI(&TB6->CR,3)))
#define TB6_CR_TB6WBF         (*((__IO uint32_t *)BITBAND_PERI(&TB6->CR,7)))
#define TB6_MOD_TB6CLK0       (*((__IO uint32_t *)BITBAND_PERI(&TB6->MOD,0)))
#define TB6_MOD_TB6CLK1       (*((__IO uint32_t *)BITBAND_PERI(&TB6->MOD,1)))
#define TB6_MOD_TB6CLE        (*((__IO uint32_t *)BITBAND_PERI(&TB6->MOD,2)))
#define TB6_MOD_TB6CPM0       (*((__IO uint32_t *)BITBAND_PERI(&TB6->MOD,3)))
#define TB6_MOD_TB6CPM1       (*((__IO uint32_t *)BITBAND_PERI(&TB6->MOD,4)))
#define TB6_MOD_TB6CP0        (*((__O  uint32_t *)BITBAND_PERI(&TB6->MOD,5)))
#define TB6_MOD_TB6RSWR       (*((__IO uint32_t *)BITBAND_PERI(&TB6->MOD,6)))
#define TB6_FFCR_TB6FF0C0     (*((__IO uint32_t *)BITBAND_PERI(&TB6->FFCR,0)))
#define TB6_FFCR_TB6FF0C1     (*((__IO uint32_t *)BITBAND_PERI(&TB6->FFCR,1)))
#define TB6_FFCR_TB6E0T1      (*((__IO uint32_t *)BITBAND_PERI(&TB6->FFCR,2)))
#define TB6_FFCR_TB6E1T1      (*((__IO uint32_t *)BITBAND_PERI(&TB6->FFCR,3)))
#define TB6_FFCR_TB6C0T1      (*((__IO uint32_t *)BITBAND_PERI(&TB6->FFCR,4)))
#define TB6_FFCR_TB6C1T1      (*((__IO uint32_t *)BITBAND_PERI(&TB6->FFCR,5)))
#define TB6_ST_INTTB60        (*((__I  uint32_t *)BITBAND_PERI(&TB6->ST,0)))
#define TB6_ST_INTTB61        (*((__I  uint32_t *)BITBAND_PERI(&TB6->ST,1)))
#define TB6_ST_INTTBOF6       (*((__I  uint32_t *)BITBAND_PERI(&TB6->ST,2)))
#define TB6_IM_TBIM60         (*((__IO uint32_t *)BITBAND_PERI(&TB6->IM,0)))
#define TB6_IM_TBIM61         (*((__IO uint32_t *)BITBAND_PERI(&TB6->IM,1)))
#define TB6_IM_TBIMOF6        (*((__IO uint32_t *)BITBAND_PERI(&TB6->IM,2)))

#define TB7_EN_TB7EN          (*((__IO uint32_t *)BITBAND_PERI(&TB7->EN,7)))
#define TB7_RUN_TB7RUN        (*((__IO uint32_t *)BITBAND_PERI(&TB7->RUN,0)))
#define TB7_RUN_TB7PRUN       (*((__IO uint32_t *)BITBAND_PERI(&TB7->RUN,2)))
#define TB7_CR_CSSEL7         (*((__IO uint32_t *)BITBAND_PERI(&TB7->CR,0)))
#define TB7_CR_TRGSEL7        (*((__IO uint32_t *)BITBAND_PERI(&TB7->CR,1)))
#define TB7_CR_I2TB7          (*((__IO uint32_t *)BITBAND_PERI(&TB7->CR,3)))
#define TB7_CR_TB7WBF         (*((__IO uint32_t *)BITBAND_PERI(&TB7->CR,7)))
#define TB7_MOD_TB7CLK0       (*((__IO uint32_t *)BITBAND_PERI(&TB7->MOD,0)))
#define TB7_MOD_TB7CLK1       (*((__IO uint32_t *)BITBAND_PERI(&TB7->MOD,1)))
#define TB7_MOD_TB7CLE        (*((__IO uint32_t *)BITBAND_PERI(&TB7->MOD,2)))
#define TB7_MOD_TB7CPM0       (*((__IO uint32_t *)BITBAND_PERI(&TB7->MOD,3)))
#define TB7_MOD_TB7CPM1       (*((__IO uint32_t *)BITBAND_PERI(&TB7->MOD,4)))
#define TB7_MOD_TB7CP0        (*((__O  uint32_t *)BITBAND_PERI(&TB7->MOD,5)))
#define TB7_MOD_TB7RSWR       (*((__IO uint32_t *)BITBAND_PERI(&TB7->MOD,6)))
#define TB7_FFCR_TB7FF0C0     (*((__IO uint32_t *)BITBAND_PERI(&TB7->FFCR,0)))
#define TB7_FFCR_TB7FF0C1     (*((__IO uint32_t *)BITBAND_PERI(&TB7->FFCR,1)))
#define TB7_FFCR_TB7E0T1      (*((__IO uint32_t *)BITBAND_PERI(&TB7->FFCR,2)))
#define TB7_FFCR_TB7E1T1      (*((__IO uint32_t *)BITBAND_PERI(&TB7->FFCR,3)))
#define TB7_FFCR_TB7C0T1      (*((__IO uint32_t *)BITBAND_PERI(&TB7->FFCR,4)))
#define TB7_FFCR_TB7C1T1      (*((__IO uint32_t *)BITBAND_PERI(&TB7->FFCR,5)))
#define TB7_ST_INTTB70        (*((__I  uint32_t *)BITBAND_PERI(&TB7->ST,0)))
#define TB7_ST_INTTB71        (*((__I  uint32_t *)BITBAND_PERI(&TB7->ST,1)))
#define TB7_ST_INTTBOF7       (*((__I  uint32_t *)BITBAND_PERI(&TB7->ST,2)))
#define TB7_IM_TBIM70         (*((__IO uint32_t *)BITBAND_PERI(&TB7->IM,0)))
#define TB7_IM_TBIM71         (*((__IO uint32_t *)BITBAND_PERI(&TB7->IM,1)))
#define TB7_IM_TBIMOF7        (*((__IO uint32_t *)BITBAND_PERI(&TB7->IM,2)))

/* Encoder input (EN) */
#define EN0_TNCR_ENDEV0       (*((__IO uint32_t *)BITBAND_PERI(&EN0->TNCR,0)))
#define EN0_TNCR_ENDEV1       (*((__IO uint32_t *)BITBAND_PERI(&EN0->TNCR,1)))
#define EN0_TNCR_ENDEV2       (*((__IO uint32_t *)BITBAND_PERI(&EN0->TNCR,2)))
#define EN0_TNCR_INTEN        (*((__IO uint32_t *)BITBAND_PERI(&EN0->TNCR,3)))
#define EN0_TNCR_NR0          (*((__IO uint32_t *)BITBAND_PERI(&EN0->TNCR,4)))
#define EN0_TNCR_NR1          (*((__IO uint32_t *)BITBAND_PERI(&EN0->TNCR,5)))
#define EN0_TNCR_ENRUN        (*((__IO uint32_t *)BITBAND_PERI(&EN0->TNCR,6)))
#define EN0_TNCR_ZEN          (*((__IO uint32_t *)BITBAND_PERI(&EN0->TNCR,7)))
#define EN0_TNCR_CMPEN        (*((__IO uint32_t *)BITBAND_PERI(&EN0->TNCR,8)))
#define EN0_TNCR_ZESEL        (*((__IO uint32_t *)BITBAND_PERI(&EN0->TNCR,9)))
#define EN0_TNCR_ENCLR        (*((__O  uint32_t *)BITBAND_PERI(&EN0->TNCR,10)))
#define EN0_TNCR_SFTCAP       (*((__O  uint32_t *)BITBAND_PERI(&EN0->TNCR,11)))
#define EN0_TNCR_ZDET         (*((__I  uint32_t *)BITBAND_PERI(&EN0->TNCR,12)))
#define EN0_TNCR_U_D          (*((__I  uint32_t *)BITBAND_PERI(&EN0->TNCR,13)))
#define EN0_TNCR_REVERR       (*((__I  uint32_t *)BITBAND_PERI(&EN0->TNCR,14)))
#define EN0_TNCR_CMP          (*((__I  uint32_t *)BITBAND_PERI(&EN0->TNCR,15)))
#define EN0_TNCR_P3EN         (*((__IO uint32_t *)BITBAND_PERI(&EN0->TNCR,16)))
#define EN0_TNCR_MODE0        (*((__IO uint32_t *)BITBAND_PERI(&EN0->TNCR,17)))
#define EN0_TNCR_MODE1        (*((__IO uint32_t *)BITBAND_PERI(&EN0->TNCR,18)))

#define EN1_TNCR_ENDEV0       (*((__IO uint32_t *)BITBAND_PERI(&EN1->TNCR,0)))
#define EN1_TNCR_ENDEV1       (*((__IO uint32_t *)BITBAND_PERI(&EN1->TNCR,1)))
#define EN1_TNCR_ENDEV2       (*((__IO uint32_t *)BITBAND_PERI(&EN1->TNCR,2)))
#define EN1_TNCR_INTEN        (*((__IO uint32_t *)BITBAND_PERI(&EN1->TNCR,3)))
#define EN1_TNCR_NR0          (*((__IO uint32_t *)BITBAND_PERI(&EN1->TNCR,4)))
#define EN1_TNCR_NR1          (*((__IO uint32_t *)BITBAND_PERI(&EN1->TNCR,5)))
#define EN1_TNCR_ENRUN        (*((__IO uint32_t *)BITBAND_PERI(&EN1->TNCR,6)))
#define EN1_TNCR_ZEN          (*((__IO uint32_t *)BITBAND_PERI(&EN1->TNCR,7)))
#define EN1_TNCR_CMPEN        (*((__IO uint32_t *)BITBAND_PERI(&EN1->TNCR,8)))
#define EN1_TNCR_ZESEL        (*((__IO uint32_t *)BITBAND_PERI(&EN1->TNCR,9)))
#define EN1_TNCR_ENCLR        (*((__O  uint32_t *)BITBAND_PERI(&EN1->TNCR,10)))
#define EN1_TNCR_SFTCAP       (*((__O  uint32_t *)BITBAND_PERI(&EN1->TNCR,11)))
#define EN1_TNCR_ZDET         (*((__I  uint32_t *)BITBAND_PERI(&EN1->TNCR,12)))
#define EN1_TNCR_U_D          (*((__I  uint32_t *)BITBAND_PERI(&EN1->TNCR,13)))
#define EN1_TNCR_REVERR       (*((__I  uint32_t *)BITBAND_PERI(&EN1->TNCR,14)))
#define EN1_TNCR_CMP          (*((__I  uint32_t *)BITBAND_PERI(&EN1->TNCR,15)))
#define EN1_TNCR_P3EN         (*((__IO uint32_t *)BITBAND_PERI(&EN1->TNCR,16)))
#define EN1_TNCR_MODE0        (*((__IO uint32_t *)BITBAND_PERI(&EN1->TNCR,17)))
#define EN1_TNCR_MODE1        (*((__IO uint32_t *)BITBAND_PERI(&EN1->TNCR,18)))

/* Serial Channel (SC) */
#define SC0_EN_SIOE           (*((__IO uint32_t *)BITBAND_PERI(&SC0->EN,0)))
#define SC0_CR_IOC            (*((__IO uint32_t *)BITBAND_PERI(&SC0->CR,0)))
#define SC0_CR_SCLKS          (*((__IO uint32_t *)BITBAND_PERI(&SC0->CR,1)))
#define SC0_CR_PE             (*((__IO uint32_t *)BITBAND_PERI(&SC0->CR,5)))
#define SC0_CR_EVEN           (*((__IO uint32_t *)BITBAND_PERI(&SC0->CR,6)))
#define SC0_CR_RB8            (*((__I  uint32_t *)BITBAND_PERI(&SC0->CR,7)))
#define SC0_MOD0_WU           (*((__IO uint32_t *)BITBAND_PERI(&SC0->MOD0,4)))
#define SC0_MOD0_RXE          (*((__IO uint32_t *)BITBAND_PERI(&SC0->MOD0,5)))
#define SC0_MOD0_CTSE         (*((__IO uint32_t *)BITBAND_PERI(&SC0->MOD0,6)))
#define SC0_MOD0_TB8          (*((__IO uint32_t *)BITBAND_PERI(&SC0->MOD0,7)))
#define SC0_BRCR_BR0ADDE      (*((__IO uint32_t *)BITBAND_PERI(&SC0->BRCR,6)))
#define SC0_MOD1_TXE          (*((__IO uint32_t *)BITBAND_PERI(&SC0->MOD1,4)))
#define SC0_MOD1_I2S0         (*((__IO uint32_t *)BITBAND_PERI(&SC0->MOD1,7)))
#define SC0_MOD2_WBUF         (*((__IO uint32_t *)BITBAND_PERI(&SC0->MOD2,2)))
#define SC0_MOD2_DRCHG        (*((__IO uint32_t *)BITBAND_PERI(&SC0->MOD2,3)))
#define SC0_MOD2_SBLEN        (*((__IO uint32_t *)BITBAND_PERI(&SC0->MOD2,4)))
#define SC0_MOD2_TXRUN        (*((__I uint32_t *)BITBAND_PERI(&SC0->MOD2,5)))
#define SC0_MOD2_RBFLL        (*((__I uint32_t *)BITBAND_PERI(&SC0->MOD2,6)))
#define SC0_MOD2_TBEMP        (*((__I uint32_t *)BITBAND_PERI(&SC0->MOD2,7)))
#define SC0_RFC_RFIS          (*((__IO uint32_t *)BITBAND_PERI(&SC0->RFC,6)))
#define SC0_RFC_RFCS          (*((__O  uint32_t *)BITBAND_PERI(&SC0->RFC,7)))
#define SC0_TFC_TFIS          (*((__IO uint32_t *)BITBAND_PERI(&SC0->TFC,6)))
#define SC0_TFC_TFCS          (*((__O  uint32_t *)BITBAND_PERI(&SC0->TFC,7)))
#define SC0_RST_ROR           (*((__I  uint32_t *)BITBAND_PERI(&SC0->RST,7)))
#define SC0_TST_TUR           (*((__I  uint32_t *)BITBAND_PERI(&SC0->TST,7)))
#define SC0_FCNF_CNFG         (*((__IO uint32_t *)BITBAND_PERI(&SC0->FCNF,0)))
#define SC0_FCNF_RXTXCNT      (*((__IO uint32_t *)BITBAND_PERI(&SC0->FCNF,1)))
#define SC0_FCNF_RFIE         (*((__IO uint32_t *)BITBAND_PERI(&SC0->FCNF,2)))
#define SC0_FCNF_TFIE         (*((__IO uint32_t *)BITBAND_PERI(&SC0->FCNF,3)))
#define SC0_FCNF_RFST         (*((__IO uint32_t *)BITBAND_PERI(&SC0->FCNF,4)))

#define SC1_EN_SIOE           (*((__IO uint32_t *)BITBAND_PERI(&SC1->EN,0)))
#define SC1_CR_IOC            (*((__IO uint32_t *)BITBAND_PERI(&SC1->CR,0)))
#define SC1_CR_SCLKS          (*((__IO uint32_t *)BITBAND_PERI(&SC1->CR,1)))
#define SC1_CR_PE             (*((__IO uint32_t *)BITBAND_PERI(&SC1->CR,5)))
#define SC1_CR_EVEN           (*((__IO uint32_t *)BITBAND_PERI(&SC1->CR,6)))
#define SC1_CR_RB8            (*((__I  uint32_t *)BITBAND_PERI(&SC1->CR,7)))
#define SC1_MOD0_WU           (*((__IO uint32_t *)BITBAND_PERI(&SC1->MOD0,4)))
#define SC1_MOD0_RXE          (*((__IO uint32_t *)BITBAND_PERI(&SC1->MOD0,5)))
#define SC1_MOD0_CTSE         (*((__IO uint32_t *)BITBAND_PERI(&SC1->MOD0,6)))
#define SC1_MOD0_TB8          (*((__IO uint32_t *)BITBAND_PERI(&SC1->MOD0,7)))
#define SC1_BRCR_BR0ADDE      (*((__IO uint32_t *)BITBAND_PERI(&SC1->BRCR,6)))
#define SC1_MOD1_TXE          (*((__IO uint32_t *)BITBAND_PERI(&SC1->MOD1,4)))
#define SC1_MOD1_I2S0         (*((__IO uint32_t *)BITBAND_PERI(&SC1->MOD1,7)))
#define SC1_MOD2_WBUF         (*((__IO uint32_t *)BITBAND_PERI(&SC1->MOD2,2)))
#define SC1_MOD2_DRCHG        (*((__IO uint32_t *)BITBAND_PERI(&SC1->MOD2,3)))
#define SC1_MOD2_SBLEN        (*((__IO uint32_t *)BITBAND_PERI(&SC1->MOD2,4)))
#define SC1_MOD2_TXRUN        (*((__I  uint32_t *)BITBAND_PERI(&SC1->MOD2,5)))
#define SC1_MOD2_RBFLL        (*((__I  uint32_t *)BITBAND_PERI(&SC1->MOD2,6)))
#define SC1_MOD2_TBEMP        (*((__I  uint32_t *)BITBAND_PERI(&SC1->MOD2,7)))
#define SC1_RFC_RFIS          (*((__IO uint32_t *)BITBAND_PERI(&SC1->RFC,6)))
#define SC1_RFC_RFCS          (*((__O  uint32_t *)BITBAND_PERI(&SC1->RFC,7)))
#define SC1_TFC_TFIS          (*((__IO uint32_t *)BITBAND_PERI(&SC1->TFC,6)))
#define SC1_TFC_TFCS          (*((__O  uint32_t *)BITBAND_PERI(&SC1->TFC,7)))
#define SC1_RST_ROR           (*((__I  uint32_t *)BITBAND_PERI(&SC1->RST,7)))
#define SC1_TST_TUR           (*((__I  uint32_t *)BITBAND_PERI(&SC1->TST,7)))
#define SC1_FCNF_CNFG         (*((__IO uint32_t *)BITBAND_PERI(&SC1->FCNF,0)))
#define SC1_FCNF_RXTXCNT      (*((__IO uint32_t *)BITBAND_PERI(&SC1->FCNF,1)))
#define SC1_FCNF_RFIE         (*((__IO uint32_t *)BITBAND_PERI(&SC1->FCNF,2)))
#define SC1_FCNF_TFIE         (*((__IO uint32_t *)BITBAND_PERI(&SC1->FCNF,3)))
#define SC1_FCNF_RFST         (*((__IO uint32_t *)BITBAND_PERI(&SC1->FCNF,4)))

#define SC2_EN_SIOE           (*((__IO uint32_t *)BITBAND_PERI(&SC2->EN,0)))
#define SC2_CR_IOC            (*((__IO uint32_t *)BITBAND_PERI(&SC2->CR,0)))
#define SC2_CR_SCLKS          (*((__IO uint32_t *)BITBAND_PERI(&SC2->CR,1)))
#define SC2_CR_PE             (*((__IO uint32_t *)BITBAND_PERI(&SC2->CR,5)))
#define SC2_CR_EVEN           (*((__IO uint32_t *)BITBAND_PERI(&SC2->CR,6)))
#define SC2_CR_RB8            (*((__I  uint32_t *)BITBAND_PERI(&SC2->CR,7)))
#define SC2_MOD0_WU           (*((__IO uint32_t *)BITBAND_PERI(&SC2->MOD0,4)))
#define SC2_MOD0_RXE          (*((__IO uint32_t *)BITBAND_PERI(&SC2->MOD0,5)))
#define SC2_MOD0_CTSE         (*((__IO uint32_t *)BITBAND_PERI(&SC2->MOD0,6)))
#define SC2_MOD0_TB8          (*((__IO uint32_t *)BITBAND_PERI(&SC2->MOD0,7)))
#define SC2_BRCR_BR0ADDE      (*((__IO uint32_t *)BITBAND_PERI(&SC2->BRCR,6)))
#define SC2_MOD1_TXE          (*((__IO uint32_t *)BITBAND_PERI(&SC2->MOD1,4)))
#define SC2_MOD1_I2S0         (*((__IO uint32_t *)BITBAND_PERI(&SC2->MOD1,7)))
#define SC2_MOD2_WBUF         (*((__IO uint32_t *)BITBAND_PERI(&SC2->MOD2,2)))
#define SC2_MOD2_DRCHG        (*((__IO uint32_t *)BITBAND_PERI(&SC2->MOD2,3)))
#define SC2_MOD2_SBLEN        (*((__IO uint32_t *)BITBAND_PERI(&SC2->MOD2,4)))
#define SC2_MOD2_TXRUN        (*((__I  uint32_t *)BITBAND_PERI(&SC2->MOD2,5)))
#define SC2_MOD2_RBFLL        (*((__I  uint32_t *)BITBAND_PERI(&SC2->MOD2,6)))
#define SC2_MOD2_TBEMP        (*((__I  uint32_t *)BITBAND_PERI(&SC2->MOD2,7)))
#define SC2_RFC_RFIS          (*((__IO uint32_t *)BITBAND_PERI(&SC2->RFC,6)))
#define SC2_RFC_RFCS          (*((__O  uint32_t *)BITBAND_PERI(&SC2->RFC,7)))
#define SC2_TFC_TFIS          (*((__IO uint32_t *)BITBAND_PERI(&SC2->TFC,6)))
#define SC2_TFC_TFCS          (*((__O  uint32_t *)BITBAND_PERI(&SC2->TFC,7)))
#define SC2_RST_ROR           (*((__I  uint32_t *)BITBAND_PERI(&SC2->RST,7)))
#define SC2_TST_TUR           (*((__I  uint32_t *)BITBAND_PERI(&SC2->TST,7)))
#define SC2_FCNF_CNFG         (*((__IO uint32_t *)BITBAND_PERI(&SC2->FCNF,0)))
#define SC2_FCNF_RXTXCNT      (*((__IO uint32_t *)BITBAND_PERI(&SC2->FCNF,1)))
#define SC2_FCNF_RFIE         (*((__IO uint32_t *)BITBAND_PERI(&SC2->FCNF,2)))
#define SC2_FCNF_TFIE         (*((__IO uint32_t *)BITBAND_PERI(&SC2->FCNF,3)))
#define SC2_FCNF_RFST         (*((__IO uint32_t *)BITBAND_PERI(&SC2->FCNF,4)))

#define SC3_EN_SIOE           (*((__IO uint32_t *)BITBAND_PERI(&SC3->EN,0)))
#define SC3_CR_IOC            (*((__IO uint32_t *)BITBAND_PERI(&SC3->CR,0)))
#define SC3_CR_SCLKS          (*((__IO uint32_t *)BITBAND_PERI(&SC3->CR,1)))
#define SC3_CR_PE             (*((__IO uint32_t *)BITBAND_PERI(&SC3->CR,5)))
#define SC3_CR_EVEN           (*((__IO uint32_t *)BITBAND_PERI(&SC3->CR,6)))
#define SC3_CR_RB8            (*((__I  uint32_t *)BITBAND_PERI(&SC3->CR,7)))
#define SC3_MOD0_WU           (*((__IO uint32_t *)BITBAND_PERI(&SC3->MOD0,4)))
#define SC3_MOD0_RXE          (*((__IO uint32_t *)BITBAND_PERI(&SC3->MOD0,5)))
#define SC3_MOD0_CTSE         (*((__IO uint32_t *)BITBAND_PERI(&SC3->MOD0,6)))
#define SC3_MOD0_TB8          (*((__IO uint32_t *)BITBAND_PERI(&SC3->MOD0,7)))
#define SC3_BRCR_BR0ADDE      (*((__IO uint32_t *)BITBAND_PERI(&SC3->BRCR,6)))
#define SC3_MOD1_TXE          (*((__IO uint32_t *)BITBAND_PERI(&SC3->MOD1,4)))
#define SC3_MOD1_I2S0         (*((__IO uint32_t *)BITBAND_PERI(&SC3->MOD1,7)))
#define SC3_MOD2_WBUF         (*((__IO uint32_t *)BITBAND_PERI(&SC3->MOD2,2)))
#define SC3_MOD2_DRCHG        (*((__IO uint32_t *)BITBAND_PERI(&SC3->MOD2,3)))
#define SC3_MOD2_SBLEN        (*((__IO uint32_t *)BITBAND_PERI(&SC3->MOD2,4)))
#define SC3_MOD2_TXRUN        (*((__I  uint32_t *)BITBAND_PERI(&SC3->MOD2,5)))
#define SC3_MOD2_RBFLL        (*((__I  uint32_t *)BITBAND_PERI(&SC3->MOD2,6)))
#define SC3_MOD2_TBEMP        (*((__I  uint32_t *)BITBAND_PERI(&SC3->MOD2,7)))
#define SC3_RFC_RFIS          (*((__IO uint32_t *)BITBAND_PERI(&SC3->RFC,6)))
#define SC3_RFC_RFCS          (*((__O  uint32_t *)BITBAND_PERI(&SC3->RFC,7)))
#define SC3_TFC_TFIS          (*((__IO uint32_t *)BITBAND_PERI(&SC3->TFC,6)))
#define SC3_TFC_TFCS          (*((__O  uint32_t *)BITBAND_PERI(&SC3->TFC,7)))
#define SC3_RST_ROR           (*((__I  uint32_t *)BITBAND_PERI(&SC3->RST,7)))
#define SC3_TST_TUR           (*((__I  uint32_t *)BITBAND_PERI(&SC3->TST,7)))
#define SC3_FCNF_CNFG         (*((__IO uint32_t *)BITBAND_PERI(&SC3->FCNF,0)))
#define SC3_FCNF_RXTXCNT      (*((__IO uint32_t *)BITBAND_PERI(&SC3->FCNF,1)))
#define SC3_FCNF_RFIE         (*((__IO uint32_t *)BITBAND_PERI(&SC3->FCNF,2)))
#define SC3_FCNF_TFIE         (*((__IO uint32_t *)BITBAND_PERI(&SC3->FCNF,3)))
#define SC3_FCNF_RFST         (*((__IO uint32_t *)BITBAND_PERI(&SC3->FCNF,4)))

/* Analog-to-Digital Converter (AD) */
#define ADA_MOD0_ADSS         (*((__O  uint32_t *)BITBAND_PERI(&ADA->MOD0,0)))
#define ADA_MOD0_DACON        (*((__IO uint32_t *)BITBAND_PERI(&ADA->MOD0,1)))
#define ADA_MOD1_ADAS         (*((__IO uint32_t *)BITBAND_PERI(&ADA->MOD1,0)))
#define ADA_MOD1_ADEN         (*((__IO uint32_t *)BITBAND_PERI(&ADA->MOD1,7)))
#define ADA_MOD2_ADBFN        (*((__I  uint32_t *)BITBAND_PERI(&ADA->MOD2,0)))
#define ADA_MOD2_ADSFN        (*((__I  uint32_t *)BITBAND_PERI(&ADA->MOD2,1)))
#define ADA_CMPCR0_ADBIG0     (*((__IO uint32_t *)BITBAND_PERI(&ADA->CMPCR0,4)))
#define ADA_CMPCR0_CMP0EN     (*((__IO uint32_t *)BITBAND_PERI(&ADA->CMPCR0,7)))
#define ADA_CMPCR1_ADBIG1     (*((__IO uint32_t *)BITBAND_PERI(&ADA->CMPCR1,4)))
#define ADA_CMPCR1_CMP1EN     (*((__IO uint32_t *)BITBAND_PERI(&ADA->CMPCR1,7)))
#define ADA_REG0_ADR0RF       (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG0,0)))
#define ADA_REG0_OVR0         (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG0,1)))
#define ADA_REG1_ADR1RF       (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG1,0)))
#define ADA_REG1_OVR1         (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG1,1)))
#define ADA_REG2_ADR2RF       (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG2,0)))
#define ADA_REG2_OVR2         (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG2,1)))
#define ADA_REG3_ADR3RF       (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG3,0)))
#define ADA_REG3_OVR3         (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG3,1)))
#define ADA_REG4_ADR4RF       (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG4,0)))
#define ADA_REG4_OVR4         (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG4,1)))
#define ADA_REG5_ADR5RF       (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG5,0)))
#define ADA_REG5_OVR5         (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG5,1)))
#define ADA_REG6_ADR6RF       (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG6,0)))
#define ADA_REG6_OVR6         (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG6,1)))
#define ADA_REG7_ADR7RF       (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG7,0)))
#define ADA_REG7_OVR7         (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG7,1)))
#define ADA_REG8_ADR8RF       (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG8,0)))
#define ADA_REG8_OVR8         (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG8,1)))
#define ADA_REG9_ADR9RF       (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG9,0)))
#define ADA_REG9_OVR9         (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG9,1)))
#define ADA_REG10_ADR10RF     (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG10,0)))
#define ADA_REG10_OVR10       (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG10,1)))
#define ADA_REG11_ADR11RF     (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG11,0)))
#define ADA_REG11_OVR11       (*((__I  uint32_t *)BITBAND_PERI(&ADA->REG11,1)))
#define ADA_PSEL0_PENS0       (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSEL0,7)))
#define ADA_PSEL1_PENS1       (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSEL1,7)))
#define ADA_PSEL2_PENS2       (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSEL2,7)))
#define ADA_PSEL3_PENS3       (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSEL3,7)))
#define ADA_PSEL4_PENS4       (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSEL4,7)))
#define ADA_PSEL5_PENS5       (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSEL5,7)))
#define ADA_PSEL6_PENS6       (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSEL6,7)))
#define ADA_PSEL7_PENS7       (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSEL7,7)))
#define ADA_PSEL8_PENS8       (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSEL8,7)))
#define ADA_PSEL9_PENS9       (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSEL9,7)))
#define ADA_PSEL10_PENS10     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSEL10,7)))
#define ADA_PSEL11_PENS11     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSEL11,7)))
#define ADA_PINTS0_INTSEL00   (*((__IO uint32_t *)BITBAND_PERI(&ADA->PINTS0,0)))
#define ADA_PINTS0_INTSEL01   (*((__IO uint32_t *)BITBAND_PERI(&ADA->PINTS0,1)))
#define ADA_PINTS1_INTSEL10   (*((__IO uint32_t *)BITBAND_PERI(&ADA->PINTS1,0)))
#define ADA_PINTS1_INTSEL11   (*((__IO uint32_t *)BITBAND_PERI(&ADA->PINTS1,1)))
#define ADA_PINTS2_INTSEL20   (*((__IO uint32_t *)BITBAND_PERI(&ADA->PINTS2,0)))
#define ADA_PINTS2_INTSEL21   (*((__IO uint32_t *)BITBAND_PERI(&ADA->PINTS2,1)))
#define ADA_PINTS3_INTSEL30   (*((__IO uint32_t *)BITBAND_PERI(&ADA->PINTS3,0)))
#define ADA_PINTS3_INTSEL31   (*((__IO uint32_t *)BITBAND_PERI(&ADA->PINTS3,1)))
#define ADA_PINTS4_INTSEL40   (*((__IO uint32_t *)BITBAND_PERI(&ADA->PINTS4,0)))
#define ADA_PINTS4_INTSEL41   (*((__IO uint32_t *)BITBAND_PERI(&ADA->PINTS4,1)))
#define ADA_PINTS5_INTSEL50   (*((__IO uint32_t *)BITBAND_PERI(&ADA->PINTS5,0)))
#define ADA_PINTS5_INTSEL51   (*((__IO uint32_t *)BITBAND_PERI(&ADA->PINTS5,1)))
#define ADA_PSET00_ENSP00     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET00,7)))
#define ADA_PSET01_ENSP01     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET01,7)))
#define ADA_PSET02_ENSP02     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET02,7)))
#define ADA_PSET03_ENSP03     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET03,7)))
#define ADA_PSET10_ENSP10     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET10,7)))
#define ADA_PSET11_ENSP11     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET11,7)))
#define ADA_PSET12_ENSP12     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET12,7)))
#define ADA_PSET13_ENSP13     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET13,7)))
#define ADA_PSET20_ENSP20     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET20,7))) 
#define ADA_PSET21_ENSP21     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET21,7)))
#define ADA_PSET22_ENSP22     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET22,7)))
#define ADA_PSET23_ENSP23     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET23,7)))
#define ADA_PSET30_ENSP30     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET30,7))) 
#define ADA_PSET31_ENSP31     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET31,7)))
#define ADA_PSET32_ENSP32     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET32,7)))
#define ADA_PSET33_ENSP33     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET33,7)))
#define ADA_PSET40_ENSP40     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET40,7))) 
#define ADA_PSET41_ENSP41     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET41,7)))
#define ADA_PSET42_ENSP42     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET42,7)))
#define ADA_PSET43_ENSP43     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET43,7)))
#define ADA_PSET50_ENSP50     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET50,7))) 
#define ADA_PSET51_ENSP51     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET51,7)))
#define ADA_PSET52_ENSP52     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET52,7)))
#define ADA_PSET53_ENSP53     (*((__IO uint32_t *)BITBAND_PERI(&ADA->PSET53,7)))
#define ADA_TSET0_ENST0       (*((__IO uint32_t *)BITBAND_PERI(&ADA->TSET0,7)))
#define ADA_TSET1_ENST1       (*((__IO uint32_t *)BITBAND_PERI(&ADA->TSET1,7)))
#define ADA_TSET2_ENST2       (*((__IO uint32_t *)BITBAND_PERI(&ADA->TSET2,7)))
#define ADA_TSET3_ENST3       (*((__IO uint32_t *)BITBAND_PERI(&ADA->TSET3,7)))
#define ADA_TSET4_ENST4       (*((__IO uint32_t *)BITBAND_PERI(&ADA->TSET4,7)))
#define ADA_TSET5_ENST5       (*((__IO uint32_t *)BITBAND_PERI(&ADA->TSET5,7)))
#define ADA_TSET6_ENST6       (*((__IO uint32_t *)BITBAND_PERI(&ADA->TSET6,7)))
#define ADA_TSET7_ENST7       (*((__IO uint32_t *)BITBAND_PERI(&ADA->TSET7,7)))
#define ADA_TSET8_ENST8       (*((__IO uint32_t *)BITBAND_PERI(&ADA->TSET8,7)))
#define ADA_TSET9_ENST9       (*((__IO uint32_t *)BITBAND_PERI(&ADA->TSET9,7)))
#define ADA_TSET10_ENST10     (*((__IO uint32_t *)BITBAND_PERI(&ADA->TSET10,7)))
#define ADA_TSET11_ENST11     (*((__IO uint32_t *)BITBAND_PERI(&ADA->TSET11,7)))
#define ADA_SSET0_ENSS0       (*((__IO uint32_t *)BITBAND_PERI(&ADA->SSET0,7)))
#define ADA_SSET1_ENSS1       (*((__IO uint32_t *)BITBAND_PERI(&ADA->SSET1,7)))
#define ADA_SSET2_ENSS2       (*((__IO uint32_t *)BITBAND_PERI(&ADA->SSET2,7)))
#define ADA_SSET3_ENSS3       (*((__IO uint32_t *)BITBAND_PERI(&ADA->SSET3,7)))
#define ADA_SSET4_ENSS4       (*((__IO uint32_t *)BITBAND_PERI(&ADA->SSET4,7)))
#define ADA_SSET5_ENSS5       (*((__IO uint32_t *)BITBAND_PERI(&ADA->SSET5,7)))
#define ADA_SSET6_ENSS6       (*((__IO uint32_t *)BITBAND_PERI(&ADA->SSET6,7)))
#define ADA_SSET7_ENSS7       (*((__IO uint32_t *)BITBAND_PERI(&ADA->SSET7,7)))
#define ADA_SSET8_ENSS8       (*((__IO uint32_t *)BITBAND_PERI(&ADA->SSET8,7)))
#define ADA_SSET9_ENSS9       (*((__IO uint32_t *)BITBAND_PERI(&ADA->SSET9,7)))
#define ADA_SSET10_ENSS10     (*((__IO uint32_t *)BITBAND_PERI(&ADA->SSET10,7)))
#define ADA_SSET11_ENSS11     (*((__IO uint32_t *)BITBAND_PERI(&ADA->SSET11,7)))
#define ADA_ASET0_ENSA0       (*((__IO uint32_t *)BITBAND_PERI(&ADA->ASET0,7)))
#define ADA_ASET1_ENSA1       (*((__IO uint32_t *)BITBAND_PERI(&ADA->ASET1,7)))
#define ADA_ASET2_ENSA2       (*((__IO uint32_t *)BITBAND_PERI(&ADA->ASET2,7)))
#define ADA_ASET3_ENSA3       (*((__IO uint32_t *)BITBAND_PERI(&ADA->ASET3,7)))
#define ADA_ASET4_ENSA4       (*((__IO uint32_t *)BITBAND_PERI(&ADA->ASET4,7)))
#define ADA_ASET5_ENSA5       (*((__IO uint32_t *)BITBAND_PERI(&ADA->ASET5,7)))
#define ADA_ASET6_ENSA6       (*((__IO uint32_t *)BITBAND_PERI(&ADA->ASET6,7)))
#define ADA_ASET7_ENSA7       (*((__IO uint32_t *)BITBAND_PERI(&ADA->ASET7,7)))
#define ADA_ASET8_ENSA8       (*((__IO uint32_t *)BITBAND_PERI(&ADA->ASET8,7)))
#define ADA_ASET9_ENSA9       (*((__IO uint32_t *)BITBAND_PERI(&ADA->ASET9,7)))
#define ADA_ASET10_ENSA10     (*((__IO uint32_t *)BITBAND_PERI(&ADA->ASET10,7)))
#define ADA_ASET11_ENSA11     (*((__IO uint32_t *)BITBAND_PERI(&ADA->ASET11,7)))

#define ADB_MOD0_ADSS         (*((__O  uint32_t *)BITBAND_PERI(&ADB->MOD0,0)))
#define ADB_MOD0_DACON        (*((__IO uint32_t *)BITBAND_PERI(&ADB->MOD0,1)))
#define ADB_MOD1_ADAS         (*((__IO uint32_t *)BITBAND_PERI(&ADB->MOD1,0)))
#define ADB_MOD1_ADEN         (*((__IO uint32_t *)BITBAND_PERI(&ADB->MOD1,7)))
#define ADB_MOD2_ADBFN        (*((__I  uint32_t *)BITBAND_PERI(&ADB->MOD2,0)))
#define ADB_MOD2_ADSFN        (*((__I  uint32_t *)BITBAND_PERI(&ADB->MOD2,1)))
#define ADB_CMPCR0_ADBIG0     (*((__IO uint32_t *)BITBAND_PERI(&ADB->CMPCR0,4)))
#define ADB_CMPCR0_CMP0EN     (*((__IO uint32_t *)BITBAND_PERI(&ADB->CMPCR0,7)))
#define ADB_CMPCR1_ADBIG1     (*((__IO uint32_t *)BITBAND_PERI(&ADB->CMPCR1,4)))
#define ADB_CMPCR1_CMP1EN     (*((__IO uint32_t *)BITBAND_PERI(&ADB->CMPCR1,7)))
#define ADB_REG0_ADR0RF       (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG0,0)))
#define ADB_REG0_OVR0         (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG0,1)))
#define ADB_REG1_ADR1RF       (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG1,0)))
#define ADB_REG1_OVR1         (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG1,1)))
#define ADB_REG2_ADR2RF       (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG2,0)))
#define ADB_REG2_OVR2         (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG2,1)))
#define ADB_REG3_ADR3RF       (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG3,0)))
#define ADB_REG3_OVR3         (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG3,1)))
#define ADB_REG4_ADR4RF       (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG4,0)))
#define ADB_REG4_OVR4         (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG4,1)))
#define ADB_REG5_ADR5RF       (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG5,0)))
#define ADB_REG5_OVR5         (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG5,1)))
#define ADB_REG6_ADR6RF       (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG6,0)))
#define ADB_REG6_OVR6         (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG6,1)))
#define ADB_REG7_ADR7RF       (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG7,0)))
#define ADB_REG7_OVR7         (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG7,1)))
#define ADB_REG8_ADR8RF       (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG8,0)))
#define ADB_REG8_OVR8         (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG8,1)))
#define ADB_REG9_ADR9RF       (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG9,0)))
#define ADB_REG9_OVR9         (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG9,1)))
#define ADB_REG10_ADR10RF     (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG10,0)))
#define ADB_REG10_OVR10       (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG10,1)))
#define ADB_REG11_ADR11RF     (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG11,0)))
#define ADB_REG11_OVR11       (*((__I  uint32_t *)BITBAND_PERI(&ADB->REG11,1)))
#define ADB_PSEL0_PENS0       (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSEL0,7)))
#define ADB_PSEL1_PENS1       (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSEL1,7)))
#define ADB_PSEL2_PENS2       (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSEL2,7)))
#define ADB_PSEL3_PENS3       (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSEL3,7)))
#define ADB_PSEL4_PENS4       (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSEL4,7)))
#define ADB_PSEL5_PENS5       (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSEL5,7)))
#define ADB_PSEL6_PENS6       (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSEL6,7)))
#define ADB_PSEL7_PENS7       (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSEL7,7)))
#define ADB_PSEL8_PENS8       (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSEL8,7)))
#define ADB_PSEL9_PENS9       (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSEL9,7)))
#define ADB_PSEL10_PENS10     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSEL10,7)))
#define ADB_PSEL11_PENS11     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSEL11,7)))
#define ADB_PINTS0_INTSEL00   (*((__IO uint32_t *)BITBAND_PERI(&ADB->PINTS0,0)))
#define ADB_PINTS0_INTSEL01   (*((__IO uint32_t *)BITBAND_PERI(&ADB->PINTS0,1)))
#define ADB_PINTS1_INTSEL10   (*((__IO uint32_t *)BITBAND_PERI(&ADB->PINTS1,0)))
#define ADB_PINTS1_INTSEL11   (*((__IO uint32_t *)BITBAND_PERI(&ADB->PINTS1,1)))
#define ADB_PINTS2_INTSEL20   (*((__IO uint32_t *)BITBAND_PERI(&ADB->PINTS2,0)))
#define ADB_PINTS2_INTSEL21   (*((__IO uint32_t *)BITBAND_PERI(&ADB->PINTS2,1)))
#define ADB_PINTS3_INTSEL30   (*((__IO uint32_t *)BITBAND_PERI(&ADB->PINTS3,0)))
#define ADB_PINTS3_INTSEL31   (*((__IO uint32_t *)BITBAND_PERI(&ADB->PINTS3,1)))
#define ADB_PINTS4_INTSEL40   (*((__IO uint32_t *)BITBAND_PERI(&ADB->PINTS4,0)))
#define ADB_PINTS4_INTSEL41   (*((__IO uint32_t *)BITBAND_PERI(&ADB->PINTS4,1)))
#define ADB_PINTS5_INTSEL50   (*((__IO uint32_t *)BITBAND_PERI(&ADB->PINTS5,0)))
#define ADB_PINTS5_INTSEL51   (*((__IO uint32_t *)BITBAND_PERI(&ADB->PINTS5,1)))
#define ADB_PSET00_ENSP00     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET00,7)))
#define ADB_PSET01_ENSP01     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET01,7)))
#define ADB_PSET02_ENSP02     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET02,7)))
#define ADB_PSET03_ENSP03     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET03,7)))
#define ADB_PSET10_ENSP10     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET10,7)))
#define ADB_PSET11_ENSP11     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET11,7)))
#define ADB_PSET12_ENSP12     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET12,7)))
#define ADB_PSET13_ENSP13     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET13,7)))
#define ADB_PSET20_ENSP20     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET20,7))) 
#define ADB_PSET21_ENSP21     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET21,7)))
#define ADB_PSET22_ENSP22     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET22,7)))
#define ADB_PSET23_ENSP23     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET23,7)))
#define ADB_PSET30_ENSP30     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET30,7))) 
#define ADB_PSET31_ENSP31     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET31,7)))
#define ADB_PSET32_ENSP32     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET32,7)))
#define ADB_PSET33_ENSP33     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET33,7)))
#define ADB_PSET40_ENSP40     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET40,7))) 
#define ADB_PSET41_ENSP41     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET41,7)))
#define ADB_PSET42_ENSP42     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET42,7)))
#define ADB_PSET43_ENSP43     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET43,7)))
#define ADB_PSET50_ENSP50     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET50,7))) 
#define ADB_PSET51_ENSP51     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET51,7)))
#define ADB_PSET52_ENSP52     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET52,7)))
#define ADB_PSET53_ENSP53     (*((__IO uint32_t *)BITBAND_PERI(&ADB->PSET53,7)))
#define ADB_TSET0_ENST0       (*((__IO uint32_t *)BITBAND_PERI(&ADB->TSET0,7)))
#define ADB_TSET1_ENST1       (*((__IO uint32_t *)BITBAND_PERI(&ADB->TSET1,7)))
#define ADB_TSET2_ENST2       (*((__IO uint32_t *)BITBAND_PERI(&ADB->TSET2,7)))
#define ADB_TSET3_ENST3       (*((__IO uint32_t *)BITBAND_PERI(&ADB->TSET3,7)))
#define ADB_TSET4_ENST4       (*((__IO uint32_t *)BITBAND_PERI(&ADB->TSET4,7)))
#define ADB_TSET5_ENST5       (*((__IO uint32_t *)BITBAND_PERI(&ADB->TSET5,7)))
#define ADB_TSET6_ENST6       (*((__IO uint32_t *)BITBAND_PERI(&ADB->TSET6,7)))
#define ADB_TSET7_ENST7       (*((__IO uint32_t *)BITBAND_PERI(&ADB->TSET7,7)))
#define ADB_TSET8_ENST8       (*((__IO uint32_t *)BITBAND_PERI(&ADB->TSET8,7)))
#define ADB_TSET9_ENST9       (*((__IO uint32_t *)BITBAND_PERI(&ADB->TSET9,7)))
#define ADB_TSET10_ENST10     (*((__IO uint32_t *)BITBAND_PERI(&ADB->TSET10,7)))
#define ADB_TSET11_ENST11     (*((__IO uint32_t *)BITBAND_PERI(&ADB->TSET11,7)))
#define ADB_SSET0_ENSS0       (*((__IO uint32_t *)BITBAND_PERI(&ADB->SSET0,7)))
#define ADB_SSET1_ENSS1       (*((__IO uint32_t *)BITBAND_PERI(&ADB->SSET1,7)))
#define ADB_SSET2_ENSS2       (*((__IO uint32_t *)BITBAND_PERI(&ADB->SSET2,7)))
#define ADB_SSET3_ENSS3       (*((__IO uint32_t *)BITBAND_PERI(&ADB->SSET3,7)))
#define ADB_SSET4_ENSS4       (*((__IO uint32_t *)BITBAND_PERI(&ADB->SSET4,7)))
#define ADB_SSET5_ENSS5       (*((__IO uint32_t *)BITBAND_PERI(&ADB->SSET5,7)))
#define ADB_SSET6_ENSS6       (*((__IO uint32_t *)BITBAND_PERI(&ADB->SSET6,7)))
#define ADB_SSET7_ENSS7       (*((__IO uint32_t *)BITBAND_PERI(&ADB->SSET7,7)))
#define ADB_SSET8_ENSS8       (*((__IO uint32_t *)BITBAND_PERI(&ADB->SSET8,7)))
#define ADB_SSET9_ENSS9       (*((__IO uint32_t *)BITBAND_PERI(&ADB->SSET9,7)))
#define ADB_SSET10_ENSS10     (*((__IO uint32_t *)BITBAND_PERI(&ADB->SSET10,7)))
#define ADB_SSET11_ENSS11     (*((__IO uint32_t *)BITBAND_PERI(&ADB->SSET11,7)))
#define ADB_ASET0_ENSA0       (*((__IO uint32_t *)BITBAND_PERI(&ADB->ASET0,7)))
#define ADB_ASET1_ENSA1       (*((__IO uint32_t *)BITBAND_PERI(&ADB->ASET1,7)))
#define ADB_ASET2_ENSA2       (*((__IO uint32_t *)BITBAND_PERI(&ADB->ASET2,7)))
#define ADB_ASET3_ENSA3       (*((__IO uint32_t *)BITBAND_PERI(&ADB->ASET3,7)))
#define ADB_ASET4_ENSA4       (*((__IO uint32_t *)BITBAND_PERI(&ADB->ASET4,7)))
#define ADB_ASET5_ENSA5       (*((__IO uint32_t *)BITBAND_PERI(&ADB->ASET5,7)))
#define ADB_ASET6_ENSA6       (*((__IO uint32_t *)BITBAND_PERI(&ADB->ASET6,7)))
#define ADB_ASET7_ENSA7       (*((__IO uint32_t *)BITBAND_PERI(&ADB->ASET7,7)))
#define ADB_ASET8_ENSA8       (*((__IO uint32_t *)BITBAND_PERI(&ADB->ASET8,7)))
#define ADB_ASET9_ENSA9       (*((__IO uint32_t *)BITBAND_PERI(&ADB->ASET9,7)))
#define ADB_ASET10_ENSA10     (*((__IO uint32_t *)BITBAND_PERI(&ADB->ASET10,7)))
#define ADB_ASET11_ENSA11     (*((__IO uint32_t *)BITBAND_PERI(&ADB->ASET11,7)))

/* Op-Amps (AMP) */
#define AMP_CTLA_AMPEN        (*((__IO uint32_t *)BITBAND_PERI(&AMP->CTLA,0)))
#define AMP_CTLB_AMPEN        (*((__IO uint32_t *)BITBAND_PERI(&AMP->CTLB,0)))
#define AMP_CTLC_AMPEN        (*((__IO uint32_t *)BITBAND_PERI(&AMP->CTLC,0)))
#define AMP_CTLD_AMPEN        (*((__IO uint32_t *)BITBAND_PERI(&AMP->CTLD,0)))

/* Analog Comparators (CMP) */
#define CMP_CTLA_CMPEN        (*((__IO uint32_t *)BITBAND_PERI(&CMP->CTLA,0)))
#define CMP_CTLA_CMPSEL       (*((__IO uint32_t *)BITBAND_PERI(&CMP->CTLA,1)))
#define CMP_CTLB_CMPEN        (*((__IO uint32_t *)BITBAND_PERI(&CMP->CTLB,0)))
#define CMP_CTLB_CMPSEL       (*((__IO uint32_t *)BITBAND_PERI(&CMP->CTLB,1)))
#define CMP_CTLC_CMPEN        (*((__IO uint32_t *)BITBAND_PERI(&CMP->CTLC,0)))
#define CMP_CTLC_CMPSEL       (*((__IO uint32_t *)BITBAND_PERI(&CMP->CTLC,1)))
#define CMP_CTLD_CMPEN        (*((__IO uint32_t *)BITBAND_PERI(&CMP->CTLD,0)))
#define CMP_CTLD_CMPSEL       (*((__IO uint32_t *)BITBAND_PERI(&CMP->CTLD,1)))

/* Watchdog Timer (WD) */
#define WD_MOD_RESCR          (*((__IO uint32_t *)BITBAND_PERI(&WD->MOD,1)))
#define WD_MOD_I2WDT          (*((__IO uint32_t *)BITBAND_PERI(&WD->MOD,2)))
#define WD_MOD_WDTE           (*((__IO uint32_t *)BITBAND_PERI(&WD->MOD,7)))

/* Clock Generator (CG) */
#define CG_SYSCR_FPSEL        (*((__IO uint32_t *)BITBAND_PERI(&CG->SYSCR,12)))
#define CG_OSCCR_WUEON        (*((__O  uint32_t *)BITBAND_PERI(&CG->OSCCR,0)))
#define CG_OSCCR_WUEF         (*((__I  uint32_t *)BITBAND_PERI(&CG->OSCCR,1)))
#define CG_OSCCR_PLLON        (*((__IO uint32_t *)BITBAND_PERI(&CG->OSCCR,2)))
#define CG_OSCCR_WUPSEL       (*((__IO uint32_t *)BITBAND_PERI(&CG->OSCCR,3)))
#define CG_OSCCR_XEN          (*((__IO uint32_t *)BITBAND_PERI(&CG->OSCCR,8)))
#define CG_OSCCR_OSCSEL       (*((__IO uint32_t *)BITBAND_PERI(&CG->OSCCR,17)))



#define CG_OSCCR_DRVOSCH      (*((__IO uint32_t *)BITBAND_PERI(&CG->OSCCR,12)))
#define CG_STBYCR_RXEN        (*((__IO uint32_t *)BITBAND_PERI(&CG->STBYCR,8)))
#define CG_STBYCR_DRVE        (*((__IO uint32_t *)BITBAND_PERI(&CG->STBYCR,16)))
#define CG_PLLSEL_PLLSEL      (*((__IO uint32_t *)BITBAND_PERI(&CG->PLLSEL,0)))
#define CG_CKSEL_SYSCKFLG     (*((__I  uint32_t *)BITBAND_PERI(&CG->CKSEL,0)))
#define CG_CKSEL_SYSCK        (*((__IO uint32_t *)BITBAND_PERI(&CG->CKSEL,1)))
#define CG_RSTFLG_PONRSTF     (*((__IO uint32_t *)BITBAND_PERI(&CG->RSTFLG,0)))
#define CG_RSTFLG_PINRSTF     (*((__IO uint32_t *)BITBAND_PERI(&CG->RSTFLG,1)))
#define CG_RSTFLG_WDTRSTF     (*((__IO uint32_t *)BITBAND_PERI(&CG->RSTFLG,2)))
#define CG_RSTFLG_DBGRSTF     (*((__IO uint32_t *)BITBAND_PERI(&CG->RSTFLG,4)))
#define CG_RSTFLG_OFDRSTF     (*((__IO uint32_t *)BITBAND_PERI(&CG->RSTFLG,5)))
#define CG_IMCGA_INT0EN       (*((__IO uint32_t *)BITBAND_PERI(&CG->IMCGA,0)))
#define CG_IMCGA_INT0F        (*((__I  uint32_t *)BITBAND_PERI(&CG->IMCGA,1)))
#define CG_IMCGA_INT1EN       (*((__IO uint32_t *)BITBAND_PERI(&CG->IMCGA,8)))
#define CG_IMCGA_INT1F        (*((__I  uint32_t *)BITBAND_PERI(&CG->IMCGA,9)))
#define CG_IMCGA_INT2EN       (*((__IO uint32_t *)BITBAND_PERI(&CG->IMCGA,16)))
#define CG_IMCGA_INT2F        (*((__I  uint32_t *)BITBAND_PERI(&CG->IMCGA,17)))
#define CG_IMCGA_INT3EN       (*((__IO uint32_t *)BITBAND_PERI(&CG->IMCGA,24)))
#define CG_IMCGA_INT3F        (*((__I  uint32_t *)BITBAND_PERI(&CG->IMCGA,25)))
#define CG_IMCGB_INT4EN       (*((__IO uint32_t *)BITBAND_PERI(&CG->IMCGB,0)))
#define CG_IMCGB_INT4F        (*((__I  uint32_t *)BITBAND_PERI(&CG->IMCGB,1)))
#define CG_IMCGB_INT5EN       (*((__IO uint32_t *)BITBAND_PERI(&CG->IMCGB,8)))
#define CG_IMCGB_INT5F        (*((__I  uint32_t *)BITBAND_PERI(&CG->IMCGB,9)))
#define CG_IMCGB_INT6EN       (*((__IO uint32_t *)BITBAND_PERI(&CG->IMCGB,16)))
#define CG_IMCGB_INT6F        (*((__I  uint32_t *)BITBAND_PERI(&CG->IMCGB,17)))
#define CG_IMCGB_INT7EN       (*((__IO uint32_t *)BITBAND_PERI(&CG->IMCGB,24)))
#define CG_IMCGB_INT7F        (*((__I  uint32_t *)BITBAND_PERI(&CG->IMCGB,25)))
#define CG_IMCGC_INT8EN       (*((__IO uint32_t *)BITBAND_PERI(&CG->IMCGC,0)))
#define CG_IMCGC_INT8F        (*((__I  uint32_t *)BITBAND_PERI(&CG->IMCGC,1)))
#define CG_IMCGC_INT9EN       (*((__IO uint32_t *)BITBAND_PERI(&CG->IMCGC,8)))
#define CG_IMCGC_INT9F        (*((__I  uint32_t *)BITBAND_PERI(&CG->IMCGC,9)))
#define CG_IMCGC_INTAEN       (*((__IO uint32_t *)BITBAND_PERI(&CG->IMCGC,16)))
#define CG_IMCGC_INTAF        (*((__I  uint32_t *)BITBAND_PERI(&CG->IMCGC,17)))
#define CG_IMCGC_INTBEN       (*((__IO uint32_t *)BITBAND_PERI(&CG->IMCGC,24)))
#define CG_IMCGC_INTBF        (*((__I  uint32_t *)BITBAND_PERI(&CG->IMCGC,25)))
#define CG_IMCGD_INTCEN       (*((__IO uint32_t *)BITBAND_PERI(&CG->IMCGD,0)))
#define CG_IMCGD_INTCF        (*((__I  uint32_t *)BITBAND_PERI(&CG->IMCGD,1)))
#define CG_IMCGD_INTDEN       (*((__IO uint32_t *)BITBAND_PERI(&CG->IMCGD,8)))
#define CG_IMCGD_INTDF        (*((__I  uint32_t *)BITBAND_PERI(&CG->IMCGD,9)))
#define CG_IMCGD_INTEEN       (*((__IO uint32_t *)BITBAND_PERI(&CG->IMCGD,16)))
#define CG_IMCGD_INTEF        (*((__I  uint32_t *)BITBAND_PERI(&CG->IMCGD,17)))
#define CG_IMCGD_INTFEN       (*((__IO uint32_t *)BITBAND_PERI(&CG->IMCGD,24)))
#define CG_IMCGD_INTFF        (*((__I  uint32_t *)BITBAND_PERI(&CG->IMCGD,25)))

/* Oscillation Frequency Detector (OFD) */
/* Nothing */

/* Power On Reset (POR), Voltage Detecting circuit (VLTD) */
#define VD_CR_VDEN            (*((__IO uint32_t *)BITBAND_PERI(&VD->CR,0)))
#define VD_SR_VDSR            (*((__I  uint32_t *)BITBAND_PERI(&VD->SR,0)))

/* Vector Engine (VE) */
#define VE_EN_VEEN            (*((__IO uint32_t *)BITBAND_PERI(&VE->EN,0)))
#define VE_EN_VEIDLEN         (*((__IO uint32_t *)BITBAND_PERI(&VE->EN,1)))
#define VE_CPURUNTRG_VCPURTA  (*((__O  uint32_t *)BITBAND_PERI(&VE->CPURUNTRG,0)))
#define VE_CPURUNTRG_VCPURTB  (*((__O  uint32_t *)BITBAND_PERI(&VE->CPURUNTRG,1)))
#define VE_TRGMODE_VTRGA0     (*((__IO uint32_t *)BITBAND_PERI(&VE->TRGMODE,0)))
#define VE_TRGMODE_VTRGA1     (*((__IO uint32_t *)BITBAND_PERI(&VE->TRGMODE,1)))
#define VE_TRGMODE_VTRGB0     (*((__IO uint32_t *)BITBAND_PERI(&VE->TRGMODE,2)))
#define VE_TRGMODE_VTRGB1     (*((__IO uint32_t *)BITBAND_PERI(&VE->TRGMODE,3)))
#define VE_ERRINTEN_VERRENA   (*((__IO uint32_t *)BITBAND_PERI(&VE->ERRINTEN,0)))
#define VE_ERRINTEN_VERRENB   (*((__IO uint32_t *)BITBAND_PERI(&VE->ERRINTEN,1)))
#define VE_COMPEND_VCENDA     (*((__O  uint32_t *)BITBAND_PERI(&VE->COMPEND,0)))
#define VE_COMPEND_VCENDB     (*((__O  uint32_t *)BITBAND_PERI(&VE->COMPEND,1)))
#define VE_ERRDET_VERRDA      (*((__I  uint32_t *)BITBAND_PERI(&VE->ERRDET,0)))
#define VE_ERRDET_VERRDB      (*((__I  uint32_t *)BITBAND_PERI(&VE->ERRDET,1)))
#define VE_SCHTASKRUN_VRSCHA  (*((__I  uint32_t *)BITBAND_PERI(&VE->SCHTASKRUN,0)))
#define VE_SCHTASKRUN_VRSCHB  (*((__I  uint32_t *)BITBAND_PERI(&VE->SCHTASKRUN,5)))
#define VE_MODE0_PVIEN        (*((__IO uint32_t *)BITBAND_PERI(&VE->MODE0,0)))
#define VE_MODE0_ZIEN         (*((__IO uint32_t *)BITBAND_PERI(&VE->MODE0,1)))
#define VE_MODE1_PVIEN        (*((__IO uint32_t *)BITBAND_PERI(&VE->MODE1,0)))
#define VE_MODE1_ZIEN         (*((__IO uint32_t *)BITBAND_PERI(&VE->MODE1,1)))
#define VE_FMODE0_C2PEN       (*((__IO uint32_t *)BITBAND_PERI(&VE->FMODE0,0)))
#define VE_FMODE0_SPWMEN      (*((__IO uint32_t *)BITBAND_PERI(&VE->FMODE0,1)))
#define VE_FMODE0_PMDSEL      (*((__IO uint32_t *)BITBAND_PERI(&VE->FMODE0,4)))
#define VE_FMODE0_MREGDIS     (*((__IO uint32_t *)BITBAND_PERI(&VE->FMODE0,9)))
#define VE_FMODE1_C2PEN       (*((__IO uint32_t *)BITBAND_PERI(&VE->FMODE1,0)))
#define VE_FMODE1_SPWMEN      (*((__IO uint32_t *)BITBAND_PERI(&VE->FMODE1,1)))
#define VE_FMODE1_PMDSEL      (*((__IO uint32_t *)BITBAND_PERI(&VE->FMODE1,4)))
#define VE_FMODE1_MREGDIS     (*((__IO uint32_t *)BITBAND_PERI(&VE->FMODE1,9)))
#define VE_MCTLF0_LAVF        (*((__IO uint32_t *)BITBAND_PERI(&VE->MCTLF0,0)))
#define VE_MCTLF0_LAVFM       (*((__IO uint32_t *)BITBAND_PERI(&VE->MCTLF0,1)))
#define VE_MCTLF0_LVTF        (*((__IO uint32_t *)BITBAND_PERI(&VE->MCTLF0,2)))
#define VE_MCTLF0_PLSLF       (*((__IO uint32_t *)BITBAND_PERI(&VE->MCTLF0,4)))
#define VE_MCTLF0_PLSLFM      (*((__IO uint32_t *)BITBAND_PERI(&VE->MCTLF0,5)))
#define VE_MCTLF1_LAVF        (*((__IO uint32_t *)BITBAND_PERI(&VE->MCTLF1,0)))
#define VE_MCTLF1_LAVFM       (*((__IO uint32_t *)BITBAND_PERI(&VE->MCTLF1,1)))
#define VE_MCTLF1_LVTF        (*((__IO uint32_t *)BITBAND_PERI(&VE->MCTLF1,2)))
#define VE_MCTLF1_PLSLF       (*((__IO uint32_t *)BITBAND_PERI(&VE->MCTLF1,4)))
#define VE_MCTLF1_PLSLFM      (*((__IO uint32_t *)BITBAND_PERI(&VE->MCTLF1,5)))
#define VE_OUTCR0_UOCU        (*((__IO uint32_t *)BITBAND_PERI(&VE->OUTCR0,0)))
#define VE_OUTCR0_UOCX        (*((__IO uint32_t *)BITBAND_PERI(&VE->OUTCR0,1)))
#define VE_OUTCR0_VOCV        (*((__IO uint32_t *)BITBAND_PERI(&VE->OUTCR0,2)))
#define VE_OUTCR0_VOCY        (*((__IO uint32_t *)BITBAND_PERI(&VE->OUTCR0,3)))
#define VE_OUTCR0_WOCW        (*((__IO uint32_t *)BITBAND_PERI(&VE->OUTCR0,4)))
#define VE_OUTCR0_WOCZ        (*((__IO uint32_t *)BITBAND_PERI(&VE->OUTCR0,5)))
#define VE_OUTCR0_UPWM        (*((__IO uint32_t *)BITBAND_PERI(&VE->OUTCR0,6)))
#define VE_OUTCR0_VPWM        (*((__IO uint32_t *)BITBAND_PERI(&VE->OUTCR0,7)))
#define VE_OUTCR0_WPWM        (*((__IO uint32_t *)BITBAND_PERI(&VE->OUTCR0,8)))
#define VE_OUTCR1_UOCU        (*((__IO uint32_t *)BITBAND_PERI(&VE->OUTCR1,0)))
#define VE_OUTCR1_UOCX        (*((__IO uint32_t *)BITBAND_PERI(&VE->OUTCR1,1)))
#define VE_OUTCR1_VOCV        (*((__IO uint32_t *)BITBAND_PERI(&VE->OUTCR1,2)))
#define VE_OUTCR1_VOCY        (*((__IO uint32_t *)BITBAND_PERI(&VE->OUTCR1,3)))
#define VE_OUTCR1_WOCW        (*((__IO uint32_t *)BITBAND_PERI(&VE->OUTCR1,4)))
#define VE_OUTCR1_WOCZ        (*((__IO uint32_t *)BITBAND_PERI(&VE->OUTCR1,5)))
#define VE_OUTCR1_UPWM        (*((__IO uint32_t *)BITBAND_PERI(&VE->OUTCR1,6)))
#define VE_OUTCR1_VPWM        (*((__IO uint32_t *)BITBAND_PERI(&VE->OUTCR1,7)))
#define VE_OUTCR1_WPWM        (*((__IO uint32_t *)BITBAND_PERI(&VE->OUTCR1,8)))
#define VE_EMGRS0_EMGRS       (*((__IO uint32_t *)BITBAND_PERI(&VE->EMGRS0,0)))
#define VE_EMGRS1_EMGRS       (*((__IO uint32_t *)BITBAND_PERI(&VE->EMGRS1,0)))

/* Programmable Motor Driver (PMD) */
#define PMD0_MDEN_PWMEN       (*((__IO uint32_t *)BITBAND_PERI(&PMD0->MDEN,0)))
#define PMD0_PORTMD_PORTMDL   (*((__IO uint32_t *)BITBAND_PERI(&PMD0->PORTMD,0)))
#define PMD0_PORTMD_PORTMDH   (*((__IO uint32_t *)BITBAND_PERI(&PMD0->PORTMD,1)))
#define PMD0_MODESEL_MDSEL    (*((__IO uint32_t *)BITBAND_PERI(&PMD0->MODESEL,0)))
#define PMD0_MDCR_PWMMD       (*((__IO uint32_t *)BITBAND_PERI(&PMD0->MDCR,0)))
#define PMD0_MDCR_PINT        (*((__IO uint32_t *)BITBAND_PERI(&PMD0->MDCR,3)))
#define PMD0_MDCR_DTYMD       (*((__IO uint32_t *)BITBAND_PERI(&PMD0->MDCR,4)))
#define PMD0_MDCR_SYNTMD      (*((__IO uint32_t *)BITBAND_PERI(&PMD0->MDCR,5)))
#define PMD0_MDCR_PWMCK       (*((__IO uint32_t *)BITBAND_PERI(&PMD0->MDCR,6)))
#define PMD0_CNTSTA_UPDWN     (*((__I  uint32_t *)BITBAND_PERI(&PMD0->CNTSTA,0)))
#define PMD0_MDPOT_POLL       (*((__IO uint32_t *)BITBAND_PERI(&PMD0->MDPOT,2)))
#define PMD0_MDPOT_POLH       (*((__IO uint32_t *)BITBAND_PERI(&PMD0->MDPOT,3)))
#define PMD0_MDOUT_UPWM       (*((__IO uint32_t *)BITBAND_PERI(&PMD0->MDOUT,8)))
#define PMD0_MDOUT_VPWM       (*((__IO uint32_t *)BITBAND_PERI(&PMD0->MDOUT,9)))
#define PMD0_MDOUT_WPWM       (*((__IO uint32_t *)BITBAND_PERI(&PMD0->MDOUT,10)))
#define PMD0_EMGCR_EMGEN      (*((__IO uint32_t *)BITBAND_PERI(&PMD0->EMGCR,0)))
#define PMD0_EMGCR_EMGRS      (*((__O  uint32_t *)BITBAND_PERI(&PMD0->EMGCR,1)))
#define PMD0_EMGCR_EMGISEL    (*((__IO uint32_t *)BITBAND_PERI(&PMD0->EMGCR,2)))
#define PMD0_EMGCR_INHEN      (*((__IO uint32_t *)BITBAND_PERI(&PMD0->EMGCR,5)))
#define PMD0_EMGSTA_EMGST     (*((__I  uint32_t *)BITBAND_PERI(&PMD0->EMGSTA,0)))
#define PMD0_EMGSTA_EMGI      (*((__I  uint32_t *)BITBAND_PERI(&PMD0->EMGSTA,1)))
#define PMD0_OVVCR_OVVEN      (*((__IO uint32_t *)BITBAND_PERI(&PMD0->OVVCR,0)))
#define PMD0_OVVCR_OVVRS      (*((__IO uint32_t *)BITBAND_PERI(&PMD0->OVVCR,1)))
#define PMD0_OVVCR_OVVISEL    (*((__IO uint32_t *)BITBAND_PERI(&PMD0->OVVCR,2)))
#define PMD0_OVVCR_ADIN0EN    (*((__IO uint32_t *)BITBAND_PERI(&PMD0->OVVCR,5)))
#define PMD0_OVVCR_ADIN1EN    (*((__IO uint32_t *)BITBAND_PERI(&PMD0->OVVCR,6)))
#define PMD0_OVVSTA_OVVST     (*((__I  uint32_t *)BITBAND_PERI(&PMD0->OVVSTA,0)))
#define PMD0_OVVSTA_OVVI      (*((__I  uint32_t *)BITBAND_PERI(&PMD0->OVVSTA,1)))
#define PMD0_TRGCR_TRG0BE     (*((__IO uint32_t *)BITBAND_PERI(&PMD0->TRGCR,3)))
#define PMD0_TRGCR_TRG1BE     (*((__IO uint32_t *)BITBAND_PERI(&PMD0->TRGCR,7)))
#define PMD0_TRGCR_TRG2BE     (*((__IO uint32_t *)BITBAND_PERI(&PMD0->TRGCR,11)))
#define PMD0_TRGCR_TRG3BE     (*((__IO uint32_t *)BITBAND_PERI(&PMD0->TRGCR,15)))
#define PMD0_TRGMD_EMGTGE     (*((__IO uint32_t *)BITBAND_PERI(&PMD0->TRGMD,0)))
#define PMD0_TRGMD_TRGOUT     (*((__IO uint32_t *)BITBAND_PERI(&PMD0->TRGMD,1)))

#define PMD1_MDEN_PWMEN       (*((__IO uint32_t *)BITBAND_PERI(&PMD1->MDEN,0)))
#define PMD1_PORTMD_PORTMDL   (*((__IO uint32_t *)BITBAND_PERI(&PMD1->PORTMD,0)))
#define PMD1_PORTMD_PORTMDH   (*((__IO uint32_t *)BITBAND_PERI(&PMD1->PORTMD,1)))
#define PMD1_MODESEL_MDSEL    (*((__IO uint32_t *)BITBAND_PERI(&PMD1->MODESEL,0)))
#define PMD1_MDCR_PWMMD       (*((__IO uint32_t *)BITBAND_PERI(&PMD1->MDCR,0)))
#define PMD1_MDCR_PINT        (*((__IO uint32_t *)BITBAND_PERI(&PMD1->MDCR,3)))
#define PMD1_MDCR_DTYMD       (*((__IO uint32_t *)BITBAND_PERI(&PMD1->MDCR,4)))
#define PMD1_MDCR_SYNTMD      (*((__IO uint32_t *)BITBAND_PERI(&PMD1->MDCR,5)))
#define PMD1_MDCR_PWMCK       (*((__IO uint32_t *)BITBAND_PERI(&PMD1->MDCR,6)))
#define PMD1_CNTSTA_UPDWN     (*((__I  uint32_t *)BITBAND_PERI(&PMD1->CNTSTA,0)))
#define PMD1_MDPOT_POLL       (*((__IO uint32_t *)BITBAND_PERI(&PMD1->MDPOT,2)))
#define PMD1_MDPOT_POLH       (*((__IO uint32_t *)BITBAND_PERI(&PMD1->MDPOT,3)))
#define PMD1_MDOUT_UPWM       (*((__IO uint32_t *)BITBAND_PERI(&PMD1->MDOUT,8)))
#define PMD1_MDOUT_VPWM       (*((__IO uint32_t *)BITBAND_PERI(&PMD1->MDOUT,9)))
#define PMD1_MDOUT_WPWM       (*((__IO uint32_t *)BITBAND_PERI(&PMD1->MDOUT,10)))
#define PMD1_EMGCR_EMGEN      (*((__IO uint32_t *)BITBAND_PERI(&PMD1->EMGCR,0)))
#define PMD1_EMGCR_EMGRS      (*((__O  uint32_t *)BITBAND_PERI(&PMD1->EMGCR,1)))
#define PMD1_EMGCR_EMGISEL    (*((__IO uint32_t *)BITBAND_PERI(&PMD1->EMGCR,2)))
#define PMD1_EMGCR_INHEN      (*((__IO uint32_t *)BITBAND_PERI(&PMD1->EMGCR,5)))
#define PMD1_EMGSTA_EMGST     (*((__I  uint32_t *)BITBAND_PERI(&PMD1->EMGSTA,0)))
#define PMD1_EMGSTA_EMGI      (*((__I  uint32_t *)BITBAND_PERI(&PMD1->EMGSTA,1)))
#define PMD1_OVVCR_OVVEN      (*((__IO uint32_t *)BITBAND_PERI(&PMD1->OVVCR,0)))
#define PMD1_OVVCR_OVVRS      (*((__IO uint32_t *)BITBAND_PERI(&PMD1->OVVCR,1)))
#define PMD1_OVVCR_OVVISEL    (*((__IO uint32_t *)BITBAND_PERI(&PMD1->OVVCR,2)))
#define PMD1_OVVCR_ADIN0EN    (*((__IO uint32_t *)BITBAND_PERI(&PMD1->OVVCR,5)))
#define PMD1_OVVCR_ADIN1EN    (*((__IO uint32_t *)BITBAND_PERI(&PMD1->OVVCR,6)))
#define PMD1_OVVSTA_OVVST     (*((__I  uint32_t *)BITBAND_PERI(&PMD1->OVVSTA,0)))
#define PMD1_OVVSTA_OVVI      (*((__I  uint32_t *)BITBAND_PERI(&PMD1->OVVSTA,1)))
#define PMD1_TRGCR_TRG0BE     (*((__IO uint32_t *)BITBAND_PERI(&PMD1->TRGCR,3)))
#define PMD1_TRGCR_TRG1BE     (*((__IO uint32_t *)BITBAND_PERI(&PMD1->TRGCR,7)))
#define PMD1_TRGCR_TRG2BE     (*((__IO uint32_t *)BITBAND_PERI(&PMD1->TRGCR,11)))
#define PMD1_TRGCR_TRG3BE     (*((__IO uint32_t *)BITBAND_PERI(&PMD1->TRGCR,15)))
#define PMD1_TRGMD_EMGTGE     (*((__IO uint32_t *)BITBAND_PERI(&PMD1->TRGMD,0)))
#define PMD1_TRGMD_TRGOUT     (*((__IO uint32_t *)BITBAND_PERI(&PMD1->TRGMD,1)))

/* Flash Control (FC) */
/* nothing */

/** @} */ /* End of group Device_Peripheral_registers */

#ifdef __cplusplus
}
#endif

#endif  /* __TMPM370_H__ */

/** @} */ /* End of group TMPM370 */
/** @} */ /* End of group TOSHIBA_TX03_MICROCONTROLLER */
